#include <zaf.hpp>
#include <cfz_zcht.h>

const int NUM_POINTS = 10;

double	SET1[ NUM_POINTS ] = { 20, 10, 20, 10, 20, 10, 20, 10, 20, 10 };
double	SET2[ NUM_POINTS ] = { 20, 18, 14,  2, 14, 37, 35, 30, 24, 23 };
double	SET3[ NUM_POINTS ] = { 33, 30, 27, 24, 21, 18, 15, 12,  9,  6 };
double	SET4[ NUM_POINTS ] = { 20, 15, 10, 12, 14, 20, 24, 28, 30, 32 };
double	SET5[ NUM_POINTS ] = { 16, 15, 23, 17, 24, 20, 18, 13, 28, 16 };


class ChartWindow
		: public ZafWindow
{
  public:
    ChartWindow();
    void    AddChartData( cf_ChartModel *p_cmTarget );
    void    AddAreaChart();
    void    AddBarChart();
    void    AddLineChart();
    void    AddStackChart();
};


ChartWindow ::ChartWindow()
		    : ZafWindow( 10, 2, 65, 15 )
{
    // Make a typical Zinc ZAF v5.x Window:
	SetStringID( "CFWindow" );
	Add( new ZafBorder );
	Add( new ZafMaximizeButton );
	Add( new ZafMinimizeButton );
	Add( new ZafSystemButton );
	Add( new ZafTitle
             ( new ZafStringData ( "ChartFolio Charts for Zinc ZAF v5.x" )));

    // Now let's add some ZafChart objects to our ZafWindow:
    AddLineChart();
    AddBarChart();
    AddAreaChart();
    AddStackChart();
}

void ChartWindow::AddChartData( cf_ChartModel *p_cmTarget )
{
	int				i,j;
    double          *p_dTargetArray = &SET1[ 0 ];

    for ( i = 1; i <= 5; i++ )
    {
        // Iterate through the target data array/pointer addresses:
        if ( i == 2 ) p_dTargetArray = &SET2[ 0 ];
        if ( i == 3 ) p_dTargetArray = &SET3[ 0 ];
        if ( i == 4 ) p_dTargetArray = &SET4[ 0 ];
        if ( i == 5 ) p_dTargetArray = &SET5[ 0 ];

        // Activate the appropriate data group:
	    p_cmTarget->activateGroup( i );

        // Add data points into the active data group:
    	for ( j = 0; j < NUM_POINTS; j++ )
        {
		    p_cmTarget->addDataYVal( *p_dTargetArray );
            p_dTargetArray++;
        }
    }
}

void ChartWindow::AddAreaChart()
{
    // Create the chart-window-object:
	ZafChart    *p_zcUserChart
                = new ZafChart( 1, 7, 30, 7, // placement & size
                                CF_ID_AREA_2D, CF_ID_YVALS_AT_INT_IS );

    // Now stuff data into the ZafChart's chart-model:
	AddChartData( p_zcUserChart->ActiveModel() );

    // Add the chart-window-object into the parent window
    Add( p_zcUserChart );
}

void ChartWindow::AddBarChart()
// Let's permute the operational sequence immediately above:
{
    // Create the chart-window-object:
	ZafChart    *p_zcUserChart
                = new ZafChart( 33, 0, 30, 7, // placement & size
                                CF_ID_BAR_2D, CF_ID_YVALS_AT_INT_IS );

    // Add the chart-window-object into the parent window
    Add( p_zcUserChart );

    // Now stuff data into the ZafChart's chart-model:
	AddChartData( p_zcUserChart->ActiveModel() );
}


void ChartWindow::AddLineChart()
// Let's try ANOTHER permutation of the operational sequence(s) above:
{
    // Create the chart-window-object:
	ZafChart  *p_zcUserChart;  // keeping a pointer to this new object...
    Add( p_zcUserChart = new ZafChart(  1, 0, 30, 7, // placement & size
                                        CF_ID_LINE_2D, CF_ID_YVALS_ON_NAT_NS ));
    // Now stuff data into the ZafChart's chart-model:
	AddChartData( p_zcUserChart->ActiveModel() );
}

void ChartWindow::AddStackChart()
// Yet ANOTHER permutation of the operational sequence(s) above:
{
    // Create the chart-window-object -- In "ZAF_MINICELL" Coordinates:
	ZafChart    *p_zcUserChart
                = new ZafChart( 330, 70, 300, 70, // placement & size
                                CF_ID_STACK_2D, CF_ID_YVALS_ON_NAT_NS );
    p_zcUserChart->SetCoordinateType( ZAF_MINICELL );
    Add( p_zcUserChart );

    // Now stuff data into the ZafChart's chart-model:
	AddChartData( p_zcUserChart->ActiveModel() );
}


ZafApplication::Main()
{
	zafWindowManager->Add( new ChartWindow );
    Control();
	return( 0 );
}



