///////////////////////////////////////
// START of File: CFZ_ZCHT.H
///////////////////////////////////////

#ifndef CFZ_ZCHT_H            // Avoid multiple #includes...
#define CFZ_ZCHT_H    TRUE    // ...holds to a terminal #endif below...

//================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved.
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of
// copyright and license specifications, and contact information.
//
//================================================================

//-------------------------------------------------------------------
//
// "cfz_zcht.h" -- Product-Level Master Header Include File:
//
// USAGE:
//
// (1)  Define a global include path via whatever method your
//      IDE/Compiler provides to: "~/cf_zaf/include", where "~"
//		  is the installed directory tree root of this product.
//
//  (2) Once the global include path to ~/cf_zaf/include is defined,
//		  your application can simply perform includes as follows:
//
//          #include <zaf.hpp> 		// To use Zinc's ZAF v5.x...
//          #include <cfz_zcht.h>	// To use ChartFolio/ZafChart...
//
//-------------------------------------------------------------------

#define     CF_ID_PRODUCT_NAME  CFZ_ZCHT
#define     CF_ID_PRODUCT_SKU   CZCHT___
#define     CF_ID_PRODUCT_VER   1
#define     CF_ID_PRODUCT_REV   01
#define     CF_ID_PRODUCT_DATE  970519

//---------------------------------------------
// "Resource-ID" Type Files:
//---------------------------------------------
#include    <cf_api/id_chart.h> // Chart API Type IDs
#include    <cf_api/api_cmds.h> // Chart API Command IDs

//---------------------------------------------
// Chart Framework Definitions and Constants:
//---------------------------------------------
#include    <cf_api/bf_typed.h>
            // Pull-in BASE Framework's assumed typedefs...

#include    <cf_api/cf_typed.h>
            // Chart Framework typedefs

#include    <cf_api/cf_const.h>
            // Chart Framework Math/Geometry constants

//--------------------------------------
// Chart Framework Data Infrastructure:
//--------------------------------------
#include    <cf_api/c0_dystr.hpp>
            // Dynamic String object.

#include    <cf_api/c0_stack.hpp>
            // "Generic (void pointer) Push/Pop Stack" object.
            // MAY be subclassed for specific instances of explicitly
            // typed stacked objects throughout the framework...

#include    <cf_api/c0_dyarr.hpp>
            // "Generic (void pointer) Dynamic Array" object.
            // MAY be subclassed for specific instances of explicitly
            // typed dynamic array objects throughout the framework...

//------------------------------------------------------
// "Level Zero" -- Base Framework Mapping Layer Files:
//------------------------------------------------------
#include    <cf_api/b0_color.hpp>
            // Abstract Color object mappings onto base framework.

#include    <cf_api/b0_dspla.hpp>
            // Mapping(s) onto base framework "output device
            // display" objects and ops.
            //
            // Objects include "Pens", "Brushes", "PenStyles",
            // "BrushHatches", "Device Dimension Mapping Modes",
            // "Binary Draw Ops", "Display Logical Font Dimensions",
            // "Bitmaps", "Bitmap Displays", "Fonts", "Font Info",
            // etc.
            //
            // Operations include "push-Pen", "pop-Pen", (or
            // brushs, etc), point/poly-line/polygon draw/fill
            // ops, device dimansion measurement ops, etc.

#include    <cf_api/b0_dsbmp.hpp>
            // Mapping onto base framework "bitmaps" and "bitmap displays".
            // These objects are used during display output ops.
            // NOTE: Dependencies between this file and b0_dspla.hpp!

#include    <cf_api/b0_fonts.hpp>
            // Base Framework abstractions of "font-specifier",
            // "display-font", and "display-font-info" objects.
            // These objects are used during display output ops.
            // NOTE: Dependencies between this file and b0_dspla.hpp!

//---------------------------------------------
// Chart "Object Property/Attribute" objects:
//---------------------------------------------
#include    <cf_api/c0_dimen.hpp>
            // Abstract "Device 2D Point" and "Device 2D Rectangle"
            // objects typically used as params for display ops.

#include    <cf_api/c0_datb.hpp>
            // "Display Attribute Set Collection" object.

#include    <cf_api/c0_tatb.hpp>
            // "Text Attribute Set/Collection"
            // (w/ Auto-Scaling Font Spec) object.

//---------------------------------------
// Chart "User Data Management" Objects:
//---------------------------------------
#include    <cf_api/c1_xyz.hpp>
            // Chart system "Data XYZ Point" object.

#include    <cf_api/c1_dgrou.hpp>
            // Dynamic arrays of XYZ Data points with
            // associated chart/display attributes.

//---------------------------------------------------
// Chart 2D/3D "Virtual Geometry" Objects
// WITHOUT Hidden Surface Elimination/Depth-Sorting:
//---------------------------------------------------
#include    <cf_api/c1_kernl.hpp>
            // Chart "2D/3D Geometry Kernel State Machine" object.

#include    <cf_api/c1_vobj.hpp>
            // Base class -- Virtual 3D Object.

#include    <cf_api/co1_poin.hpp>
            // 3D Point Object.

#include    <cf_api/co1_pser.hpp>
            // 3D Point Series Object.  Holds 3D Points in
            // a "letter/envelope" relationship.

#include    <cf_api/co1_plin.hpp>
            // 3D PolyLine.  Derived from 3D Point Series.

#include    <cf_api/co1_pgon.hpp>
            // 3D Polygon.  Derived from 3D PolyLine.

#include    <cf_api/co1_tmar.hpp>
            // 3D Text Marker.  Derived from 3D PolyLine.

#include    <cf_api/co1_tlin.hpp>
            // 3D "Rotated Font Text Line".
            // Derived from 3D PolyLine.

#include    <cf_api/co1_gts.hpp>
            // "Chart/Engine-Axis Grid/Tick Series" objects. Holds
            // 3D Text markers in a envelope/letter relationship.

#include    <cf_api/co1_tms.hpp>
            // "3D Text Marker Series".  Holds 3D Text markers
            // in a envelope/letter relationship.
            // Also has "TitleMarkerManager" object derived from
            // "3D TextMarkerSeries

//-------------------------------------------
// "Main Chart Component/SubSystem" Objects:
//-------------------------------------------
#include    <cf_api/c1_engin.hpp>
            // Chart Engine Object.

#include    <cf_api/c1_cmodl.hpp>
            // "Chart (Data) Model" object in an M/V/C Hierarchy.

#include    <cf_api/b1_cview.hpp>
            // Base Framework Mapping of a "Chart View" object
            // in an M/V/C Hierarchy.

#include    <cf_api/bz_zcht.hpp>
            // Zinc ZAF v5.x base framework "Chart Window Object" object.

//-------------------------------------------
// "Main-Line DERIVED Chart Engine" Objects:
//-------------------------------------------
#include    <cf_api/ce1_pcor.hpp>
            // XYZ-to-Polar Coordinate Engine

#include    <cf_api/ce1_ccor.hpp>
            // Normal XYZ/Cartesian Engine.

//-----------------------------------------------
// 2D (and some 2D/3D) Charting Engine" objects:
//-----------------------------------------------
#include    <cf_api/ce2_pie.hpp>
            // "2D Pie" Chart Engine object(s)
            // derived from cf_engPolar.

#include    <cf_api/ce2_bar.hpp>
            // "2D Bar(s)" Chart Engine object(s)
            // derived from cf_engCartXYZ.

#include    <cf_api/ce2_colm.hpp>
            // "2D Column(s)" Chart Engine object(s)
            // derived from cf_engCartXYZ.

#include    <cf_api/ce2_stac.hpp>
            // "2D Stack(s)" Chart Engine object(s)
            // derived from cf_engCartXYZ.

#include    <cf_api/ce2_pser.hpp>
            // 2D display of 2D/3D Point Series Chart Engine object
            // derived from cf_engCartXYZ.

#include    <cf_api/ce2_line.hpp>
            // 2D display of 2D/3D Line Chart Engine object
            // derived from cf_engCartXYZ.

#include    <cf_api/ce2_area.hpp>
            // 2D display of 2D/3D Area Chart Engine object
            // derived from cf_engCartXYZ.

//-------------------------------------------------------
// Chart 2D/3D "Virtual Geometry" Objects WITH
// Hidden Surface Elimination/Depth-Sorting:
// True 3D Objects use HSE (Hidden Surface Elimination)
// and DSORTing (Depth Sorting).  These are objects that
// would NOT be used by a strictly 2D charting system...
//
// These objects are include in the Standard 2D/3D API.
// See DPC Technology (info@dpc-tech.com) for details.
//
//-------------------------------------------------------
#include    <cf_api/co1_cbmp.hpp>
            // Chart Framework "3D (Projected) Bitmap" object.

#endif      // Terminates #include re-definition protection

///////////////////////////////////////
// END of File: CFZ_ZCHT.HPP
///////////////////////////////////////


