//////////////////////////////////////
// START of File: B0_DSPLA.HPP
//////////////////////////////////////

#ifndef B0_DSPLA_HPP            // Avoid multiple includes...
#define B0_DSPLA_HPP    TRUE    // ...holds to a terminal #endif below...

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Master Header File Listing:
//
// #include  <api/include/b0_dspla.hpp>
// Base Framework abstractions of "display/display context" 
// objects such as "pens", "brushes", "display/HDCs", etc.
//


//
// Include the Zinc base headers.
//

#include    <z_dsp.hpp>
#include    <z_win.hpp>
            // Include the Zinc framework's base-class headers.

#include    <cf_api/bf_typed.h>
            // Include base-framework-dependent typedefs and #defines.

// Map onto the Base Framework's "Display Pen Style" 
// (usually enum var) definitions. 
enum bf_PenStyle   {   CF_PS_SOLID   = ZAF_LINE_SOLID,
                        CF_PS_DASH    = ZAF_LINE_DOTTED, // BNR -- should be DASHED
                        CF_PS_DOT     = ZAF_LINE_DOTTED };
                    // zApp implementation was originally: 
                    // "typedef PenStyle    bf_PenStyle;".  
                    // (see zApp's "zapph/enumdef.hpp").


// Map onto the Base Framework's "Display Brush Hatch" 
// (usually enum var) definitions. 
enum bf_BrushHatch {   CF_BHAT_VERT   = ZAF_PTN_SOLID_FILL,  // BNR
                        CF_BHAT_HORZ   = ZAF_PTN_INTERLEAVE_FILL, // BNR
                        CF_BHAT_NONE   = ZAF_PTN_NULL }; // BNR
                    // zApp implementation was originally:
                    // "typedef BrushHatch  bf_BrushHatch;".
                    // (see zApp's "zapph/enumdef.hpp").




//--------------------------------------------------------------------
// Include Chart Framework class or ID defs/headers this file needs.
// Paths are relative to the IDE's (global) include path: ~/cfrz
//--------------------------------------------------------------------
class       bf_DynaString;
#include    <cf_api/c0_dystr.hpp>

class       bf_Color;
#include    <cf_api/b0_color.hpp>

class       cf_2DPoint;
class       cf_2DRect;
#include    <cf_api/c0_dimen.hpp>
            //
            // This header file maps (typically display-type) 
            // 2D Units such as: "dimensions", "points" 
            // and "rectangles" onto the base class framework...
            //

class       bf_Font;
class       bf_FontSpec;
#include    <cf_api/b0_fonts.hpp>
            // Unfortunately, there's quite a bit of dependency
            // between these two files and sets of objects...

class       bf_2DBitmap;
#include    <cf_api/b0_dsbmp.hpp>

class       cf_Stack;
#include    <cf_api/c0_stack.hpp>

//
// Let's make forward declarations 
// to define the ChartFolio mappings 
// of the underlying framework classes
// we will need to map onto:
//
class bf_Pen;
class bf_Brush;  
class bf_DispLogFontDim;
class bf_Display;
class		bf_ChartView;





class   bf_Pen
{

public:

    bf_Pen(    bf_Color       bf_clrUseThisColor );
                // bf_PenStyle will be CF_SOLID
                // iWidth will be 1 (one).

    bf_Pen(    bf_Color       bf_clrUseThisColor,
                bf_PenStyle    bf_pstyUseThisStyle,
                int             iWidth = 1 );

    ~bf_Pen();

	bf_Color		getColor () { return (color_); }
	bf_PenStyle	getPenStyle () { return pen_style_; }
	int				getWidth () { return width_; }

private:

	bf_Color		color_;
	bf_PenStyle	pen_style_;
	int				width_;

};





class   bf_Brush
{

public:

    bf_Brush( bf_Color bf_clrUseThisColor );

    bf_Brush(  bf_Color       bf_clrUseThisColor,
                bf_BrushHatch  bf_bhatUseThisHatching );

    ~bf_Brush();

	bf_Color		getColor () { return (color_); }

private:

	bf_Color		color_;

};





class   bf_DispLogFontDim
{

public:

    bf_DispLogFontDim();

    bf_DispLogFontDim( int iWidth,
                        int jHeight,
                        bf_Display *p_bf_dspUseThisDisplay = NULL );

    ~bf_DispLogFontDim();

};





class   bf_Display
{

public:

    bf_Display (ZafDisplay* zdisplay, ZafWindowObject* view );

    ~bf_Display();

    //
    // The ChartFolio kernel makes the following calls
    // into this class bf_Display, which must be mapped
    // onto the underlying class application framework:
    //
    BOOL        dsp_devToLog( cf_2DRect &r );

    void        dsp_getVisible( cf_2DRect &r );

    void        dsp_lock();

    BOOL        dsp_logToDev( cf_2DRect &r );

    void        dsp_unlock();

    //
    // The bf_Object-derived classes such as bf_Point
    // extend the set of display-oriented calls that
    // must be mapped onto the base framework's display
    // functionality:
    //
    void        dsp_pushBrush( bf_Brush* p_bf_brushUseThisBrush );
    void        dsp_pushBrush( bf_Color bf_clrForNoHatchBrush );
                // Default op uses brush-hatch == "NoHatch"

    void        dsp_pushPen( bf_Pen* p_bf_penUseThisPen );
    void        dsp_pushPen( bf_Color bf_clrForSolidLine );
                // Default op uses pen-style == "Solid"

    BOOL        dsp_circle( cf_2DPoint     bf_2dpUseThisPoint,
                            int             iRadius );

    bf_Color   dsp_pixel(  int         iX,
                            int         iY );

    bf_Color   dsp_pixel(  int         iX,
                            int         iY,
                            bf_Color   bf_clrUseThisColor );

    bf_Color   dsp_pixel(  cf_2DPoint bf_2dpUseThisPoint );

    bf_Color   dsp_pixel(  cf_2DPoint bf_2dpUseThisPoint,
                            bf_Color   bf_clrUseThisColor );

    bf_Brush*  dsp_popBrush();

    bf_Pen*    dsp_popPen();

    //
    // The bf_Object-derived classes such as bf_PolyLine
    // extend the set of display-oriented calls that
    // must be mapped onto the base framework's display
    // functionality:
    //
    BOOL    dsp_polyline(   cf_2DPoint*    a_bf_2dpStartFrom,
                            int             iNumPoints );

    //
    // The bf_Object-derived classes such as bf_Polygon
    // extend the set of display-oriented calls that
    // must be mapped onto the base framework's display
    // functionality:
    //
    BOOL    dsp_polygon(    cf_2DPoint*    a_bf_2dpStartFrom,
                            int             iNumPoints );


    //
    // The cf_3DBitmap class
    // extends the set of display-oriented calls that
    // must be mapped onto the base framework's display
    // functionality:
    //
    BOOL    dsp_bitmap( bf_2DBitmap*   p_bf_2dbmpUseThisBitMap,
                        cf_2DPoint     bf_2dpPlaceAt,
                        cf_2DRect      bf_2drSizedBy );


    void    dsp_getDirty( cf_2DRect&  r_bf_2drTargetRect );

    //
    // The bf_Chart and bf_Engine classes
    // extend the set of display calls that must be mapped
    // onto a base application framework:
    //
    void        dsp_setDirty();

    void        dsp_pushFont( bf_Font *p_bf_fonUseThisFont );

    bf_Font*   dsp_popFont();

    bf_Color   dsp_textColor();
    bf_Color   dsp_textColor( bf_Color bf_clrUseThisColor );

    bf_Color   dsp_backColor();
    bf_Color   dsp_backColor( bf_Color bf_clrUseThisColor );

    BOOL        dsp_text(   cf_2DPoint bf_2dpPutTextHere,
                            char*       szStringAddress     );
    BOOL        dsp_text(   cf_2DPoint bf_2dpPutTextHere,
                            cf_String  cf_strUseThisString );

    cf_2DRect  dsp_getTextRect( cf_String cf_strUseThisString );
    cf_2DRect  dsp_getTextRect( char*      szStringAddress,
                                 int        iStringLength   );

    //
    // Polar Charts extend the call mapping even more:
    //
    BOOL    dsp_ellipse(    cf_2DPoint bf_2dpUpperLeft,
                            cf_2DPoint bf_2dpLowerRight );

    //
    // As expected, Pie Charts extend the call mapping even more:
    //
    double  zz_calcAngleDegrees(    int iOriginX,
                                    int iOriginY,
                                    int iRadialX,
                                    int iRadialY );

    BOOL    dsp_pie(    cf_2DRect  bf_2drBoundaryEllipseRect,
                        cf_2DPoint bf_2dpStartArcPoint,
                        cf_2DPoint bf_2dpEndArcPoint );

    BOOL    dsp_arc(    cf_2DRect  bf_2drBoundaryEllipseRect,
                        cf_2DPoint bf_2dpStartArcPoint,
                        cf_2DPoint bf_2dpEndArcPoint );

    void    dsp_drawModeSolid();
    void    dsp_drawModeTransparent();
            //
            // Pie slices need to set their pen style
            // to be "transparent" when they draw their
            // front-facing slice face wall, and
            // then set things back to normal...
            //
            // This memeber calls the dsp_drawMode(...)
            // member above with a *specific* value
            // that yields transparent draw ops.
            //

    //
    // Chart Controls and Chart Dialogs also extend the call
    // mapping when they draw rectangular borders on controls:
    //
    BOOL    dsp_rectangle(  cf_2DRect  bf_2drBoundaryRect );
    BOOL    dsp_polyline(   cf_2DRect  bf_2drBoundaryRect );

private:

	int				    curr_width_;
	cf_Stack		    pens_;
	cf_Stack		    brushes_;
	cf_Stack		    fonts_;
	ZafDisplay*		    zdisplay_;
	ZafWindowObject*	view_;

	void			setPenAttributes (bf_Pen*);
	void			setBrushAttributes (bf_Brush*);

};  //
    // End of bf_Display Declaration.
    //

#endif      // Terminates include re-definition protection

//////////////////////////////////////
// END of File: B0_DSPLA.HPP
//////////////////////////////////////
            
