/////////////////////////////////////////
// START of File: B1_CVIEW.HPP
/////////////////////////////////////////

#ifndef B1_CVIEW_HPP            // Avoid multiple includes...
#define B1_CVIEW_HPP    TRUE    // ...holds to a terminal #endif below...

//===================================================================
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997
// by DPC Technology Corporation.  All rights reserved.
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of
// copyright and license specifications, and contact information.
//===================================================================

// Master Header File Listing (from <zafchart.hpp> and/or <cfzz_api.hpp> ):
// #include     <api/include/b1_cview.hpp>
                // Class bf_ChartView is a "Chart View" object
                // in a Model/View/Controller Hierarchy.

//===================================================================
// Define EXACT uses of Base Framework Classes/Files/Libraries:
// Paths are relative to the IDE's (global) include path: ~/cfol_zaf
//===================================================================
class       ZafWindowObject;
#include    <z_win.hpp>
            // Include the Zinc framework's base-class headers.

#include    <cf_api/id_chart.h>
            // Chart type IDs.  Note location specifier
            // is RELATIVE to the Environment/IDE "Include Path".

#include    <cf_api/api_cmds.h>
            // Chart command-IDs.  Note location specifier
            // is RELATIVE to the Environment/IDE "Include Path".

class       bf_Color;
#include    <cf_api/b0_color.hpp>

class       bf_Display;
#include    <cf_api/b0_dspla.hpp>

class       cf_ChartModel;
#include    <cf_api/c1_cmodl.hpp>

//=====================================================================
// Declare a "Chart View" Object in a Model/View/Control Configuration.
//=====================================================================
class   bf_ChartView
        //==============================================================
        // Functions as a "Chart View" object in a Model/View/Controller
        // Hierarchy.  Implemented to be ENCAPASULATED by the base
        // framework's abstraction of a "chart-window-object" object.
        //
        // Used by the Chart Framework to abstract and control display
        // output ops, and system/dsplay events such as window re-draw
        // and re-size events, print events, etc.
        //
        // NOTE: Heavy dependencies exist between the chart-view
        // (b1_cview.hpp) and chart-model (c1_cmodl.hpp) objects!
		//==============================================================
{

public:

    bf_ChartView(   ZafWindowObject     *p_zwoParentForChartView,
                    int     iChartTypeID = CF_ID_BASE_CHART,
                   	int     iIndexMethodID = CF_ID_YVALS_ON_NAT_NS );
                    // This c-tor Makes its OWN Chart Model
                    // in a TIGHTLY-COUPLED Configuration...

    bf_ChartView(   ZafWindowObject    *p_zwoParentForChartView,
                    cf_ChartModel       *p_cf_cmodlUseThisChart );
                    // This c-tor uses an EXISTING Chart Model
                    // In a LOOSELY-COUPLED Configuration...

    virtual         ~bf_ChartView();
    			    // D-tor needs to discriminate whether we allocated a new
    			    // chart model, or were just looking at someone else's...

    void            activateModel( BOOL bForceActivationUpdate = FALSE );
                    // Bool return val shows us the chart's original state

    void            colorDispBack( bf_Color bf_clrUseThisColor );

    virtual int     draw();

    virtual void    dem_setDirty();

    virtual void    dem_show();

    virtual void    dem_show( int iShowFlags );

    bf_Display*     getDisplay();

    BOOL            isModelActivated();

    int             processPrintJob();

    virtual         int print();

    virtual         void setMetrics();

    virtual         int size();

    cf_ChartModel*      viewModel();
                        // Get a pointer to the "Chart Model" to which
                        // this "Chart View" provides a display context.

    ZafWindowObject*    viewWindow();
    void                viewWindow( ZafWindowObject *p_zwoParentForChartView );
                        // Get/Set the pointer to the "Window Object"
                        // where this Chart-View object is encapsulated.
                        // This parent window object will also need to
                        // provide our chart-view with an implicit
                        // pointer to a display-context object...

    //=======================================================================
    // Start of Zinc/ZAF v5.x ZafChart-specific API-interface/helper methods:
    //=======================================================================
    cf_3DGridTickSeries*    zz_accessXAxisLower();
    cf_3DGridTickSeries*    zz_accessYAxisLeft();
    cf_3DTextMarker*        zz_accessChartTitle();

    cf_DataGroup*       zz_addDataGroup( int iGroupIndex );
    cf_XYZ*             zz_addDataPoint( int iGroupIndex, double dDataValue );

    cf_DataGroup*       zz_clearDataGroup( int iGroupIndex );
    cf_DataGroup*       zz_clearDataAll();


    void                zz_dataGroupPenColor(  int             iGroupIndex,
                                            ZafLogicalColor zlcNewColor );
    ZafLogicalColor     zz_dataGroupPenColor(  int iGroupIndex );


    void                zz_dataGroupBrushColor( int                iGroupIndex,
                                             ZafLogicalColor    zlcNewColor );
    ZafLogicalColor     zz_dataGroupBrushColor( int iGroupIndex );


    void                zz_dataGroupBrushHatch(  int             iGroupIndex,
                                        ZafLogicalFillPattern zlfpNewPat );
    ZafLogicalFillPattern   zz_dataGroupBrushHatch(  int iGroupIndex );


    void                zz_dataGroupFontSpec(  int             iGroupIndex,
                                            ZafLogicalFont  zlfNewFont );
    ZafLogicalFont      zz_dataGroupFontSpec( int iGroupIndex );


    void                zz_dataGroupPenStyle(  int               iGroupIndex,
                                        ZafLogicalLineStyle   zllsNewStyle );
    ZafLogicalLineStyle zz_dataGroupPenStyle(  int iGroupIndex );


    cf_DataGroup*       zz_getDataGroup(   int iGroupIndex );

    cf_XYZ*             zz_getDataXYZ(     int iGroupIndex,
                                        int iDataIndex );

    const ZafIChar*     zz_getTextTitle();
    const ZafIChar*     zz_getTextXAxisLower();
    const ZafIChar*     zz_getTextYAxisLeft();


    bool                zz_getXAxisLower();
    bool                zz_getYAxisLeft();

    ZafLogicalColor     zz_getXYAxisColor();

    void                zz_setChartType( int iChartTypeID );
	                    // Note that old engine will be DELETED!

    void                zz_setIndexMethod( int iIndexMethodID );
                        // Index IDs per ~/api/res/id_chart.h

    bool                zz_setXAxisLower( bool bOnOff );
    bool                zz_setYAxisLeft( bool bOnOff );

    ZafLogicalColor     zz_setXYAxisColor( ZafLogicalColor zlcNewColor );

    void                zz_setTextTitle( const ZafIChar* cp_zicUserString );
    void                zz_setTextXAxisLower( const ZafIChar* cp_zicUserString );
    void                zz_setTextYAxisLeft( const ZafIChar* cp_zicUserString );

    //=====================================================================
    // End of Zinc ZAF v5.x ZafChart-specific API-interface/helper methods.
    //=====================================================================

protected:

    void			initialize();
                    // Modularized common c-tor functionality...

    ZafWindowObject *p_zwoForChartView_;
                    // This object will need to keep track of who owns it,
                    // so that it can pass events and/or msgs BACK to this
                    // parent-window-object.

    bf_Display      *p_bf_dspForChartView_;
                    // We'll also need to have a display-context provided
                    // by our parent window-object.

    cf_ChartModel   *p_cf_cmodlTarget_;
                    // The target chart-model which will receives the
                    // appropriate chart-view events (which will be turned
                    // into ABSTRACTED objects NOT dependent upon the
                    // base framework...

    BOOL            bWasTargetNewed_;
                    // We might have created our chart-model, or just be
                    // using someone else's, so we won't delete the
                    // chart-model without checking first!

    BOOL            bIsModelActivated_;
                    // Flag to track whether igeometry/projection/draw
                    // parameters, etc. have been initialized.  C-tor
                    // inits to FALSE, activateModel() sets to TRUE.

};  // Ends bf_ChartView Declaration.

#endif      // Terminates include re-definition protection

/////////////////////////////////////////
// END of File: B1_CVIEW.HPP
/////////////////////////////////////////


