//	Zinc Application Framework - BZ_ZCHT.HPP
//	COPYRIGHT (C) 1990-1997.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#ifndef BZ_ZCHT_HPP
#define BZ_ZCHT_HPP

#include <z_chart.hpp>
#include <cf_api/id_chart.h>
#include <cf_api/b1_cview.hpp>

class ZafChart : public ZafChartStub
{
public:
	// --- General members ---
	ZafChart(int left, int top, int width, int height,
		int iChartTypeID = CF_ID_BASE_CHART,
		int iIndexMethodID = CF_ID_YVALS_ON_NAT_NS);
	ZafChart(const ZafChart &copy);
	virtual ~ZafChart(void);

	// --- Persistence ---
	virtual ZafWindowObject *Duplicate(void);
	ZafChart(const ZafIChar *name, ZafObjectPersistence &persist);
	static ZafElement *Read(const ZafIChar *name, ZafObjectPersistence &persist);
	virtual void Write(ZafObjectPersistence &persist);

	// --- Zinc to ChartFolio Data Method Mappings ---	
	//get
	ZafLogicalColor DataGroupBackgroundColor(int groupIndex, ZafLogicalColor *mono = ZAF_NULLP(ZafLogicalColor));
	ZafLogicalFillPattern DataGroupFillPattern(int groupIndex) const;
	ZafLogicalColor DataGroupForegroundColor(int groupIndex, ZafLogicalColor *mono = ZAF_NULLP(ZafLogicalColor));
	ZafLogicalFont DataGroupFont(int groupIndex) const;
	ZafLogicalLineStyle DataGroupLineStyle(int groupIndex) const;
	//set
	ZafError AddDataGroup(int groupIndex);
	ZafError AddDataPoint(int groupIndex, double dDataValue);
	ZafError ClearDataAll(void);
	ZafError ClearDataGroup(int groupIndex);
	ZafLogicalColor SetDataGroupBackgroundColor(int groupIndex, ZafLogicalColor color, ZafLogicalColor mono = ZAF_MONO_NULL);
	ZafLogicalFillPattern SetDataGroupFillPattern(int groupIndex, ZafLogicalFillPattern fillPattern);
	ZafLogicalColor SetDataGroupForegroundColor(int groupIndex, ZafLogicalColor color, ZafLogicalColor mono = ZAF_MONO_NULL);
	ZafLogicalFont SetDataGroupFont(int groupIndex, ZafLogicalFont font);
	ZafLogicalLineStyle SetDataGroupLineStyle(int groupIndex, ZafLogicalLineStyle lineStyle);

	// --- Attributes ---
	// set
	ZafLogicalColor SetAxisColor(ZafLogicalColor color, ZafLogicalColor mono = ZAF_MONO_NULL);
	int SetChartType(int chartTypeID);
	int SetIndexMethod(int indexMethodID);
	bool SetMarksXAxis(bool xAxisMarks);
	bool SetMarksYAxis(bool yAxisMarks);
	const ZafIChar *SetTextTitle(const ZafIChar *text);
	const ZafIChar *SetTextXAxis(const ZafIChar *text);
	const ZafIChar *SetTextYAxis(const ZafIChar *text);

	// --- Optional entry points into ChartFolio ---
	cf_ChartModel *ActiveModel(void);
	cf_DataGroup *DataGroup(int groupIndex);
	cf_XYZ *DataPoint(int groupIndex, int dataIndex);

	// --- Window object members ---
	ZafEventType Draw(const ZafEventStruct&, ZafEventType);
	ZafEventType Event(const ZafEventStruct&);

	// --- Class identification ---
	static ZafClassID classID;
	static ZafClassNameChar ZAF_FARDATA className[];
	virtual ZafClassID ClassID(void) const { return (classID); }
	virtual ZafClassName ClassName(void) const { return (className); }
	virtual bool IsA(ZafClassID compareID) const { return ((compareID == classID) ? true : ZafChartStub::IsA(compareID)); }
	virtual bool IsA(ZafClassName compareName) const { return ((compareName == className) ? true : ZafChartStub::IsA(compareName)); }

protected:
	bf_ChartView *p_cf_cviewForChartModel_;

	// --- Persistence ---
	virtual ZafObjectConstructor ReadFunction(void) { return (ZafChart::Read); }
};

#endif // BZ_ZCHT_HPP

