//=====================================
// START of File: C1_CMODL.HPP
//=====================================

#ifndef C1_CMODL_HPP            // Avoid multiple includes...
#define C1_CMODL_HPP    TRUE    // ...holds to a terminal #endif below...

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Master Header File Listing:
//
// #include    <cf_api/c1_cmodl.hpp>
//
// This "Chart-Model" object hold user data and chart attributes,
// (in "Data-Groups") and controls a "Chart-Engine" (which actually 
// generates and renders the chart's geometrical objects).
//
// A Chart-Model exists as part of a "Chart-View" which is
// usually owned by some sort of "Chart-Control", e.g. a
// "Chart-Window" object, or a "Chart-Dialog" object, or
// some other object in a Model/View/Controller relationship.
//


//--------------------------------------------------------------------
// Include Chart Framework class or ID defs/headers this file needs.
// Paths are relative to the IDE's (global) include path: ~/cfrz
//--------------------------------------------------------------------
#include    <cf_api/id_chart.h>
            // Pull in "chart type-IDs" used by our cpp file.

#include    <cf_api/cf_typed.h>
            // contains Chart Framework typedef(s)

#include    <cf_api/cf_const.h>
            // defines chart engine default values

class       cf_DyArr;   // We'll use dynamic arrays of (void) ptrs
#include    <cf_api/c0_dyarr.hpp>  // to cf-data-group objects

class       cf_String;
#include    <cf_api/c0_dystr.hpp>

class       bf_Color;
#include    <cf_api/b0_color.hpp>

class       bf_Display;
            // also has enums bf_PenStyle and bf_BrushHatch...
#include    <cf_api/b0_dspla.hpp>

class       bf_2DBitmap;
#include    <cf_api/b0_dsbmp.hpp>

class       cf_DataGroup;
#include    <cf_api/c1_dgrou.hpp>

class       cf_ChartEngine;
#include    <cf_api/c1_engin.hpp>

class       bf_ChartView;
#include    <cf_api/b1_cview.hpp>

// Define values used throughout this module:
#define     C1_CMODL_NUM_PALETTE_COLORS         16
#define     C1_CMODL_NUM_PEN_STYLES              3
#define     C1_CMODL_NUM_BRUSH_HATCHES           3

//
// Declare a ChartFolio "User Chart Data-Model" Base Class:
//
class   cf_ChartModel 

{

public:

    cf_ChartModel(  bf_ChartView    *p_bf_cviewOwner,
                    int             iChartTypeID    = CF_ID_BASE_CHART,
                    int             iIndexMethodID  = CF_ID_YVALS_ON_NAT_NS );

    cf_ChartModel(  int             iChartTypeID    = CF_ID_BASE_CHART,
                    int             iIndexMethodID  = CF_ID_YVALS_ON_NAT_NS );
                                    // Chart IDs defined in file:
                                    // ~/api/res/id_chart.h

    ~cf_ChartModel();

    cf_DataGroup*   accessGroupsList();
    cf_DataGroup*   accessDomainItems();
    cf_DataGroup*   accessRangeValues();
                    // Allow users to access the implicit "domain",
                    // "range", and "group-list" data-groups this
                    // model constructs from user data.


    cf_DataGroup*   activateGroup(  int         iGroupIndex,
                                    const char* a_cDataGroupLabel = NULL );
                    // Users interact with (e.g. add data to) a pointer
                    // to the current "active" group in a "dynamic array"
                    // of data groups.  Optional label from const char*
                    // string shows up on the group (usually Z) axis.


    cf_XYZ*     addDataYVal( double      dUserYVal,
                             const char* a_cDomainItemLabel = NULL,
                             const char* a_cRangeValueLabel = NULL );
                //===================================================
                // Add a (y) data item to the current "activated"
                // (z) group.  Domain (x) index values automatically
                // generated according to the chart-model's current
                // domain-indexing method ID. Group (z) index
                // generated from current "activateGroup()" call.
                // Optional labels from const char* strings show up
                // on respective domain and range axes. Note that
                // THIS IS THE RECOMMENDED DATA STORAGE METHOD for
                // about 90 to 95 percent of all Chart Types!
                //===================================================

    cf_XYZ*     addDataXYVal( double      dUserXVal,
                              double      dUserYVal,
                              const char* a_cDomainItemLabel = NULL,
                              const char* a_cRangeValueLabel = NULL );
                //===================================================
                // Add a (x,y) data item to the current "activated"
                // (z) group. Domain (x) index value is SPECIFIED by
                // the user.  Domain index method AUTOMATICALLY reset
                // to CF_ID_YVALS_AT_USR_XS; e.g. x = 2.3, 10.2 .. X.
                // Z-index generated from current "activateGroup()"
                // call.  Optional labels from const char* strings
                // show up on respective domain and range axes.
                // Note that this data storage method is ONLY
                // RECOMMENDED for X/Y "Scatter Plots" or EXPLICIT
                // "Time-Series Plots"!
                //===================================================


    cf_XYZ*     addDataXYZVal( double      dUserXVal,
                               double      dUserYVal,
                               double      dUserZVal,
                               const char* a_cDomainItemLabel = NULL,
                               const char* a_cRangeValueLabel = NULL );
                //===================================================
                // Add a (x,y,z) data item to the current "activated"
                // group. Domain (x) index value is SPECIFIED by the
                // user.  Domain Index method AUTOMATICALLY reset
                // to CF_ID_YVALS_AT_USR_XS; e.g. x = 2.3, 10.2 .. X.
                // Note that y AND z values are specified by the user,
                // so there is NOT implicit z-group indexing. The
                // optional labels from const char* string show up
                // on respective domain and range axes.  Note that
                // this data storage method is ONLY RECOMMENDED
                // for X/Y/Z "Scatter Plots"!
                //===================================================



    cf_XYZ* addLabelDomainItem( double      dDomainItemIndex,
                                const char* a_cDomainItemLabel );
    cf_XYZ* addLabelDomainItem( double      dDomainItemIndex,
                                double      dValueForLabel );
            // Add an index (char string OR user-specified
            // value) label to the "domain axis item" group:

    cf_XYZ* addLabelRangeValue( double      dRangeValueIndex,
                                const char* a_cRangeValueLabel );
    cf_XYZ* addLabelRangeValue( double      dRangeValueIndex,
                                double      dValueForLabel );
            // Add an user-specified (char string OR user-specified)
            // value) label to the "range values" group:

    void    applyAllXforms();
    void    applyGeomXform();
    void    applyProjXform();
            // Apply geometry xforms to the child-chart-engine
            // encapsulated wihin this chart-model.

    int     dataGroupActive();
            // Returns the current active index value

    int     dataGroupCount();
            // Returns the number of allocated data-groups

    int     dataGroupsStartAt();
    int     dataGroupsEndAt();
            // Yields the min and max index values for
            // the current set of activated data-groups

    cf_DataGroup*   dataGroupsReset();
                    // Performs a "reset" op on each activated
                    // data-group (which will delete each group's
                    // allocated XYZ points, but retain other
                    // attributes, such as colors, labels, etc).
                    // Returns a pointer to the most-recently
                    // activated data group.
                    
    cf_DataGroup*   dataGroupsDelete();
                    // Performs a "delete" op on each activated 
                    // data-group, then re-inits a default active
                    // data-group number one, and returns a pointer
                    // to this default data-group.


    int     dataIndexMethod();
    void    dataIndexMethod(    int     iIndexMethodID,
                                BOOL    bForceEngineUpdate = TRUE );
            //=======================================================
            // Get/set Index Method IDs as follows:
            //  CF_ID_YVALS_ON_NAT_NS; e.g: n = 1, 2, 3, .. N.
            //  CF_ID_YVALS_AT_INT_IS; e.g: i = 0, 1, 2, .. I.
            //  CF_ID_YVALS_AT_USR_XS; e.g. x = 2.3, 10.2 .. X.
            //
            // NOTE that if you pass an int val != {one of the IDs}
            // above, this chart-model will DEFAULT to use Natural-N
            // indexing!
            //=======================================================


    double      dataRangeInclMaxBuffer();
    void        dataRangeInclMaxBuffer( double dAddOntoCurrentRange );
                // Get/set extra "margin" onto the user-data-range.
                // Subject to range's "use-max-ceiling" state (below).  
                // C-tor inits this buffer value to zero.

    double      dataRangeInclMaxCandidate();
    void        dataRangeInclMaxCandidate( double dPossibleMaxVal );
                // Get/set a "candidate" max value for user-data-range.
                // Subject to range's "max-buffer" state (above).
                // Subject to range's "use-max-ceiling" state (below).

    BOOL        dataRangeInclMaxCandidateInUse();
    void        dataRangeInclMaxCandidateInUse( BOOL bNewState );
                // Turn consideration of any candidate max value for 
                // the user data-range ON/OFF.  C-tor inits to FALSE.
                // Calls to dataRangeInclMaxCandidate( double ); 
                // (immediately above) reset this state to TRUE!

    BOOL        dataRangeInclMaxCeiling();
    void        dataRangeInclMaxCeiling( BOOL bNewState ); 
                // If set TRUE, uses math.h ceil() to round user data
                // ranges UP to an integer value.  C-tor inits to FALSE.

    double      dataRangeInclMinBuffer();
    void        dataRangeInclMinBuffer( double dAddOntoCurrentRange );
                // Get/set extra "margin" onto the user-data-range.
                // Subject to range's "use-min-floor" state (below).  
                // C-tor inits this buffer value to zero.

    double      dataRangeInclMinCandidate();
    void        dataRangeInclMinCandidate( double dPossibleMinVal );
                // Get/set a "candidate" min value for user-data-range.
                // Subject to range's "min-buffer" state (above).
                // Subject to range's "use-min-floor" state (below).

    BOOL        dataRangeInclMinCandidateInUse();
    void        dataRangeInclMinCandidateInUse( BOOL bNewState );
                // Turn consideration of any candidate min value for 
                // the user data-range ON/OFF.  C-tor inits to FALSE.
                // Calls to dataRangeInclMinCandidate( double ); 
                // (immediately above) reset this state to TRUE!

    BOOL        dataRangeInclMinFloor();
    void        dataRangeInclMinFloor( BOOL bNewState ); 
                // If set TRUE, uses math.h floor() to round user data
                // ranges DOWN to an integer value.  C-tor inits to FALSE.


    BOOL        dataRangeInclOrigAlways();
    void        dataRangeInclOrigAlways( BOOL bNewState ); 
                // Get/set data-range-origin-in(/ex)clusion treatment.
                // C-tor inits this FORCED origin inclusion to FALSE;
                
    double      dataRangeInclOrigFromDelta();
    void        dataRangeInclOrigFromDelta( double dNumDataDeltasAway );
                // Get/set data-range-origin-in(/ex)clusion treatment.
                // C-tor inits this data-dependent limit to 3 data deltas.


    double      dataXItemsMax();
    double      dataXItemsMin();
    double      dataXItemsExtent();
                // Query methods for special use.  See note below...

    double      dataYValuesMax();
    double      dataYValuesMin();
    double      dataYValuesExtent();
                // Query methods for special use.  See note below...

    double      dataYValSumsMax();
    double      dataYValSumsMin();
    double      dataYValSumsExtent();
                // Query methods for special use.  See note below...

    double      dataZGroupsMax();
    double      dataZGroupsMin();
    double      dataZGroupsExtent();
                // Query the active data groups for min/max/range
                // values.  For use in engine axis/tick ops, etc...
                // ...or for "advanced" or customized-engine usage.

    void        draw( bf_Display *p_bf_dspUseThisDisplay );
                // Generally called by an associated chart-view object
                // configured in a model/view/controller "chart-window".
                // Public, but not for general use.  See initiateDrawOp
                // immediately below...

    void        initiateDrawOp();
    void        initiateDisplayMetricsOp();
                // Preferred method to cause chart rendering into a
                // chart-view display object.  Tells the chart's
                // model/view/controller configuration to render
                // itself into the current display context.


    bf_ChartView*   modelView();
    void            modelView(  bf_ChartView    *p_bf_cviewNewView,
                                BOOL    bDeleteOldView = TRUE );
                                        
    cf_ChartEngine* modelEngine();
    void            modelEngine(    int     iChartTypeID,   
                                    int     iIndexMethodID,
                                    BOOL    bDeleteOldEngine = TRUE );
                                    // IDs in "~/api/res/id_chart.h"

    void            modelEngine(    cf_ChartEngine  *p_cf_engCustomized,
                                    BOOL    bDeleteOldEngine = TRUE ); 

    cf_Kernel*      modelKernel();
    void            modelKernel(    cf_Kernel   *p_cf_kNewKernel,
                                    BOOL    bDeleteOldKernel = TRUE );


    bf_Color    paletteColor(   int iModIntoAvailColors );
    void        paletteColor(   int     iModIntoAvailColors, 
                                bf_Color   bf_clrNew );
                                // get/set from the palette of available 
                                // colors that are applied to newly-
                                // created data groups...

    void        paletteColors(  );
                                // Allow users to (re)init a palette of
                                // colors to apply to newly created
                                // data-groups...


    bf_BrushHatch   paletteBrushHatch(  int     iModIntoAvailHatches );
    void            paletteBrushHatch(  int         iModIntoAvailHatches, 
                                        bf_BrushHatch  bf_bhatNewHatch );
                    // get/set from the palette of available brush hatches

    void            paletteBrushHatches(// C-tor inits #1 to "CF_BHAT_NONE"
                                        bf_BrushHatch  bf_bhat01, 
                                        // C-tor inits #2 to "CF_BHAT_VERT"
                                        bf_BrushHatch  bf_bhat02, 
                                        // C-tor inits #3 to "CF_BHAT_HORZ"
                                        bf_BrushHatch  bf_bhat03 );
                    // (re)init a palette of available brush hatches.
        
                      
    BOOL            paletteMarkers2D3D( int     i2For2D_3For3D );
                    // Boolean query method return TRUE if flag is on,
                    // FALSE otherwise.  Pass int value of 2 to report
                    // on 2D-markers and 3 to report on 3D-markers. 
                    // Any other int value return boolean FALSE.
    
    void            paletteMarkers2D3D( BOOL    bDisp2DMarkers,
                                        BOOL    bDisp3DMarkers,
                                        BOOL    bResetCurrentGroups );
                    // Get/set 2D/3D point marking on/off -- only
                    // affects objects/engines that perform separate
                    // 2D/3D marking, (e.g. 2D/3D Line Charts). 



    bf_PenStyle     palettePenStyle(    int     iModIntoAvailStyles );
    void            palettePenStyle(    int         iModIntoAvailStyles, 
                                        bf_PenStyle    bf_pstyNewStyle );
                    // get/set from the palette of available pen styles

    void            palettePenStyles(   // C-tor inits #1 to "CF_PS_SOLID"
                                        bf_PenStyle  bf_psty01, 
                                        // C-tor inits #2 to "CF_PS_DASH"
                                        bf_PenStyle  bf_psty02, 
                                        // Cot-r inits #3 to "CF_PS_DOT"
                                        bf_PenStyle  bf_psty03 ); 
                    // (re)init a palette of available pen styles.
        

    void    updateEngineObjects( BOOL bForceDataUpdate = FALSE,
                                 BOOL bForceReConfig = FALSE,
                                 BOOL bApplyXforms = TRUE );
            // Re-generate and/or re-configure the chart-engine's 
            // chart-objects -- depending upon the data-state of the
            // chart-model.  IF the data-state of the chart-model 
            // has changed, chart-engine object re-generation and 
            // chart-engine re-configuration and re-transformation
            // will naturally occur.  But data-resets can also be 
            // forced, and engine object re-configs and re-transforms
            // can also be suppressed (for performance purposes, 
            // etc.) depending upon the update needs of the caller.

    void    updateDisplayMetrics( bf_Display *p_bf_dspHasChanged );
            // Called by this chart-model's associated chart-view in
            // response to display context changes (e.g. re-sizes).


protected:

    //=======================================================
    // These protected methods are called by the standard 
    // chart-model's addData*() interface method according 
    // to the current state of the data indexing mode set 
    // by either the c-tor or the user's calls to the
    // chart-model's dataIndexMethod(...) method(s).
    //
    // The methods are defined as protected so that users
    // will adhere to the (hopefully clear and simple)
    // dataIndexMethod(...) ... addData*(...) methodlogy,
    // but are NOT defined as private, so that if the user
    // wants to derive their OWN custom chart-model object,
    // these methods can thence be re-used!...
    //=======================================================
    cf_XYZ* addDataYValIndexNatN( double    dUserYVal,
                                const char* a_cDomainItemLabel = NULL,
                                const char* a_cRangeValueLabel = NULL );
            //--------------------------------------------------
            // Add a data item to the current "activated" group:
            // Adds user's Y-val with an X-Index equal to
            // Natural Number Sequence 'N' (1, 2, 3, .. 'n').
            // Z-index generated from current "activateGroup()" 
            // call.  Optional labels from const char* strings
            // show up on respective domain and range axes.
            //--------------------------------------------------
    
    cf_XYZ* addDataYValIndexIntI( double    dUserYVal,
                                const char* a_cDomainItemLabel = NULL,
                                const char* a_cRangeValueLabel = NULL );
            //--------------------------------------------------
            // Add a data item to the current "activated" group:
            // Adds user's Y-val with an X-Index equal to
            // Integer Number Sequence "I" (0, 1, 2, .. 'i')
            // Z-index generated from current "activateGroup()" 
            // call.  Optional labels from const char* strings
            // show up on respective domain and range axes.
            //--------------------------------------------------

    //============================================================
    // Declare critical protected vars, which are non-accessible
    // to the public, but potentially accessible for any user
    // derivations of customized chart-model objects:
    //============================================================
    bf_ChartView    *p_bf_cviewParent;
                    //--------------------------------------------
                    // Pointer to a "display event manager" that
                    // keeps a display which provides:
                    //
                    // (1)  a display context for the kernel
                    //      to perform 3D-to-2D projection 
                    //      transformation calculations, and 
                    //
                    // (2)  the actual chart display rendering
                    //      context...
                    //--------------------------------------------

    cf_Kernel       *p_cf_kChartKernel;
                    //--------------------------------------------
                    // Pointer to the kernel which controls the 
                    // state of this chart's tramsformation geometry...
                    //--------------------------------------------

    cf_ChartEngine  *p_cf_engChartEngine;
                    //--------------------------------------------
                    // Pointer to the engine which controls the
                    // "geometry model" of this "chart data model"...
                    //--------------------------------------------
    
    int             iIndexID;
                    //--------------------------------------------
                    // Indexing method set by indexMethod() member.
                    // C-tor inits default of CF_ID_YVALS_ON_NAT_NS
                    //--------------------------------------------

private:

    int         i,j,k; // Std. ICVs (Iteration Control Variables).

    bf_Color    a_bf_clrChartPalette[ C1_CMODL_NUM_PALETTE_COLORS ];
                // Declare a "Color Palette" for users to select
                // colors for the objects (particularly dynamically
                // created data groups) that this cf_ChartModel object
                // will create and control...

    bf_BrushHatch
                a_bf_bhatChartPalette[ C1_CMODL_NUM_BRUSH_HATCHES ];
                // Same Concept:  A "Palette" of brush hatches for the
                // dynamically created data groups to use...

    bf_PenStyle    
                a_bf_pstyChartPalette[ C1_CMODL_NUM_PEN_STYLES ];
                // Same Concept:  A "Palette" of pen styles for the
                // dynamically created data groups to use...
                
    BOOL        bDraw2DPointMarkers;    // C-tor inits = FALSE.
    BOOL        bDraw3DPointMarkers;    // C-tor inits = FALSE.
                // Same Concept:  A "Palette" of BOOL flags that
                // indicate 2D/3D markers for objects created from
                // the data model...
    
    BOOL        bDataRangeInclOrigAlways;       // C-tor inits = FALSE; 
    double      dNumDataDeltasForOrigInclus;    // C-tor inits = 3.0;
                // Attribute vars for Data/Range/Origin Inclusion Treatment:

    double      dBufferRangeMinLimitBy;     // C-tor inits = 0.0;
    BOOL        bRangeMinLimitUsesFloor;    // C-tor inits = FALSE
                // Range-calculation modifier-vars.    
    
    double      dBufferRangeMaxLimitBy;     // C-tor inits = 0.0;
    BOOL        bRangeMaxLimitUsesCeil;     // C-tor inits = FALSE
                // Range-calculation modifier-vars.    

    double      dCandidateRangeMaxLimit;     // C-tor inits = 0.0;
    BOOL        bRangeMaxLimitUsesCand;      // C-tor inits = FALSE
                // Range-calculation modifier-vars.    

    double      dCandidateRangeMinLimit;     // C-tor inits = 0.0;
    BOOL        bRangeMinLimitUsesCand;      // C-tor inits = FALSE
                // Range-calculation modifier-vars.    

    int         iNumAllocatedGroups;
    int         iMaxGroupIndex;
    int         iMinGroupIndex;
    int         iActiveIndex;
    BOOL        bUpdatedChartData;
                // Run-time state vars.

    cf_DyArr    da_p_cf_dgDataGroups;
    //===============================================================
    // This cf_ChartModel will coordinate the creation, destruction, 
    // and manipulation of a set of cf_DataGroup objects...
    //
    // C-tor inits iNumAllocatedGroups = 0,  and addData*Val(...) 
    // methods perform an ++ op before any cf_DataGroup is added.
    // This implies: indexing by natural numbers, 1..n.
    //
    // C-tor inits iMinGroupIndex = iMaxGroupIndex = 1, and creates 
    // a default "active" group at index 1.
    // iMax val != iNum val indicates "empty" (non-assigned) index vals.
    //
    // Dynamic Array "grow-by" val is #define'd in this *.cpp 
    // file as C1_CMODL_DGROU_GROWBY.
    //
    // C-tor inits bUpdatedChartData = FALSE; After this, 
    // any addData*Val(...) methods reset this flag = TRUE.
    // The updateChartEngine() method (when complete) resets 
    // this flag = FALSE, which implies that the chart engine 
    // has been updated from the last chart/user data update, 
    // and is now awaiting any further chart/user date updates
    // (i.e. for bUpdatedChartData to be assigned = TRUE, and
    // updateChartEngine() to be called...).
    //===============================================================

    cf_DataGroup    *p_cf_dgGroupsList;
    cf_DataGroup    *p_cf_dgDomainItems;
    cf_DataGroup    *p_cf_dgRangeValues;
    //===============================================================
    // These data groups are responsible for keeping track of
    // impliticly generated or explicitly set (by the user)
    // index values that can be used by axis grid/tick-series
    // objects, etc.
    //===============================================================
    

};  //
    // End of cf_ChartModel declaration.
    //

#endif      // Terminates include re-definition protection

//////////////////////////////////////
// END of File: C1_CMODL.HPP
//////////////////////////////////////

            
