//////////////////////////////////////
// START of File: C1_ENGIN.HPP
//////////////////////////////////////
               
#ifndef C1_ENGIN_HPP            // Avoid multiple includes...
#define C1_ENGIN_HPP    TRUE    // ...holds to a terminal #endif below...

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Master Header File Listing:
//
// #include    <api/include/c1_engin.hpp> (alt: ce1_veng)
// Chart Framework "Chart Engine" BASE class object.
//
 
//--------------------------------------------------------------------
// Include Chart Framework class or ID defs/headers this file needs.
// Paths are relative to the IDE's (global) include path: ~/cfrz
//--------------------------------------------------------------------
#include    <cf_api/id_chart.h> 
            // Pull in "chart type-IDs" used by our cpp file.

#include    <cf_api/cf_typed.h>
            // contains Chart Framework typedef(s)
            
#include    <cf_api/cf_const.h>    
            // defines chart engine default values

class       cf_String;
#include    <cf_api/c0_dystr.hpp>

class       bf_2DBitmap;
class       bf_Display;
#include    <cf_api/b0_dspla.hpp>
            // which pulls in the classes above...

class       cf_DyArr;   // We'll use dynamic arrays of (void) ptrs
#include    <cf_api/c0_dyarr.hpp>  // to "virtual-3D-objects"

class       cf_3DPoint;
class       cf_3DPolygon;
#include    <cf_api/co1_pgon.hpp>
            // which pulls in the classes above...

class       cf_TitleMarkerMgr;
class       cf_3DTextMarkerSeries;
#include    <cf_api/co1_tms.hpp>

class       cf_3DGridTickSeries;
#include    <cf_api/co1_gts.hpp>

class       cf_3DBitmap;
#include    <cf_api/co1_cbmp.hpp>

class       cf_ChartModel;
#include    <cf_api/c1_cmodl.hpp>

//
// Declare a Chart Framework "Chart Engine" Base Class:
//
class   cf_ChartEngine 

{
                      
public:

    cf_ChartEngine( cf_ChartModel   *p_cf_cmodlOwner,
                    int             iChartTypeID,
                    int             iIndexMethodID
                                    = CF_ID_YVALS_ON_NAT_NS );
                                    // Chart IDs are defined in 
                                    // file: ~/api/res/id_chart.h
      
    virtual     ~cf_ChartEngine();
                //...and virtual d-tors will be VERY important for
                // any chart-engines derived from this base class...

    cf_3DGridTickSeries*    axis_X_BackFace();
    cf_3DGridTickSeries*    axis_X_BottomFace();
    cf_3DGridTickSeries*    axis_Y_LeftFace();
    cf_3DGridTickSeries*    axis_Y_BackFace();
    cf_3DGridTickSeries*    axis_Z_LeftFace();
    cf_3DGridTickSeries*    axis_Z_BottomFace();
    cf_3DGridTickSeries*    axis_Polar_Floor();
                            // accessor methods for implicit grid/tick
                            // series objects associated with a standard
                            // chart-engine.  If the objects do not yet
                            // exist they are new()-ed by the engine.

    virtual void    applyDispMetrics();
    virtual void    applyGeomXform();
    virtual void    applyProjXform();
                    // Causes a cascading geometry/projection update(s)
                    // throughout this chart-engine's collection of
                    // components and/or virtual-geometry-objects...

    virtual void    autoConfigure();
                    // Causes a chart-component reconfiguration to
                    // conform to latest user-programmer preferences
                    // (such as perspective, proportioning, etc.).

    BOOL            bitmapChartWallFile();
    void            bitmapChartWallFileName( const char *a_cFileNameString );
    const char*     bitmapChartWallFileName();
                    // Methods for a "background-bitmap" source file.
                    // Also see dispBitmapChartWall(); below...

    BOOL            bitmapWallPaperFile();
    void            bitmapWallPaperFileName( const char *a_cFileNameString );
    const char*     bitmapWallPaperFileName();
                    // Methods for a "wall-projected-bitmap" source file.
                    // Also see dispBitmapWallPaper(); below...

    bf_Color        colorDispBack();
    void            colorDispBack( const bf_Color &cr_bf_clrNew );
                    // display-attribute methods for background-color

    bf_Color        colorFaceBack();
    void            colorFaceBack( const bf_Color &cr_bf_clrNew );

    bf_Color        colorFaceBottom();
    void            colorFaceBottom( const bf_Color &cr_bf_clrNew );

    bf_Color        colorFaceLeft();
    void            colorFaceLeft( const bf_Color &cr_bf_clrNew );
                    // display-attribute methods for chart-face-colors


    double          dataChartXRangeMin();
    double          dataChartXRangeMax();
    double          dataChartYRangeMin();
    double          dataChartYRangeMax();
    double          dataChartZRangeMin();
    double          dataChartZRangeMax();
                
    double          dataVirtXOrigin();
    double          dataVirtYOrigin();
    double          dataVirtZOrigin();

    double          dataVirtXRangeMin();
    double          dataVirtXRangeMax();
    double          dataVirtYRangeMin();
    double          dataVirtYRangeMax();
    double          dataVirtZRangeMin();
    double          dataVirtZRangeMax();
                    // Data-query methods used by chart-objects 
                    // encapsulated by this chart-engine (which likely
                    // do NOT have direct access to this chart-engine's
                    // parent-chart-model) -- such as automatic-axis
                    // grid/tick-series objects.
                                
    BOOL            disp2DMode();
    virtual void    disp2DMode( BOOL bNewState );

    BOOL            disp3DMode();
    virtual void    disp3DMode( BOOL bNewState );
                    // Display mode attribute methods.  Note that the
                    // 2D and 3D methods are logically complementary.

    BOOL            dispBitmapChartWall();
    void            dispBitmapChartWall( BOOL bNewState );
                    // Displays a "wall-projected-bitmap" source file.
                    // Also see bitmapChartWallFileName(); above...

    BOOL            dispBitmapWallPaper();
    void            dispBitmapWallPaper( BOOL bNewState );
                    // Displays a "background-bitmap" source file.
                    // Also see bitmapWallPaperFileName(); above...

    BOOL            dispFaceBack();
    void            dispFaceBack( BOOL bNewState );

    BOOL            dispFaceBottom();
    void            dispFaceBottom( BOOL bNewState );

    BOOL            dispFaceLeft();
    void            dispFaceLeft( BOOL bNewState );
                    // Manipulate the ON/OFF display attributes of 
                    // the Back and/or Left and/or Bottom chart-faces

    BOOL            dispForceBackBorder();
    void            dispForceBackBorder( BOOL bNewState );
                    // Force background rect poly-line to render 
                    // into the display.  Useful for "stubborn" 
                    // print-display contexts.  C-tor inits OFF.

    BOOL            dispForceBackColor();
    void            dispForceBackColor( BOOL bNewState );
                    // Force FILLED background rectangle to render 
                    // into the display.  Useful for "stubborn" 
                    // print-display contexts.  C-tor inits OFF.

    BOOL            dispWireFrameMode();
    void            dispWireFrameMode( BOOL bNewState );
                    // Wire-Frameing skips Hidden-Surface-Elimination
                    // and/or Depth-Sorting ops and so is MUCH more
                    // efficient when directly manipulating chart
                    // configurations (such as in dialogs, etc...)

    BOOL            dispZoomMode();
    void            dispZoomMode( BOOL bNewState );
                    // When Zoon-Mode is TRUE, the chart-engine IGNORES
                    // any user-specified margins when it calcs the
                    // display-context rendering projection-rectangle.

    virtual void    draw( bf_Display *p_bf_dspOutputDisplay );
    virtual void    draw2D( bf_Display *p_bf_dspOutputDisplay );
    virtual void    draw3D( bf_Display *p_bf_dspOutputDisplay );
                    // Self-explanatory.  Chart-Engines render into
                    // a display-context provided by their parent
                    // chart-model, which itself has a display-
                    // context provided by a parent chart-view.

    void            engineKernel( cf_Kernel *p_cf_kUseThisKernel );
    cf_Kernel*      engineKernel();
    cf_ChartModel*  engineModel();
                    // Any given chart-engine is USUALLY "borrowing"
                    // a geometry-kernel provided by a parent chart-
                    // model, but this member method provides even
                    // greater flexibility.  For example, some chart-
                    // format dialogs in the Advanced User Interface
                    // actually hot-swap the original chart-engine
                    // kernel for a temporary "scratch" geometry-
                    // kernel during the dialog...

    void            indexMethod( int    iIndexMethodID
                                        = CF_ID_YVALS_ON_NAT_NS );
    int             indexMethodIs();
                    // Mirror index-method(s) as per the chart-model
                    // object. See c1_cmodl.hpp for this spec.

    cf_TitleMarkerMgr*      
                    markerMgrTitlesAxes();
                    // Access the chart-engine "Text-Marker-Manager"
                    // object that can be used to control and access 
                    // the EXISTING set of title, sub-title, and
                    // axis-label 3D-Text-Markers.


    cf_3DTextMarkerSeries*      
                    markerMgrUserLegends();
                    // Access the chart-engine "3D-Text-Marker-Series"
                    // that can be used to create and control EXTRA
                    // user-specified 3D-text-markers for use as
                    // chart-legends, etc.


    BOOL    positPointAt(   cf_3DPoint  *p_cf_3dpTargetPoint,
                            double      dPctWidthFromCtr,
                            double      dPctHeightFromCtr,
                            double      dPctDepthFromCtr = 0.0,
                            BOOL        bCorrectOriginOffset = FALSE );
            // Allows user classes (e.g. axis grid/tick/series) to 
            // over-ride any default positioning of a 3D Point (e.g. 
            // Axis Label Text Marker Points) and specify a new posit
            // as an X/Y/Z % val away from the CENTER of the 3D Box 
            // of the chart.  Positive OR negative % vals are allowed.
            // Note that optional correction for default origin xlation
            // (e.g. when grid/tick-series are added to engines via
            // the "add-Component-Object(...)" method) is available, 
            // but is NOT the default mode of operation.

    //------------------------------------------------------------
    // The chart-engine "preference" methods below typically
    // pass their parameters directly to a corresponding member 
    // method of the geometry-kernel object associated with this
    // chart-engine, and are provided in this chart-engine object
    // basically for convenience...
    //------------------------------------------------------------
    void    preferMarginTop(    int iMarginInTwips );
    void    preferMarginBottom( int iMarginInTwips );
    void    preferMarginLeft(   int iMarginInTwips );
    void    preferMarginRight(  int iMarginInTwips );

    void    preferMargins(  int iTopMarginInTwips,
                            int iBottomMarginInTwips,
                            int iLeftMarginInTwips,
                            int iRightMarginInTwips );
            // Adds a margin in Twips (1/20 of a standard typographic
            // point, which is 1/72 of an English Inch) to the the 
            // display-context rendering projection-rectangle.
                            
    void    preferPerspective(  double  dPercentPerspective );
    void    preferPerspectiveMax();
    void    preferPerspectiveNormal();
    void    preferPerspectiveZero();
            // The "normal" chart-engine disply mode is ISOMETRIC
            // projection (i.e. zero-perspective).  However, any
            // given chart-engine has full visual perspective 
            // rendering capability.
               
    void    preferProportions(  double  dPrefHWPercentRatio,
                                double  dPrefDHPercentRatio,
                                BOOL    bPerformAutoConfigure = TRUE );
            // Sets the height-to-width and depth-to-height ratios
            // of the 3D-Box that the chart-engine renders its 
            // collection of objects into...

    void    preferViewAngle(    double  dSetXAngle, 
                                double  dSetYAngle );
            // Sets a view-angle used in 3D-display mode.  Note that
            // the y-view angle parameter should be NEGATIVE, as 
            // per a right-hand-rule/system rotation..,

    int             typeID();
    BOOL            typeBarColStack();
    BOOL            typeCartesian();
    BOOL            typePolar();
                    // Query methods yielding information on various
                    // chart-attributes.  The flags that yield these
                    // boolean values are typically set in the c-tor.

    void            updateDisplayMetrics( bf_Display *p_bf_dspHasChanged );
                    // Called by this chart-engine's parent chart-model
                    // and/or chart-view configuration in response to
                    // display context changes (e.g. re-sizes, re-sets).

    virtual void    updateFromCurrentModel();
                    // When a chart-model has a data-update, it will
                    // call this "umbrella" chart-engine method to
                    // tell the chart-engine to begin (re)generating
                    // its components and collection of virtual-
                    // geometry objects according to the current
                    // data-state of the chart-model.  Derived chart-
                    // engines (including customized derivations)
                    // should use their own specific over-rides
                    // of this virtual method.  This method might
                    // also call one or more protected (and possibly
                    // re-derived update() methods, (such as 
                    // updateYValsAtIntIs(), updateAxisGridTickSeries(),
                    // etc.) according to a given specific 
                    // implementation of a derived chart-engine.

    void            updateTextBackColors( const bf_Color 
                                                &cr_bf_clrNewColor );
                    // Updates the text-background colors used by
                    // any associated text-objects used within this
                    // chart-engine...


protected:  //===================================================
            // ChartFolio "Chart-Engine" classes derived from 
            // this virtual base class MIGHT need to be able to 
            // touch these vars & methods, but (non-expert) 
            // users should NOT muck around with this stuff...   
            //===================================================

    void    addCompoObject( cf_Virt3DObj *p_cf_objAddThisObject );
    void    addFaceObject( cf_Virt3DObj *p_cf_objAddThisObject );
    void    addSpaceObject( cf_Virt3DObj *p_cf_objAddThisObject );
    void    addMarkerObject( cf_Virt3DObj *p_cf_objAddThisObject );
            // As (derived) chart-engines generate 2D/3D geometric
            // chart-objects based on the data-state of the parent
            // chart-model, they call these methods to add these
            // newly-created objects into one of a set of dynamic 
            // arrays so they can be tracked for geometric 
            // transforms and projections, scaling and xlation, 
            // rendering ops, etc.

    //-------------------------------------------------------------
    // These *virtual* methods provide modular functional
    // implmentations that are OFTEN over-ridden for specific
    // and/or added functionality in derived chart-engine classes...
    //-------------------------------------------------------------
    virtual void    applyVirtScalings();
    virtual void    applyVirtTranslations();
                    // Modular functions used during the auto-configure
                    // phase of chart-engine update ops.  These guys
                    // use values calc-ed by the "calc-methods" below...
                   
    virtual void    calcVirtScaleFactors(); 
    virtual void    calcVirtTransFactors(); 
                    // These modular functions calc values used during
                    // the auto-configure phase of chart-engine update
                    // ops. These guys use values measured by the
                    // "measure-methods" below...

    virtual void    config3DBoxETC();
    virtual void    configOrigins(); 
                    // Modular sub-functions used in auto-configuration.
    
    virtual void    init3DBoxETC();
    virtual void    initDrawingAttribs();
    virtual void    initGridTickSeries();
    virtual void    initObjectDynamicStorage();
    virtual void    initTextMarkers();
                    // Modular initializers for use in c-tor(s)

    virtual void    measureDataMinMax();
    virtual void    measureVirtMinMax(); 
                    // These modular functions measure values used 
                    // during the auto-configure phase of chart-engine 
                    // update ops. These guys yield values used by the
                    // "calc-methods" above...

    virtual void    updateAxisGridTickSeries();
                    // Modular sub-method called by the chart-engine's
                    // updateFromChartModel() "master-update-method".
                    // These are VERY virtual -- all the "real" 
                    // work takes place in the derived engine types

    cf_3DInvisPoint cf_3dipEngObjVirtMin;
    cf_3DInvisPoint cf_3dipEngObjVirtMax;
    virtual void    updateAxisRangeBufferPoints();
                    // Modular sub-method called by the chart-engine's
                    // updateFromChartModel() "master-update-method".
                    // Implements axis range-buffering by updating 
                    // "invisible 3D-points" which track the virtual 
                    // max/min dimensions of the chart-engine and the
                    // objects therein...
                    
    virtual void    updateYValsOnNatNs();
    virtual void    updateYValsAtIntIs();
                    // Modular sub-method called by the chart-engine's
                    // updateFromChartModel() "master-update-method".
                    // These are VERY virtual -- all the "real" 
                    // work takes place in the derived engine types

    //====================================
    // Chart Engine "Type/Method-ID" vars:
    //====================================
    int     iEngineID;
            // A Chart "Type ID" lets us know what TYPE of
            // chart we are dealing with, e.g. Pie, Bar, 
            // Point Series, Line, etc. C-tor inits by 
            // "Chart ID" parameter, and current state is 
            // accessible through the typeID() member method.

    BOOL    bIsEngineCartesian;
    BOOL    bIsEnginePolar;
            // Once we get beyond this "base" class, charts 
            // will be either "Cartesian" or "Polar".  This 
            // may cause different requirements for dialogs/
            // displays/etc -- so we'll need some polling 
            // status vars.  Current status is available via 
            // typeCartesian() and/or typePolar() member methods.

    BOOL    bIsEngineBarColStack;
            // Bar/Col/Stack-type charts have *slightly* different
            // indexing requirements from "point"-type charts such 
            // as point series, line, and (integral) area charts, 
            // so it helps to keep track of this type/category.
    
    int     iIndexID;
            // Indexing method set by indexMethod() member.
            // C-tor inits default of CF_ID_YVALS_ON_NAT_NS

    //========================================
    // Declare "critical pointers" vars:
    //========================================
    cf_ChartModel   *p_cf_cmodlUserData;
                    // Pointer to this cf_ChartEngine's "parent/
                    // owner" Chart Model, which is responsible 
                    // for managing this cf_ChartEngine and a 
                    // set of User Data...

    cf_Kernel       *p_cf_kEngineKernel;
                    // Pointer to the geometry kernel which 
                    // controls the state of this chart/engine's 
                    // tramsformation geometry...

    //--------------------------------------------------------
    // Declare vars for the 3D Box Faces (and background):
    //--------------------------------------------------------
    cf_3DPolygon    cf_3dpgBackFace,
                    cf_3dpgLeftFace,
                    cf_3dpgBottomFace;
    
    cf_DispAttribs  *p_cf_datbBackFace,
                    *p_cf_datbLeftFace,
                    *p_cf_datbBottomFace;

    cf_DispAttribs  *p_cf_datbDispBack;

    BOOL        bDoWeDrawBackFace,
                bDoWeDrawLeftFace,
                bDoWeDrawBottomFace;
                            
    int         iPrefLeftFaceFontHeight,
                iPrefLeftFaceFontWidth,
                iPrefBackFaceFontHeight,
                iPrefBackFaceFontWidth,
                iPrefBottomFaceFontHeight,
                iPrefBottomFaceFontWidth;

    //---------------------------------------------
    // Declare vars for the Chart's Bitmap Wall:
    //---------------------------------------------
    cf_3DBitmap     *p_cf_3dbmpProjectedBitmap;
    BOOL            bDoWeDrawBitmapWall;

    //----------------------------------------
    // Declare vars for Grid/Tick Mark Series:
    //----------------------------------------
    cf_3DGridTickSeries *p_cf_3dgtsXonBackFace;
    cf_3DGridTickSeries *p_cf_3dgtsXonBottomFace;
    cf_3DGridTickSeries *p_cf_3dgtsYonLeftFace;
    cf_3DGridTickSeries *p_cf_3dgtsYonBackFace;
    cf_3DGridTickSeries *p_cf_3dgtsZonLeftFace;
    cf_3DGridTickSeries *p_cf_3dgtsZonBottomFace;
    cf_3DGridTickSeries *p_cf_3dgtsPolarOnFloor;

    //-----------------------------------------------------------
    // Declare a Chart Text-Marker-Manager that can be used to
    // control and access the existing title, sub-title, and
    // axis-label text markers:
    //-----------------------------------------------------------
    cf_TitleMarkerMgr       *p_cf_tmmTitlesAxes;

    //------------------------------------------------------------
    // Declare a Chart Text-Marker-Series Object that can be used 
    // to create and control EXTRA user-specified 3D-text-markers 
    // such as chart-legends, etc.
    //------------------------------------------------------------
    cf_3DTextMarkerSeries   *p_cf_3dtmsUserLegends;

    //------------------------------------------------------------
    // Declare vars for an (optional) BACKGROUND Display Bitmap:
    //------------------------------------------------------------
    cf_String          cf_strWallPaperFile;
    BOOL                bDoWeDrawWallPaper;
    bf_2DBitmap        *p_bf_2dbmpWallPaper;

    //---------------------------------------------------
    // Members for the chart origin and dimensions:
    //---------------------------------------------------
    double  d3DBoxXMin, d3DBoxXMax, d3DBoxYMin,
            d3DBoxYMax, d3DBoxZMin, d3DBoxZMax;
            // To be set = this->p_cf_kEngineKernel->getXMax(), etc.

    double  dChartXOrigin,  dChartYOrigin,  dChartZOrigin;
            // For cartestian/XYZ set equal to the far right corner,
            // for polar/pie, set equal to the middle of the floor.

    //------------------------------------------------------------
    // Members to hold the Chart Engine's cf_Virt3DObj Components:
    //------------------------------------------------------------
    cf_DyArr    da_p_cf_objCompoObjects;
    int         iNumCompoObjects;
                // This list is primarily for cf_ChartEngine-derived 
                // classes to add additional objects ONTO the chart, 
                // e.g. class Cartesian Engines, will add X,Y,Z axis 
                // grod/tick-mark series, etc.

    cf_DyArr    da_p_cf_objFaceObjects;
    int         iNumFaceObjects;
                // This list is primarily for DATA objects AGAINST 
                // THE FACES of the cf_ChartModel, e.g., Control 
                // Line objects, or the Marker-Line objects, 
                // that are ON the faces of the 3D-Box:

    cf_DyArr    da_p_cf_objSpaceObjects;
    int         iNumSpaceObjects;
                // This list is primarily for objects IN THE 3D SPACE of
                // the engine's 3D-Box, e.g. Ribbons, PolyHedraSeries, etc,

    cf_DyArr    da_p_cf_objMarkerObjects;
    int         iNumMarkerObjects;
                // This list is primarily for text-marker/chart-legend
                // objects projected ON TOP OF the 2D/3D Objects in
                // a 2D/3D Chart-Engine's 3D-Box.
         
    //------------------------------------------------------------- 
    // Member Vars to track this cf_ChartEngine's "DATA" (Data 
    // & Range) Mins/Maxes for Dynamic and/or Data-Dependent Ops:
    //------------------------------------------------------------- 
    double  dChartXRangeMin, dChartYRangeMin, dChartZRangeMin;
    double  dChartXRangeMax, dChartYRangeMax, dChartZRangeMax;
            // Keeping track of the original DATA values enables 
            // operations such as tick marks resets (e.g. dynamically, 
            // every time this Engine gets re-proportioned)...

    //------------------------------------------------------------ 
    // Member Vars to track this cf_ChartEngine's "VIRTUAL" (Data 
    // & Range) Mins/Maxes. for Dynamic Chart (re)Scaling Ops:
    //------------------------------------------------------------ 
    double  dVirtXRangeMin, dVirtYRangeMin, dVirtZRangeMin;
    double  dVirtXRangeMax, dVirtYRangeMax, dVirtZRangeMax;
            // It is essential to keep track of the virtual 
            // transformations resulting from operations such 
            // as translations, scalings, geometric transformations, 
            // etc of the  original DATA values, so that this 
            // cf_ChartEngine's cf_Virt3DObjs can be re-scaled 
            // correctly -- and dynamically -- with each Engine
            // re-proportioning event:

    //------------------------------------------------------
    // Member Vars for "User Preferences for Scale Ratios" 
    //------------------------------------------------------
    double  dPrefChartHWRatio, dPrefChartDHRatio;
            // The user can set these two ratios, and they will
            // control the other Scale Factors and Ratios below:

    double  dChartObjectXScaleFactor;
    double  dChartObjectYScaleFactor;
    double  dChartObjectZScaleFactor;
            // These are the ratios that this cf_ChartEngine's  
            //"virtual" cf_Virt3DObjs will actualy get scaled to.

    double  dChartObjectXTransFactor;
    double  dChartObjectYTransFactor;
    double  dChartObjectZTransFactor;

    //-------------------------------------------------------------
    // Member vars for "Run-Time Geometry/Draw States":
    //-------------------------------------------------------------
    BOOL    b3DMode;                        // C-tor inits to TRUE.
    BOOL    bInWireFrameMode;               // C-tor inits to FALSE.
    BOOL    bForceRenderOfBackColorRect;    // C-tor inits to FALSE.
    BOOL    bForceRenderOfBackBorder;       // C-tor inits to FALSE.

    BOOL    bHasEverRendered_;              // C-tor inits to FALSE.
                                            // First draw() op sets TRUE.

    double  dSaveXAngle, dSaveYAngle, dSaveZAngle, dSavePRatio;
            // When we switch to 2D mode, we'll need to
            // save view params so we can restore 3D mode.

    int     i,j,k;  // Std "scratch" index vars...
    
};  // End of cf_ChartEngine declaration.

#endif      // Terminates include re-definition protection

//////////////////////////////////////
// END of File: C1_ENGIN.HPP
//////////////////////////////////////
            