//////////////////////////////////////
// START of File: CE1_PCOR.HPP
//////////////////////////////////////
                                    
#ifndef CE1_PCOR_HPP            // Avoid multiple includes...
#define CE1_PCOR_HPP    TRUE    // ...holds to a terminal #endif below...
    
//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.txt" and "cf_licen.txt" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Master Header File Listing:
//
// #include    <cf_api/ce1_pcor.hpp>
//
// "Polar Coord" Chart Engine object (derived from base "Chart-Engine").
//

//--------------------------------------------------------------------
// Include Chart Framework class or ID defs/headers this file needs.
// Paths are relative to the IDE's (global) include path: ~/cfrz
//--------------------------------------------------------------------
class       bf_Display;
class       cf_3DPoint;
class       cf_ChartModel;
class       cf_ChartEngine;
#include    <cf_api/c1_engin.hpp>
            // which pulls in the classes above...
    
//
// Declare a "Polar Coord Engine" object:
//
// Class cf_eng_Polar models a Polar Coordinate Chart which
// will be used to produce derived Pie Charts, Radar Charts, etc.
//
class   cf_eng_Polar : 
        public 
        cf_ChartEngine 
{

public:

    cf_eng_Polar(   cf_ChartModel   *p_cf_cmodlOwner,
                    int             iChartTypeID,
                    int             iIndexMethodID
                                    = CF_ID_YVALS_ON_NAT_NS );
                                    // Refer to "Index Method" ID 
                                    // specs in base class method
                                    // cf_ChartEngine::indexMethod()

    virtual         ~cf_eng_Polar();

    virtual void    applyGeomXform();

    virtual void    applyProjXform();

    virtual void    config3DBoxETC();

            void    configOrigins();
                    // defined invariantly as the CENTER of 
                    // the BOTTOM FACE of the 3D Chart Box.

            void    disp2DMode( BOOL    bNewState );
                    // defined as thisd->disp3DMode( !bNewState );.
                    
            void    disp3DMode( BOOL    bNewState );
                    // 2D Mode defined invariantly to X = 90, 
                    // Y = 0, Z = 0, which makes the viewer 
                    // look DOWN at the BOTTOM Face.

    virtual void    draw2D( bf_Display *p_bf_dspFromChartView );

    virtual void    draw3D( bf_Display *p_bf_dspFromChartView );

   
protected:
    
    //---------------------------------------------------------
    // Polar Coord Charts will need to display/reference 
    // a reference/projection Ellipse/Circle, which is 
    // typically defined by a projected rectangular area, 
    // on the FLOOR (XZ Plane) (or possibly the WALLs 
    // (XY or YZ Planes) ) of the cf_ChartEngine's 3D Box.  
    //
    // To determine these rectangular areas, we will project 
    // the virtual boundaries of these circles onto the 
    // current display.
    //---------------------------------------------------------
    cf_3DPoint  cf_3dpXZPlaneEllipseVirtXMax,
                cf_3dpXZPlaneEllipseVirtXMin,
                cf_3dpXZPlaneEllipseVirtZMax,
                cf_3dpXZPlaneEllipseVirtZMin;
                // These points will mark the "polar coord circle"'s
                // virtual geometry ellipse boundaries (subject
                // to rotation effects, etc)...
            
    cf_3DPoint  cf_3dpXZPlaneEllipseProjXMax,
                cf_3dpXZPlaneEllipseProjXMin,
                cf_3dpXZPlaneEllipseProjZMax,
                cf_3dpXZPlaneEllipseProjZMin;
                // These points will mark the "polar coord circle"'s
                // projected ellipse boundaries, after correction
                // for rotation effects, etc
            
    cf_3DPoint  cf_3dpXZPlaneEllipseCenter;
                // We'll also keep track of our polar coord origin.
    

};  //
    // End of cf_eng_Polar declaration.
    //

#endif      // Terminates include re-definition protection

//////////////////////////////////////
// START of File: CE1_PCOR.HPP
//////////////////////////////////////


            