/////////////////////////////////////////
// START of File: B1_CVIEW.CPP
/////////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved.
//
// See files "cf_copyr.h" and "cf_licen.h" for details of
// copyright and license specifications, and contact information.
//
//===================================================================

/*
 *	b1_cview.cpp -- Zinc version
 */
#include        <cf_api/b1_cview.hpp>

bf_ChartView
    ::bf_ChartView(	ZafWindowObject *p_zwoParentForChartView,
                    int iChartTypeID, int iIndexMethodID )
{
    this->p_cf_cmodlTarget_ = new cf_ChartModel(    this,
                                                    iChartTypeID,
                                                    iIndexMethodID );
    this->bWasTargetNewed_ = TRUE;

    this->p_zwoForChartView_ = p_zwoParentForChartView;
    this->initialize();
    // Postcondition: bIsModelActivated_ = FALSE
    // to FORCE activaton/synchronization via this->activateModel()!
}


bf_ChartView
    ::bf_ChartView( ZafWindowObject *p_zwoParentForChartView,
                    cf_ChartModel   *p_cf_cmodlUseThisChart )
{
    this->p_cf_cmodlTarget_ =  p_cf_cmodlUseThisChart;
    this->bWasTargetNewed_ = FALSE;

    this->p_zwoForChartView_ = p_zwoParentForChartView;
    this->initialize();
    // Postcondition: bIsModelActivated_ = FALSE
    // to FORCE activaton/synchronization via this->activateModel()!
}


bf_ChartView
    ::~bf_ChartView()
{
    if ( this->bWasTargetNewed_ == TRUE )
        if ( this->p_cf_cmodlTarget_ != NULL )
            delete this->p_cf_cmodlTarget_;
}



void
    bf_ChartView
        ::activateModel( BOOL bForceActivationUpdate )
{
    // If we're not pointing off into the wild blue...
    if ( this->p_cf_cmodlTarget_ != NULL )
    {
        // Is it really neccessary to perform this update?
        if ( ( this->bIsModelActivated_ == FALSE )
            || ( bForceActivationUpdate == TRUE ) )
                // Or did the user DEMAND we do this?
        // Then make it so...
        {
            // Synchronize background color with the chart model/engine
            this->colorDispBack( p_cf_cmodlTarget_
                                        ->modelEngine()->colorDispBack() );

            // Synchronize the projection-state of the chart-model's
            // encapsulated geometry-kernel and chart-engine:
            this->p_cf_cmodlTarget_->updateDisplayMetrics( this->getDisplay() );

            // Force a chart-engine update from the current chart-model:
            this->p_cf_cmodlTarget_->updateEngineObjects( TRUE, TRUE, TRUE );
                // Force a chart-engine update from the current chart-model;
                // which also performs a default engine re-config-phase
                // and a default geom/proj-xform phases

            this->bIsModelActivated_ = TRUE;
                // ... i.e keep track that we ARE now activated
        }   // end of activation/synchronization activity...
    }   // end of screen for valid pointer context

}   // end of bf_ChartView::activateModel ()



void
    bf_ChartView
        ::colorDispBack ( bf_Color  bf_clrUseThisColor )
{
}


// Draw the chart that has been assigned to this bf_ChartView
// on a display (screen or printer context).  The objective is to
// pass the current display context to the client cf_ChartModel's
// draw() member method...
int
    bf_ChartView
        ::draw()
{
    // Make sure background color is up-to-date with the chart-engine:
    this->colorDispBack( p_cf_cmodlTarget_->modelEngine()->colorDispBack() );

    // Does user need to sync view/model with an activateModel ()?
    if ( this->bIsModelActivated_ == FALSE ) this->activateModel ();

    // Let the Model and it's Engine do the rest!
    this->p_cf_cmodlTarget_->draw ( this->getDisplay () );

    return 1;
}

void
    bf_ChartView
        ::dem_setDirty ()
{}

void
    bf_ChartView
        ::dem_show()
{ this->p_cf_cmodlTarget_->initiateDrawOp (); }

void
    bf_ChartView
        ::dem_show ( int /*iShowFlags*/ )
{}


bf_Display*
    bf_ChartView
        ::getDisplay ()
{ return( p_bf_dspForChartView_ ); }


void
	bf_ChartView
		::initialize()
{
    // Create a pointer to an abstracted display-context object:
	this->p_bf_dspForChartView_
                = new bf_Display(   p_zwoForChartView_->display,
                                    // A static member?
                                    this->p_zwoForChartView_ );

	// FORCE the configuration to automatically sync the view/model
    // to this new display with a later call to activateModel()
    this->bIsModelActivated_ = FALSE;

	// Synchronize background color with the chart model/engine
    this->colorDispBack ( this->p_cf_cmodlTarget_
                                    ->modelEngine ()->colorDispBack () );
}


BOOL
    bf_ChartView
        ::isModelActivated ()
            { return( this->bIsModelActivated_ ); }


int
    bf_ChartView
        ::processPrintJob ()
	        { return 1; }
	        // end of bf_ChartView::processPrintJob ()

int
    bf_ChartView
        ::print ()
{
    draw ();
    return 0;
}


void
    bf_ChartView
        ::setMetrics ()
    // Since the window MAY have re-sized, we'd better recalc
    // our chart's projection matrix, and anything dependent
    // upon such a state change:
    {
	this->p_cf_cmodlTarget_->updateDisplayMetrics ( this->getDisplay () );}
    // Attempt to make this method in-line...


int
    bf_ChartView
        ::size ()
    // To process a (re)size event, we're gonna need to recompute
    // the projection matrix based on the NEW window size.
	{ 
		this->setMetrics(); 
		// this->draw(); 
		return 1; 
	}
    // So now we have initiated a re-draw event...
    // We have also attempted to make this method compile as in-line...


cf_ChartModel*
    bf_ChartView
        ::viewModel()
            { return this->p_cf_cmodlTarget_; }

ZafWindowObject*
    bf_ChartView
        ::viewWindow()
            { return this->p_zwoForChartView_; }

void
    bf_ChartView
        ::viewWindow( ZafWindowObject *p_zwoParentForChartView )
{
    this->p_zwoForChartView_ = p_zwoParentForChartView;
	delete this->p_bf_dspForChartView_;
    this->initialize();
}

//========================================================================
// Start of Zinc ZAF v5.x ZafChart-specific API-interface/helper methods:
//========================================================================

cf_3DGridTickSeries*
    bf_ChartView
        ::zz_accessXAxisLower()
        { return( this->viewModel()->modelEngine()->axis_X_BottomFace() ); }

cf_3DGridTickSeries*
    bf_ChartView
        ::zz_accessYAxisLeft()
        { return( this->viewModel()->modelEngine()->axis_Y_LeftFace() ); }

cf_3DTextMarker*
    bf_ChartView
        ::zz_accessChartTitle()
        { return( this->viewModel()
                            ->modelEngine()->markerMgrTitlesAxes()
                                                        ->markerTitle() ); }

cf_DataGroup*
    bf_ChartView
        ::zz_addDataGroup( int iGroupIndex )
        { return( this->p_cf_cmodlTarget_->activateGroup( iGroupIndex ) ); }

cf_XYZ*
    bf_ChartView
        ::zz_addDataPoint( int iGroupIndex, double dDataValue )
{
    p_cf_cmodlTarget_->activateGroup(iGroupIndex);
	return( p_cf_cmodlTarget_->addDataYVal(dDataValue) );
}

cf_DataGroup*
    bf_ChartView
        ::zz_clearDataGroup( int iGroupIndex )
{
	cf_DataGroup *p_cf_dgToClear = this->p_cf_cmodlTarget_
                                    ->activateGroup(iGroupIndex);
	p_cf_dgToClear->reset();
	return p_cf_dgToClear;
}

cf_DataGroup*
    bf_ChartView
        ::zz_clearDataAll()
        { return( p_cf_cmodlTarget_->dataGroupsDelete()); }



ZafLogicalColor
    bf_ChartView
        ::zz_dataGroupBrushColor( int /*iGroupIndex*/ )
{
    return( ZAF_CLR_BLACK );
}

void
    bf_ChartView
        ::zz_dataGroupBrushColor( int iGroupIndex, ZafLogicalColor zlcNewColor )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

	// ...then update its abstracted "display-attribute" collection:
	p_cf_dgTarg->dispAttribs()->colorBrush( bf_Color( zlcNewColor ) );
}

void
    bf_ChartView
        ::zz_dataGroupBrushHatch(  int                     iGroupIndex,
                                ZafLogicalFillPattern   zlfpNewPat )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

	// ...then update its abstracted "display-attribute" collection:
	p_cf_dgTarg->dispAttribs()->styleBrushHatch( bf_BrushHatch( zlfpNewPat ) );
}

ZafLogicalFillPattern
    bf_ChartView
        ::zz_dataGroupBrushHatch(  int iGroupIndex )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

    bf_BrushHatch bf_bhTarg = p_cf_dgTarg->dispAttribs()->styleBrushHatch();

    // Convert the ChartFolio enum var to its equivalent Zinc definition
    ZafLogicalFillPattern  zlfpReturnVal  = (ZafLogicalFillPattern) bf_bhTarg;

    return( zlfpReturnVal );
}


void
    bf_ChartView
        ::zz_dataGroupFontSpec(  int             iGroupIndex,
                              ZafLogicalFont  /*zlfNewFont*/ )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

    //p_cf_dgTarg->textAttribs();//-> ???...;

    /* START BLOCK COMMENT
	cf_3DGridTickSeries
        *p_cf_gtsOnAxis
            = p_cf_cviewForChartModel_->viewModel()
                                    ->modelEngine()->axis_Y_LeftFace();

	cf_3DTextMarker *p_cf_3dtmAxisLabel = p_cf_gtsOnAxis->markerLabel();
	// We need to think about how to apply: ->markerTitle();
	// p_cf_gtsOnAxis->autoScaleFontSpec();
	// p_cf_3dtmAxisLabel->autoScaleFontSpec();
	// Let's plan this implementation w/ Zinc's crew...
	p_cf_3dtmAxisLabel->autoScaleFontSpec()->colorText(bf_Color(zlcNewColor));
    END BLOCK COMMENT */
}


ZafLogicalFont
    bf_ChartView
        ::zz_dataGroupFontSpec( int /*iGroupIndex*/ )
{ return( ZAF_FNT_DIALOG ); }


void
    bf_ChartView
        ::zz_dataGroupPenColor( int iGroupIndex, ZafLogicalColor zlcNewColor )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

	// ...then update its abstracted "display-attribute" collection:
	p_cf_dgTarg->dispAttribs()->colorPen( bf_Color( zlcNewColor ) );
}

ZafLogicalColor
    bf_ChartView
        ::zz_dataGroupPenColor( int /*iGroupIndex*/ )
{
    return( ZAF_CLR_BLACK );
}




void
    bf_ChartView
        ::zz_dataGroupPenStyle(  int                       iGroupIndex,
                              ZafLogicalLineStyle       zllsNewStyle )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

// BUGFIX

	// ...then update its abstracted "display-attribute" collection:
	p_cf_dgTarg->dispAttribs()->stylePen( bf_PenStyle( zllsNewStyle ) );
}

ZafLogicalLineStyle
    bf_ChartView
        ::zz_dataGroupPenStyle(  int iGroupIndex )
{
	// Access the data-group...
	cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );

    bf_PenStyle bf_psTarg = p_cf_dgTarg->dispAttribs()->stylePen();

    // Convert the ChartFolio enum var to its equivalent Zinc definition
    ZafLogicalLineStyle  zllsReturnVal  = (ZafLogicalLineStyle) bf_psTarg;

    return( zllsReturnVal );
}


cf_DataGroup*
    bf_ChartView
        ::zz_getDataGroup(   int iGroupIndex )
        { return( this->viewModel()->activateGroup( iGroupIndex ) ); }

cf_XYZ*
    bf_ChartView
        ::zz_getDataXYZ(     int iGroupIndex,
                          int iDataIndex )
{
    cf_DataGroup *p_cf_dgTarg = this->viewModel()->activateGroup( iGroupIndex );
    return( p_cf_dgTarg->activateXYZValAt( iDataIndex ) );

}


const ZafIChar*
    bf_ChartView
     ::zz_getTextTitle()
    { return( this->zz_accessChartTitle()->labelIs() ); }

const ZafIChar*
    bf_ChartView
     ::zz_getTextXAxisLower()
{
    cf_3DTextMarker *p_cf_3dtmTarget
                    = this->zz_accessXAxisLower()->markerLabel();
    return p_cf_3dtmTarget->labelIs();
}

const ZafIChar*
    bf_ChartView
     ::zz_getTextYAxisLeft()
{
    cf_3DTextMarker *p_cf_3dtmTarget
                    = this->zz_accessYAxisLeft()->markerLabel();
    return p_cf_3dtmTarget->labelIs();
}

bool
    bf_ChartView
        ::zz_getXAxisLower()
{
    bool bCurrState = ( (bool) ( this->zz_accessXAxisLower()->isActive() )  );
    return( bCurrState );
}

bool
    bf_ChartView
        ::zz_getYAxisLeft()
{
    bool bCurrState = ( (bool) ( this->zz_accessYAxisLeft()->isActive() )  );
    return( bCurrState );
}

ZafLogicalColor
    bf_ChartView
        ::zz_getXYAxisColor()
        { return( ZAF_CLR_BLACK ); }


void
    bf_ChartView
        ::zz_setChartType( int iChartTypeID )
{
    int iCurrIndexMode = this->viewModel()->dataIndexMethod();
    this->viewModel()->modelEngine( iChartTypeID, iCurrIndexMode, TRUE );
}                          // ...which will DELETE the old chart-engine!

void
    bf_ChartView
        ::zz_setIndexMethod( int iIndexMethodID )
        { this->viewModel()->dataIndexMethod( iIndexMethodID ); }

void
    bf_ChartView
        ::zz_setTextTitle( const ZafIChar* cp_zicUserString )
    { this->zz_accessChartTitle()->labelNew( (const char*) cp_zicUserString );}

void
    bf_ChartView
     ::zz_setTextXAxisLower( const ZafIChar* cp_zicUserString )
{
    cf_3DTextMarker *p_cf_3dtmTarget
                    = this->zz_accessXAxisLower()->markerLabel();
    p_cf_3dtmTarget->labelNew( (const char*) cp_zicUserString );
}

void
    bf_ChartView
     ::zz_setTextYAxisLeft( const ZafIChar* cp_zicUserString )
{
    cf_3DTextMarker *p_cf_3dtmTarget
                    = this->zz_accessYAxisLeft()->markerLabel();
    p_cf_3dtmTarget->labelNew( (const char*) cp_zicUserString );
}


ZafLogicalColor
    bf_ChartView
        ::zz_setXYAxisColor( ZafLogicalColor zlcNewColor )
{
    bf_Color    bf_clrNew( zlcNewColor );
    cf_3DTextMarker *p_cf_3dtmTarget = NULL;

    p_cf_3dtmTarget = this->zz_accessXAxisLower()->markerLabel();
    p_cf_3dtmTarget->dispAttribsCurrent()->colorPen( bf_clrNew );

    p_cf_3dtmTarget = this->zz_accessYAxisLeft()->markerLabel();
    p_cf_3dtmTarget->dispAttribsCurrent()->colorPen( bf_clrNew );

    return zlcNewColor; // Dicuss???
}


bool
    bf_ChartView
        ::zz_setXAxisLower( bool bOnOff )
{
    bool bOldState = ((bool) this->zz_accessXAxisLower()->isActive() );
    BOOL bNewState = (BOOL) bOnOff;
    this->zz_accessXAxisLower()->isActive( bNewState );
    return bOldState;
}

bool
    bf_ChartView
        ::zz_setYAxisLeft( bool bOnOff )
{
    // BUGFIX?
    bool bOldState = ((bool) this->zz_accessYAxisLeft()->isActive() );
    BOOL bNewState = (BOOL) bOnOff;
    this->zz_accessYAxisLeft()->isActive( bNewState );
    return bOldState;
}

//=====================================================================
// End of Zinc ZAF v5.x ZafChart-specific API-interface/helper methods.
//=====================================================================


// End of bf_ChartView Implementation

/////////////////////////////////////////
// END of File: B1_CVIEW.CPP
/////////////////////////////////////////

