//	Zinc Application Framework - BZ_ZCHT.CPP
//	COPYRIGHT (C) 1990-1997.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#include <cf_api/bz_zcht.hpp>
ZafClassID ZafChart::classID = ID_ZAF_CHART;
ZafClassNameChar ZAF_FARDATA ZafChart::className[] = ZAF_ITEXT("ZafChart");
            
ZafChart::ZafChart(int left, int top, int width, int height,
	int iChartTypeID, int iIndexMethodID) :
	ZafChartStub(left, top, width, height, iChartTypeID, iIndexMethodID)
{
	SetNoncurrent(true);
	SetOSDraw(false);
	this->p_cf_cviewForChartModel_ = new bf_ChartView(this, iChartTypeID, iIndexMethodID);
}

ZafChart::ZafChart(const ZafChart &copy) :
	ZafChartStub(copy)
{
	SetNoncurrent(true);
	SetOSDraw(false);
	this->p_cf_cviewForChartModel_ = new bf_ChartView(this, chartType, indexMethod);
	SetAxisColor(axisColor);
	SetMarksXAxis(marksXAxis);
	SetMarksYAxis(marksYAxis);
	SetTextTitle(textTitle);
	SetTextXAxis(textXAxis);
	SetTextYAxis(textYAxis);
}

ZafChart::~ZafChart(void)
{
	if (this->p_cf_cviewForChartModel_ != NULL)
		delete this->p_cf_cviewForChartModel_;
}

// --- Zinc to ChartFolio Data Method Mappings ---

ZafLogicalColor ZafChart::DataGroupBackgroundColor(int groupIndex, ZafLogicalColor *)
{
	return (p_cf_cviewForChartModel_->zz_dataGroupBrushColor(groupIndex));
}

ZafLogicalFillPattern ZafChart::DataGroupFillPattern(int groupIndex) const
{
	return (p_cf_cviewForChartModel_->zz_dataGroupBrushHatch(groupIndex));
}

ZafLogicalColor ZafChart::DataGroupForegroundColor(int groupIndex, ZafLogicalColor *)
{
	return (p_cf_cviewForChartModel_->zz_dataGroupPenColor(groupIndex));
}

ZafLogicalFont ZafChart::DataGroupFont(int groupIndex) const
{
	return (p_cf_cviewForChartModel_->zz_dataGroupFontSpec(groupIndex));
}

ZafLogicalLineStyle ZafChart::DataGroupLineStyle(int groupIndex) const
{
	return (p_cf_cviewForChartModel_->zz_dataGroupPenStyle(groupIndex));
}

ZafError ZafChart::AddDataGroup(int groupIndex)
{
	return (p_cf_cviewForChartModel_->zz_addDataGroup(groupIndex) ?
		ZAF_ERROR_NONE : ZAF_ERROR_INVALID);
}

ZafError ZafChart::AddDataPoint(int groupIndex, double dataValue)
{
	return (p_cf_cviewForChartModel_->zz_addDataPoint(groupIndex, dataValue) ?
		ZAF_ERROR_NONE : ZAF_ERROR_INVALID);
}

ZafError ZafChart::ClearDataAll(void)
{
	return (p_cf_cviewForChartModel_->zz_clearDataAll() ?
		ZAF_ERROR_NONE : ZAF_ERROR_INVALID);
}

ZafError ZafChart::ClearDataGroup(int groupIndex)
{
	return (p_cf_cviewForChartModel_->zz_clearDataGroup(groupIndex) ?
		ZAF_ERROR_NONE : ZAF_ERROR_INVALID);
}

ZafLogicalColor ZafChart::SetDataGroupBackgroundColor(int groupIndex, ZafLogicalColor color, ZafLogicalColor )
{
	ZafLogicalColor zlcReturnVal = p_cf_cviewForChartModel_->zz_dataGroupBrushColor(groupIndex);
	p_cf_cviewForChartModel_->zz_dataGroupBrushColor(groupIndex, color);
	if (AutomaticUpdate())
		Redisplay();
	return (zlcReturnVal);
}

ZafLogicalFillPattern ZafChart::SetDataGroupFillPattern(int groupIndex, ZafLogicalFillPattern fillPattern)
{
	ZafLogicalFillPattern zlfpReturnVal = p_cf_cviewForChartModel_->zz_dataGroupBrushHatch(groupIndex);
	p_cf_cviewForChartModel_->zz_dataGroupBrushHatch(groupIndex, fillPattern);
	if (AutomaticUpdate())
		Redisplay();
	return (zlfpReturnVal);
}

ZafLogicalColor ZafChart::SetDataGroupForegroundColor(int groupIndex, ZafLogicalColor color, ZafLogicalColor )
{
	ZafLogicalColor zlcReturnVal = p_cf_cviewForChartModel_->zz_dataGroupPenColor(groupIndex);
	p_cf_cviewForChartModel_->zz_dataGroupPenColor(groupIndex, color);
	if (AutomaticUpdate())
		Redisplay();
	return (zlcReturnVal);
}

ZafLogicalFont ZafChart::SetDataGroupFont(int groupIndex, ZafLogicalFont font)
{
	ZafLogicalFont zlfReturnVal = p_cf_cviewForChartModel_->zz_dataGroupFontSpec(groupIndex);
	p_cf_cviewForChartModel_->zz_dataGroupFontSpec(groupIndex, font);
	if (AutomaticUpdate())
		Redisplay();
	return (zlfReturnVal);
}

ZafLogicalLineStyle ZafChart::SetDataGroupLineStyle(int groupIndex, ZafLogicalLineStyle lineStyle)
{
	ZafLogicalLineStyle zllsReturnVal = p_cf_cviewForChartModel_->zz_dataGroupPenStyle(groupIndex);
	p_cf_cviewForChartModel_->zz_dataGroupPenStyle(groupIndex, lineStyle);
	if (AutomaticUpdate())
		Redisplay();
	return (zllsReturnVal);
}

// --- Optional entry points into ChartFolio ---

cf_ChartModel *ZafChart::ActiveModel(void)
{
	return (p_cf_cviewForChartModel_->viewModel());
}

cf_DataGroup *ZafChart::DataGroup(int groupIndex)
{
	return (p_cf_cviewForChartModel_->zz_getDataGroup(groupIndex));
}

cf_XYZ *ZafChart::DataPoint(int groupIndex, int dataIndex)
{
	return (p_cf_cviewForChartModel_->zz_getDataXYZ(groupIndex, dataIndex));
}

// --- Attributes ---

ZafLogicalColor ZafChart::SetAxisColor(ZafLogicalColor color, ZafLogicalColor mono)
{
	ZafChartStub::SetAxisColor(color, mono);
	p_cf_cviewForChartModel_->zz_setXYAxisColor(color);
	if (AutomaticUpdate())
		Redisplay();
	return (ZAF_CLR_NULL);
}

int ZafChart::SetChartType(int chartTypeID)
{
	p_cf_cviewForChartModel_->zz_setChartType(chartTypeID);

	// ...now -- FORCE a chart-model/chart-view re-synchronization...
	p_cf_cviewForChartModel_->activateModel(TRUE);

	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetChartType(p_cf_cviewForChartModel_->viewModel()->modelEngine()->typeID()));
}

int ZafChart::SetIndexMethod(int indexMethodID)
{
	p_cf_cviewForChartModel_->zz_setIndexMethod(indexMethodID);

	// ...now -- FORCE a chart-model/chart-view re-synchronization...
	p_cf_cviewForChartModel_->activateModel(TRUE);

	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetIndexMethod(p_cf_cviewForChartModel_->viewModel()->dataIndexMethod()));
}

bool ZafChart::SetMarksXAxis(bool xAxisMarks)
{
	p_cf_cviewForChartModel_->zz_setXAxisLower(xAxisMarks);
	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetMarksXAxis(p_cf_cviewForChartModel_->zz_getXAxisLower()));
}

bool ZafChart::SetMarksYAxis(bool yAxisMarks)
{
	p_cf_cviewForChartModel_->zz_setYAxisLeft(yAxisMarks);
	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetMarksYAxis(p_cf_cviewForChartModel_->zz_getYAxisLeft()));
}

const ZafIChar *ZafChart::SetTextTitle(const ZafIChar *text)
{
	p_cf_cviewForChartModel_->zz_setTextTitle(text);
	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetTextTitle(p_cf_cviewForChartModel_->zz_getTextTitle()));
}

const ZafIChar *ZafChart::SetTextXAxis(const ZafIChar *text)
{
	p_cf_cviewForChartModel_->zz_setTextXAxisLower(text);
	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetTextXAxis(p_cf_cviewForChartModel_->zz_getTextXAxisLower()));
}

const ZafIChar *ZafChart::SetTextYAxis(const ZafIChar *text)
{
	p_cf_cviewForChartModel_->zz_setTextYAxisLeft(text);
	if (AutomaticUpdate())
		Redisplay();
	return (ZafChartStub::SetTextYAxis(p_cf_cviewForChartModel_->zz_getTextYAxisLeft()));
}

// --- Window object members ---

ZafEventType ZafChart::Draw(const ZafEventStruct& event, ZafEventType ccode)
{
	// First render the window object portion.
	ZafRegionStruct drawRegion = BeginDraw();
	if (bordered)
    	DrawBorder(drawRegion, ccode);
	DrawBackground(drawRegion, ccode);

	// Now render the encapsulated child chart-view object (and
	// thus the chart-view's chart-model, and chart-engine objects).
	p_cf_cviewForChartModel_->draw();
	// Note that we are relying on the initialization of the
	// child chart-view's display-context pointer set up during
	// the child chart-view's constructor!

	EndDraw();
	return (event.type);
}

ZafWindowObject *ZafChart::Duplicate(void)
{
	return (new ZafChart(*this));
}

ZafEventType ZafChart::Event(const ZafEventStruct& event)
{
	ZafEventType ccode = event.type;

	switch (ccode)
	{
	case N_SIZE:
		// Let the chart-view/chart-model process the re-size.
		p_cf_cviewForChartModel_->size();
		// Now force the ZafChart to redisplay.
		Redisplay();
		break;

	default:
		ccode = ZafChartStub::Event(event);
		break;
	}
	return (ccode);
}

// --- Persistence ---

ZafChart::ZafChart(const ZafIChar *name, ZafObjectPersistence &persist) :
	ZafChartStub(name, persist.PushLevel(className, classID, ZAF_PERSIST_DIRECTORY))
{
	persist.PopLevel();

	this->p_cf_cviewForChartModel_ = new bf_ChartView(this, chartType, indexMethod);
	SetAxisColor(axisColor);
	SetMarksXAxis(marksXAxis);
	SetMarksYAxis(marksYAxis);
	SetTextTitle(textTitle);
	SetTextXAxis(textXAxis);
	SetTextYAxis(textYAxis);
}

ZafElement *ZafChart::Read(const ZafIChar *name, ZafObjectPersistence &persist)
{
	return (new ZafChart(name, persist));
}

void ZafChart::Write(ZafObjectPersistence &persist)
{
	ZafChartStub::Write(persist.PushLevel(className, classID, ZAF_PERSIST_DIRECTORY));
	persist.PopLevel();
}
