//////////////////////////////////////
// START of File: C1_CMODL.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved.
//
// See files "cf_copyr.h" and "cf_licen.h" for details of
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/c1_cmodl.hpp>

//
// Include any other files which reference
// neccessary #includes, #defines, defintions,
// typedefs, class declarations, etc:
//
#include    <math.h>
            // for floor() and ceil() calls below...

#define     C1_CMODL_DGROU_GROWBY       16
            // Dynamic Array of Data Groups "Grow-By" val

//
//============================================================
// Implement class cf_ChartModel.  Note that complete
// implementation is version-dependent and is completed in
// ONE of two source files: "c2_cmodl.cpp" or "c3_cmodl.cpp"
//============================================================
//
cf_ChartModel
    ::cf_ChartModel(    bf_ChartView    *p_bf_cviewOwner,
                        int             iChartTypeID,
                        int             iIndexMethodID    )
{
    //--------------------------------------------------------------
    // Since the Chart Framework is Data-Driven, and "Data-Model
    // Centric", we MUST init the Chart's user data value storage
    // (that engines, grid/tick/series, etc. will be referencing)!
    //--------------------------------------------------------------
    this->da_p_cf_dgDataGroups.growBy( C1_CMODL_DGROU_GROWBY );
    this->iNumAllocatedGroups = 0;
    this->iMaxGroupIndex = 0;
    this->iMinGroupIndex = 0;
    this->iActiveIndex = 0;
        // Just for now.  We'll call this->activateGroup( 1 ) below...

    this->p_cf_dgGroupsList = new cf_DataGroup();
    this->p_cf_dgDomainItems = new cf_DataGroup();
    this->p_cf_dgRangeValues = new cf_DataGroup();

    this->p_cf_dgGroupsList->labelNew( "Groups" );
    this->p_cf_dgDomainItems->labelNew( "Domain" );
    this->p_cf_dgRangeValues->labelNew( "Range" );

    //------------------------------------------------
    // Init default attributes (for rendering, etc)
    //------------------------------------------------
    this->paletteColors();      // Init with the DEFAULT colors!
    this->paletteMarkers2D3D(   FALSE,      // No 2D Markers
                                FALSE,      // No 3D Markers
                                FALSE );    // And DON'T re-apply!
    this->palettePenStyles( CF_PS_SOLID, CF_PS_DASH, CF_PS_DOT );
    this->paletteBrushHatches( CF_BHAT_NONE, CF_BHAT_VERT,  CF_BHAT_HORZ );
                    // NOTE -- we MUST load the chart palettes
                    // BEFORE any activateGroup( ) calls!

    //---------------------------------------------
    // Init Data/Range/Origin Inclusion Treatment:
    //---------------------------------------------
    this->bDataRangeInclOrigAlways = FALSE;
    this->dNumDataDeltasForOrigInclus = 3.0;

    //--------------------------------------------
    // Init Data-Range Calculation modifier-vars:
    //--------------------------------------------
    this->dBufferRangeMinLimitBy = this->dBufferRangeMaxLimitBy = 0.0;
    this->bRangeMinLimitUsesFloor = this->bRangeMaxLimitUsesCeil = FALSE;

    this->dCandidateRangeMaxLimit = this->dCandidateRangeMinLimit = 0.0;
    this->bRangeMaxLimitUsesCand = this->bRangeMinLimitUsesCand = FALSE;

    //----------------------------------------------
    // Avoid any "ambiguous" state by setting up at
    // least ONE "default" user data group:
    //----------------------------------------------
    this->activateGroup( 1 );
    this->bUpdatedChartData = FALSE;
            // but note that we HAVE NOT added any data yet...

    //---------------------------------------------------------------
    // Now we can init or new/create other critical pointers vars:
    //---------------------------------------------------------------
    this->p_bf_cviewParent = p_bf_cviewOwner;
            // Set up a pointer to an (already-created, usually
            // by an "owning" window/frame object) "Chart View"
            // which provides: (1) display event management,
            // e.g. for "draw" events, and mouse events, (2)
            // a display context for the kernel to perform
            // 3D-to-2D projection transformation calculations,
            // and (3) the actual chart display rendering context...

    this->p_cf_kChartKernel = new cf_Kernel();
            // The kernel controls the state of this chart's
            // (and its engine's) tramsformation geometry...

    this->p_cf_engChartEngine = NULL;
            // This NULL init is important for the chartEngine(...)
            // call immediately below that configs a run-time engine...

    this->modelEngine( iChartTypeID, iIndexMethodID, TRUE );
            // The engine controls the "display model" of
            // this chart's data... The TRUE param deletes
            // any previous cf_ChartEngine* allocation

    this->dataIndexMethod(  iIndexMethodID, // Set the index method
                            FALSE );        // but do NOT run an update!
    return;

}   // end of view-owned c-tor



cf_ChartModel
    ::cf_ChartModel(    int iChartTypeID,
                        int iIndexMethodID    )
{
    //--------------------------------------------------------------
    // Since the Chart Framework is Data-Driven, and "Data-Model
    // Centric", we MUST init the Chart's user data value storage
    // (that engines, grid/tick/series, etc. will be referencing)!
    //--------------------------------------------------------------
    this->da_p_cf_dgDataGroups.growBy( C1_CMODL_DGROU_GROWBY );
    this->iNumAllocatedGroups = 0;
    this->iMaxGroupIndex = 0;
    this->iMinGroupIndex = 0;
    this->iActiveIndex = 0;
        // Just for now. We'll call this->activateGroup( 1 ) below...

    this->p_cf_dgGroupsList = new cf_DataGroup();
    this->p_cf_dgDomainItems = new cf_DataGroup();
    this->p_cf_dgRangeValues = new cf_DataGroup();

    this->p_cf_dgGroupsList->labelNew( "Groups" );
    this->p_cf_dgDomainItems->labelNew( "Domain" );
    this->p_cf_dgRangeValues->labelNew( "Range" );

    //------------------------------------------------
    // Init default attributes (for rendering, etc)
    //------------------------------------------------
    this->paletteColors();      // Init with the DEFAULT colors!
    this->paletteMarkers2D3D(   FALSE,      // No 2D Markers
                                FALSE,      // No 3D Markers
                                FALSE );    // And DON'T re-apply!
    this->palettePenStyles( CF_PS_SOLID, CF_PS_DASH, CF_PS_DOT );
    this->paletteBrushHatches( CF_BHAT_NONE, CF_BHAT_VERT,  CF_BHAT_HORZ );
                    // NOTE -- we MUST load the chart palettes
                    // BEFORE any activateGroup( ) calls!

    //---------------------------------------------
    // Init Data/Range/Origin Inclusion Treatment:
    //---------------------------------------------
    this->bDataRangeInclOrigAlways = FALSE;
    this->dNumDataDeltasForOrigInclus = 3.0;

    //--------------------------------------------
    // Init Data-Range Calculation modifier-vars:
    //--------------------------------------------
    this->dBufferRangeMinLimitBy = this->dBufferRangeMaxLimitBy = 0.0;
    this->bRangeMinLimitUsesFloor = this->bRangeMaxLimitUsesCeil = FALSE;

    this->dCandidateRangeMaxLimit = this->dCandidateRangeMinLimit = 0.0;
    this->bRangeMaxLimitUsesCand = this->bRangeMinLimitUsesCand = FALSE;

    //----------------------------------------------
    // Avoid any "ambigous" state by setting up at
    // least ONE "default" user data group:
    //----------------------------------------------
    this->activateGroup( 1 );
    this->bUpdatedChartData = FALSE;
            // but note that we HAVE NOT added any data yet...

    //---------------------------------------------------------------
    // Now we can init or new/create other critical pointers vars:
    //---------------------------------------------------------------
    this->p_bf_cviewParent = NULL;
            // Note VERY well that -- in THIS C-tor -- we have
            // NOT assigned the "Chart View" which provides:
            // (1) display event management, e.g. for "draw"
            // events, and mouse events, (2) a display context
            // for the kernel to perform 3D-to-2D projection
            // transformation calculations, and (3) the actual
            // chart display rendering context...

    this->p_cf_kChartKernel = new cf_Kernel();
            // The kernel controls the state of this chart's
            // (and its engine's) tramsformation geometry...

    this->p_cf_engChartEngine = NULL;
            // This NULL init is important for the chartEngine(...)
            // call immediately below that configs a run-time engine...

    this->modelEngine( iChartTypeID, iIndexMethodID, TRUE );
            // The engine controls the "display model" of
            // this chart's data... The TRUE param deletes
            // any previous cf_ChartEngine* allocation

    this->dataIndexMethod(  iIndexMethodID, // Set the index method
                            FALSE );        // but do NOT run an update!
    return;

}   // end of model-w.o.-a--view-assigned c-tor



// Destructor must clean up any extra resource allocations.
cf_ChartModel
    ::~cf_ChartModel()
{
    // Delete anything we "newed"/allocated -- BUT BE CAREFUL!
    if ( this->p_cf_kChartKernel != NULL)
                            delete this->p_cf_kChartKernel;
                            // first we made a kernel...

    if ( this->p_cf_engChartEngine != NULL)
                            delete this->p_cf_engChartEngine;
                            // then we made an engine...

    if ( this->p_cf_dgGroupsList != NULL )
                            delete this->p_cf_dgGroupsList;
    if ( this->p_cf_dgDomainItems != NULL )
                            delete this->p_cf_dgDomainItems;
    if ( this->p_cf_dgRangeValues != NULL )
                            delete this->p_cf_dgRangeValues;
                            // then we made some extra data groups...

    // Then the users new()ed some data groups (via "activate" calls)...
    for ( i = 1; i <= this->iNumAllocatedGroups; i++ )
                delete ( (cf_DataGroup*) this->da_p_cf_dgDataGroups[ i ]);
}


//----------------------------------------------------
// Allow users to access the implicit "domain",
// "range", and "group-list" data-groups this
// model constructs from user data.
//----------------------------------------------------
cf_DataGroup*   
    cf_ChartModel
        ::accessGroupsList()
            { return this->p_cf_dgGroupsList; }
cf_DataGroup*   
    cf_ChartModel
        ::accessDomainItems()
            { return this->p_cf_dgDomainItems; }
cf_DataGroup*   
    cf_ChartModel
        ::accessRangeValues()
            { return this->p_cf_dgRangeValues; }


//--------------------------------------------------
// Users interact with (e.g. add data to) 
// a pointer to the current "active" group 
// in a "dynamic array" of data groups.
//--------------------------------------------------
cf_DataGroup*   
    cf_ChartModel
        ::activateGroup(    int         iGroupIndex,
                            const char* a_cDataGroupLabel )
{
    // Make a TENTATIVE Index Assignment:
    int iValidateUserIndex = iGroupIndex;

    // Correct Bogus Index Values that are TOO LOW:
    if ( iValidateUserIndex < 1 ) iValidateUserIndex = 1;
    
    // "Fill-In" Index Values that are TOO HIGH:
    if ( iValidateUserIndex > this->iMaxGroupIndex )
    {
        cf_DataGroup    *p_cf_dgBackFill = NULL;
                        // We'll new() Data Groups to "Back Fill" 
                        // the "missing" dynamic array ptr vals!...
                        
        for ( i = ( this->iMaxGroupIndex + 1 );  
                                    i <= iValidateUserIndex;  i++ )
        {   // Make a new "INACTIVE" data-group...
            p_cf_dgBackFill = new cf_DataGroup();
                 //...with "DEFAULT" display-attribs
            this->da_p_cf_dgDataGroups[ i ] = p_cf_dgBackFill;
                //...take responsibility for it...
            this->iNumAllocatedGroups++;    
        }       //...and account for this new allocation.
    }
    
    // OK, we've got a validated index..
    this->iActiveIndex = iValidateUserIndex;
                // into our dynamic array of POINTERS to Data Groups

    // Get the address of this (newly OR re) activated data group:
    cf_DataGroup    *p_cf_dgActivated = ( (cf_DataGroup*) 
                    this->da_p_cf_dgDataGroups[ this->iActiveIndex ] ); 

    // Has this group been used before?
    if ( p_cf_dgActivated->active() == FALSE )
    {
        // Then activate this new group!...
        p_cf_dgActivated->on(); 
        
        // Set the data group's label:
        if ( a_cDataGroupLabel != NULL )
        {   // From either the user-supplied string param...
            p_cf_dgActivated->labelNew( a_cDataGroupLabel );

            cf_XYZ  *p_cf_xyzDataGroup
                    = this->p_cf_dgGroupsList
                                ->addXYZVal( 0.0, 0.0, iActiveIndex );
            p_cf_xyzDataGroup->labelNew( a_cDataGroupLabel );
        }
        else
        {   // ... or generate a default label:
            cf_String   cfsDefaultLabel( "Group " );
            cfsDefaultLabel.cf_append( this->iActiveIndex, "%d" );
            p_cf_dgActivated->labelNew( (char*) cfsDefaultLabel );
        }   // which can be changed (later) by any user's engine...

        //------------------------------------------------------
        // Now set up DEFAULT Display attributes from the set
        // of Attribute "Palettes" this Chart Model maintains:
        //------------------------------------------------------
        cf_DispAttribs  *p_cf_datbNewGroup
                        = p_cf_dgActivated->dispAttribs();
                        
        p_cf_datbNewGroup->colorPen( CF_BLACK );// a good default choice
        p_cf_datbNewGroup->colorBrush( this->paletteColor( iGroupIndex ));
                    // which gets mod'ed to a palette color selection.
        p_cf_datbNewGroup->markWhen2D( this->paletteMarkers2D3D( 2 ) );
        p_cf_datbNewGroup->markWhen3D( this->paletteMarkers2D3D( 3 ) );
                    // 2D/3D marking mode (for poly-lines, etc) now set.
    }
    
    // Update Max Group Index Val?
    if ( this->iActiveIndex > this->iMaxGroupIndex )
        this->iMaxGroupIndex = this->iActiveIndex;
    
    // Update Min Group Index Val?
    if ( this->iMinGroupIndex == 0 ) 
            this->iMinGroupIndex = this->iActiveIndex;
    else
        if ( this->iActiveIndex < this->iMinGroupIndex ) 
                    this->iMaxGroupIndex = this->iActiveIndex;

    // Return the address of this current active group:
    return  p_cf_dgActivated; 
            // Which users can use as a pointer l-value:

}   // end of cf_ChartModel::activateGroup( ... )



//==========================================================
// Add a (y) data item to the current "activated" (z) group.
//==========================================================
cf_XYZ* 
    cf_ChartModel
        ::addDataYVal( double      dUserYVal,
                       const char* a_cDomainItemLabel,
                       const char* a_cRangeValueLabel )
{
    cf_XYZ  *p_cf_xyzReturnVal = NULL;
    
    switch ( this->iIndexID )
    {
        case CF_ID_YVALS_AT_INT_IS:
             p_cf_xyzReturnVal 
                = this->addDataYValIndexIntI( dUserYVal,
                                              a_cDomainItemLabel,
                                              a_cRangeValueLabel );
            break;
        
        case CF_ID_YVALS_ON_NAT_NS:
        case CF_ID_YVALS_AT_USR_XS:
        default:
             p_cf_xyzReturnVal 
                = this->addDataYValIndexNatN( dUserYVal,
                                              a_cDomainItemLabel,
                                              a_cRangeValueLabel );
            break;

    }

    return p_cf_xyzReturnVal;    
}


//============================================================
// Add a (x,y) data item to the current "activated" (z) group.
//============================================================
cf_XYZ*
    cf_ChartModel
        ::addDataXYVal( double      dUserXVal,
                        double      dUserYVal,
                        const char* a_cDomainItemLabel,
                        const char* a_cRangeValueLabel )
{
    // Use the functionality of the general method:
    return( this->addDataXYZVal( dUserXVal, dUserYVal,
                                 this->iActiveIndex,
                                 a_cDomainItemLabel,
                                 a_cRangeValueLabel ) );
        // User can continue to operate on the return val...
        // Note that this->bUpdatedChartData now == TRUE;
}
 

//===========================================================
// Add a (x,y,z) data item to the current "activated" group.
//===========================================================
cf_XYZ*
    cf_ChartModel
        ::addDataXYZVal( double      dUserXVal,
                         double      dUserYVal,
                         double      dUserZVal,
                         const char* a_cDomainItemLabel,
                         const char* a_cRangeValueLabel )
{
    // Note that the user has assumed responsibility for auto-indexing:
    this->iIndexID = CF_ID_YVALS_AT_USR_XS;

    // Add the user value to the ACTIVE data group as a new XYZ object:
    cf_XYZ* p_cf_xyzNewUserVal
            = ( (cf_DataGroup*) 
                    this->da_p_cf_dgDataGroups[ this->iActiveIndex ] )
                                ->addXYZVal(    dUserXVal,
                                                dUserYVal,
                                                dUserZVal );
    // Consider adding a Domain ITEM Label:
    cf_XYZ  *p_cf_xyzDomainItem = NULL;
    if ( a_cDomainItemLabel != NULL )
    {
        p_cf_xyzDomainItem 
            = this->p_cf_dgDomainItems
                        ->addXYZVal( dUserXVal, 0.0, 0.0 );
        p_cf_xyzDomainItem->labelNew( a_cDomainItemLabel );
    }


    // Also consider adding a Range VALUE Label:
    cf_XYZ  *p_cf_xyzRangeValue = NULL;
    if ( a_cRangeValueLabel != NULL )
    {
        p_cf_xyzRangeValue 
            = this->p_cf_dgRangeValues
                        ->addXYZVal( 0.0, dUserYVal, 0.0 );
        p_cf_xyzRangeValue->labelNew( a_cRangeValueLabel );
    }
    
    // Wrap things up:
    this->bUpdatedChartData = TRUE;
            // If the chart engine asks, tell it we have new data...
    return  p_cf_xyzNewUserVal;
            // Lets the user continue to operate on the new object...
} 



//-----------------------------------------------------------
// Adds user's Y-val with an X-Index equal to
// subsequent Natural Numbers (1, 2, 3, .. N).
// Z-index generated from current "activateGroup()" call.
//-----------------------------------------------------------
cf_XYZ*
    cf_ChartModel
        ::addDataYValIndexNatN( double  dUserYVal,
                                const char* a_cDomainItemLabel,
                                const char* a_cRangeValueLabel )
{
    // Add the user value to the ACTIVE data group as a new XYZ object:
    int iNewGroupCount  = ( (cf_DataGroup*) this->da_p_cf_dgDataGroups
                                [ this->iActiveIndex ] )->numItems() + 1;
    cf_XYZ* p_cf_xyzNewUserVal
            = ( (cf_DataGroup*) 
                    this->da_p_cf_dgDataGroups[ this->iActiveIndex ] )
                                ->addXYZVal(    iNewGroupCount,
                                                dUserYVal,
                                                this->iActiveIndex );
    // Consider adding a Domain ITEM Label:
    cf_XYZ  *p_cf_xyzDomainItem = NULL;
    int iDomainItemCount = this->p_cf_dgDomainItems->numItems();
    if ( iDomainItemCount < iNewGroupCount )
    {
        p_cf_xyzDomainItem = this->p_cf_dgDomainItems
                                ->addXYZVal( iNewGroupCount, 0.0, 0.0 );
        // with either a "natural-N" index or user-supplied label:
        if ( a_cDomainItemLabel != NULL )
            p_cf_xyzDomainItem->labelNew( a_cDomainItemLabel );
        else
            p_cf_xyzDomainItem->labelNew( ((double) iNewGroupCount) );
    }

    // The user's new XYZ item will also use the pointer to 
    // the domain-item label at the NEW group count index:
    p_cf_xyzDomainItem 
        = this->p_cf_dgDomainItems->activateXYZValAt( iNewGroupCount );
    p_cf_xyzNewUserVal->labelUsing( p_cf_xyzDomainItem->labelIs() );

    // Also consider adding a Range VALUE Label:
    if ( a_cRangeValueLabel != NULL )
    {
        cf_XYZ  *p_cf_xyzRangeValue
                = this->p_cf_dgRangeValues
                        ->addXYZVal( iNewGroupCount, dUserYVal, 0.0 );
        p_cf_xyzRangeValue->labelNew( a_cRangeValueLabel );
        p_cf_xyzNewUserVal->labelUsing( p_cf_xyzRangeValue->labelIs() );
    }
    
    // If the chart engine asks, tell it we have new data...
    this->bUpdatedChartData = TRUE;
    return  p_cf_xyzNewUserVal;
            // Lets the user continue to operate on the new object...

}   // end of cf_ChartModel::addDataYValIndexNatN( YVal... )

    
//-----------------------------------------------------------
// Adds user's Y-val with an X-Index equal to
// subsequent Integer Numbers (0, 1, 2, .. I)
// Z-index generated from current "activateGroup()" call.
//-----------------------------------------------------------
cf_XYZ*
    cf_ChartModel
        ::addDataYValIndexIntI( double  dUserYVal,
                                const char* a_cDomainItemLabel,
                                const char* a_cRangeValueLabel )
{
    // Add the user value to the data group as a new XYZ object:
    int iCurrGroupCount = ( ( cf_DataGroup*) this->da_p_cf_dgDataGroups
                                    [ this->iActiveIndex ] )->numItems();
    cf_XYZ* p_cf_xyzNewUserVal
            = ( (cf_DataGroup*) 
                    this->da_p_cf_dgDataGroups[ this->iActiveIndex ] )
                                ->addXYZVal(    iCurrGroupCount,
                                                dUserYVal,
                                                this->iActiveIndex );
    // Consider adding a Domain ITEM Label:
    int iNewGroupCount = iCurrGroupCount + 1;
    cf_XYZ  *p_cf_xyzDomainItem = NULL;
    int iDomainItemCount = this->p_cf_dgDomainItems->numItems();
    if ( iDomainItemCount < iNewGroupCount )
    {
        p_cf_xyzDomainItem = this->p_cf_dgDomainItems
                                ->addXYZVal( iCurrGroupCount, 0.0, 0.0 );
        // with either a "integer-I" index or user-supplied label:
        if ( a_cDomainItemLabel != NULL )
            p_cf_xyzDomainItem->labelNew( a_cDomainItemLabel );
        else
            p_cf_xyzDomainItem->labelNew( ((double) iCurrGroupCount) );
    }

    // The user's new XYZ item will also use the pointer to 
    // the domain-item label at the CURRENT group count index:
    p_cf_xyzDomainItem 
        = this->p_cf_dgDomainItems->activateXYZValAt( iNewGroupCount );
    p_cf_xyzNewUserVal->labelUsing( p_cf_xyzDomainItem->labelIs() );

    // Also consider adding a Range VALUE Label:
    if ( a_cRangeValueLabel != NULL )
    {
        cf_XYZ  *p_cf_xyzRangeValue
                = this->p_cf_dgRangeValues
                        ->addXYZVal( iCurrGroupCount, dUserYVal, 0.0 );
        p_cf_xyzRangeValue->labelNew( a_cRangeValueLabel );
        p_cf_xyzNewUserVal->labelUsing( p_cf_xyzRangeValue->labelIs() );
    }
    
    // If the chart engine asks, tell it we have new data...
    this->bUpdatedChartData = TRUE;
    return  p_cf_xyzNewUserVal;
            // Lets the user continue to operate on the new object...

}   // end of cf_ChartModel::addDataYValIndexIntI( YVal... )


//------------------------------------------------------------
// Add an int-indexed label to the "domain axis item" group:
//------------------------------------------------------------
cf_XYZ* 
    cf_ChartModel
        ::addLabelDomainItem(   double      dDomainItemIndex,
                                const char* a_cDomainItemLabel )
{
    cf_XYZ  *p_cf_xyzDomainItem
            = this->p_cf_dgDomainItems
                        ->addXYZVal( dDomainItemIndex, 
                                        0.0, this->iActiveIndex );
    p_cf_xyzDomainItem->labelNew( a_cDomainItemLabel );
    return  p_cf_xyzDomainItem;
}

cf_XYZ* 
    cf_ChartModel
        ::addLabelDomainItem(   double      dDomainItemIndex,
                                double      dValueForLabel )
{
    cf_XYZ  *p_cf_xyzDomainItem
            = this->p_cf_dgDomainItems
                        ->addXYZVal( dDomainItemIndex, 
                                        0.0, this->iActiveIndex );
    p_cf_xyzDomainItem->labelNew( dValueForLabel );
    return  p_cf_xyzDomainItem;
}



//------------------------------------------------------------
// Add an user-specified label to the "range values" group:
//------------------------------------------------------------
cf_XYZ* 
    cf_ChartModel
        ::addLabelRangeValue(   double  dRangeValueIndex,
                                const char* a_cRangeValueLabel )
{
    cf_XYZ  *p_cf_xyzRangeValue
            = this->p_cf_dgRangeValues
                        ->addXYZVal( 0.0, dRangeValueIndex, 
                                                this->iActiveIndex );
    p_cf_xyzRangeValue->labelNew( a_cRangeValueLabel );
    return  p_cf_xyzRangeValue;
}

cf_XYZ* 
    cf_ChartModel
        ::addLabelRangeValue(   double  dRangeValueIndex,
                                double  dValueForLabel )
{
    cf_XYZ  *p_cf_xyzRangeValue
            = this->p_cf_dgRangeValues
                        ->addXYZVal( 0.0, dRangeValueIndex, 
                                                this->iActiveIndex );
    p_cf_xyzRangeValue->labelNew( dValueForLabel );
    return  p_cf_xyzRangeValue;
}


void
    cf_ChartModel
        ::applyAllXforms() 
        {   
            this->p_cf_engChartEngine->applyGeomXform();
            this->p_cf_engChartEngine->applyProjXform();
        }

void
    cf_ChartModel
        ::applyGeomXform() 
            { this->p_cf_engChartEngine->applyGeomXform(); }


void
    cf_ChartModel
        ::applyProjXform() 
            { this->p_cf_engChartEngine->applyProjXform(); }

int
    cf_ChartModel
        ::dataGroupActive()
            { return this->iActiveIndex; }
int
    cf_ChartModel
        ::dataGroupCount()
            { return this->iNumAllocatedGroups; }
int
    cf_ChartModel
        ::dataGroupsStartAt()
            { return this->iMinGroupIndex; }
int 
    cf_ChartModel
        ::dataGroupsEndAt()
            { return this->iMaxGroupIndex; }


cf_DataGroup*
    cf_ChartModel
        ::dataGroupsReset()
{
    // Call any activated data-group's "reset" op:
    for ( i = 1; i <= this->iNumAllocatedGroups; i++ )
            ( (cf_DataGroup*) this->da_p_cf_dgDataGroups[ i ] )->reset();

    //...and note that we have deleted our current data set!
    this->bUpdatedChartData = TRUE;

    // Reset our axis-label data groups:
    if ( this->p_cf_dgGroupsList != NULL )
        this->p_cf_dgGroupsList->reset();
            
    if ( this->p_cf_dgDomainItems != NULL ) 
        this->p_cf_dgDomainItems->reset();
                
    if ( this->p_cf_dgRangeValues != NULL ) 
        this->p_cf_dgRangeValues->reset();

    return ( (cf_DataGroup*) this->da_p_cf_dgDataGroups[ this->iActiveIndex ] );

}   // end of cf_ChartModel::dataGroupsReset();
                    


cf_DataGroup*
    cf_ChartModel
        ::dataGroupsDelete()
{
    // First reset our axis-label data groups:
    if ( this->p_cf_dgGroupsList != NULL )
    {  
        this->p_cf_dgGroupsList->reset();
        this->p_cf_dgGroupsList->labelNew( "Groups" );
    }

    if ( this->p_cf_dgDomainItems != NULL )
    {
        this->p_cf_dgDomainItems->reset();
        this->p_cf_dgDomainItems->labelNew( "Domain" );
    }

    if ( this->p_cf_dgRangeValues != NULL )
    {
        this->p_cf_dgRangeValues->reset();
        this->p_cf_dgRangeValues->labelNew( "Range" );
    }

    // Loose any data-groups new()ed by user "activate" calls:
    for ( i = 1; i <= this->iNumAllocatedGroups; i++ )
            delete ( (cf_DataGroup*) this->da_p_cf_dgDataGroups[ i ] );

    this->iNumAllocatedGroups = 0;
    this->iMaxGroupIndex = 0;
    this->iMinGroupIndex = 0;
    this->iActiveIndex = 0;     // Just for now -- we'll now
                                // IMMEDIATELY activate group one!

    // Avoid any "ambiguous" state by setting up 
    // at least ONE "default" user data group...
    cf_DataGroup *p_cf_dgReturnVal = this->activateGroup( 1 );

    //...and note that we have deleted our current data set!
    this->bUpdatedChartData = TRUE;

    return p_cf_dgReturnVal;

}   // end of cf_ChartModel::dataGroupsDelete();




//===========================================
// Get/set Index Method IDs as follows:
//===========================================
void
    cf_ChartModel
        ::dataIndexMethod(  int     iIndexMethodID,
                            BOOL    bForceEngineUpdate )
{
    switch ( iIndexMethodID )
    {
        case CF_ID_YVALS_ON_NAT_NS:
        case CF_ID_YVALS_AT_INT_IS:
        case CF_ID_YVALS_AT_USR_XS:
            this->iIndexID = iIndexMethodID;
            break;

        default:
            this->iIndexID = CF_ID_YVALS_ON_NAT_NS;
            break;
    }

    // Should we re-generate a new set of chart-engine
    // objects to reflect this new data-state?
    if ( bForceEngineUpdate == TRUE )
    {
        if ( this->p_cf_engChartEngine != NULL )
        {
            this->p_cf_engChartEngine->indexMethod( iIndexMethodID );

            this->updateEngineObjects(  TRUE,   // bForceDataUpdate,
                                        TRUE,   // bForceReConfig,
                                        TRUE ); // bApplyXforms )
        }
    }
}


int
    cf_ChartModel
        ::dataIndexMethod()
{
    return this->iIndexID;
}



//----------------------------------------------------------
// Methods to deal with User Data-Range Buffering:
//----------------------------------------------------------
double
    cf_ChartModel
        ::dataRangeInclMaxBuffer()
            { return this->dBufferRangeMaxLimitBy; }
void 
    cf_ChartModel
        ::dataRangeInclMaxBuffer( double dAddOntoCurrentRange )
            { this->dBufferRangeMaxLimitBy = dAddOntoCurrentRange; }
            // Get/set extra "margin" onto the user-data-range.
            // Subject to range's "use-max-ceiling" state (below).  
            // C-tor inits this buffer value to zero.
double
    cf_ChartModel
        ::dataRangeInclMaxCandidate()
            { return this->dCandidateRangeMaxLimit; }
void
    cf_ChartModel
        ::dataRangeInclMaxCandidate( double dPossibleMaxVal )
            {   this->dCandidateRangeMaxLimit = dPossibleMaxVal;
                this->bRangeMaxLimitUsesCand = TRUE; }
            // Get/set a "candidate" max value for user-data-range.
            // Subject to range's "max-buffer" state (above).
            // Subject to range's "use-max-ceiling" state (below).
BOOL
    cf_ChartModel
        ::dataRangeInclMaxCandidateInUse()
            { return this->bRangeMaxLimitUsesCand; }
void
    cf_ChartModel
        ::dataRangeInclMaxCandidateInUse( BOOL bNewState )
            { this->bRangeMaxLimitUsesCand = bNewState; }
            // Turn consideration of any candidate min value for 
            // the user data-range ON/OFF.  C-tor inits to FALSE.
            // Calls to dataRangeInclMinCandidate( double ); 
            // (immediately above) reset this state to TRUE!
BOOL
    cf_ChartModel
        ::dataRangeInclMaxCeiling()
            { return this->bRangeMaxLimitUsesCeil; }
void
    cf_ChartModel
        ::dataRangeInclMaxCeiling( BOOL bNewState )
            { this->bRangeMaxLimitUsesCeil = bNewState; }
            // If set TRUE, uses math.h ceil() to round user data
            // ranges UP to an integer value.  C-tor inits to FALSE.
double 
    cf_ChartModel
        ::dataRangeInclMinBuffer()
            { return this->dBufferRangeMinLimitBy; }
void 
    cf_ChartModel
        ::dataRangeInclMinBuffer( double dAddOntoCurrentRange )
            { this->dBufferRangeMinLimitBy = dAddOntoCurrentRange; }
            // Get/set extra "margin" onto the user-data-range.
            // Subject to range's "use-min-floor" state (below).  
            // C-tor inits this buffer value to zero.
double
    cf_ChartModel
        ::dataRangeInclMinCandidate()
            { return this->dCandidateRangeMinLimit; }
void
    cf_ChartModel
        ::dataRangeInclMinCandidate( double dPossibleMinVal )
            {   this->dCandidateRangeMinLimit = dPossibleMinVal;
                this->bRangeMinLimitUsesCand = TRUE; }
            // Get/set a "candidate" min value for user-data-range.
            // Subject to range's "min-buffer" state (above).
            // Subject to range's "use-min-floor" state (below).
BOOL
    cf_ChartModel
        ::dataRangeInclMinCandidateInUse()
            { return this->bRangeMinLimitUsesCand; }
void
    cf_ChartModel
        ::dataRangeInclMinCandidateInUse( BOOL bNewState )
            { this->bRangeMinLimitUsesCand = bNewState; }
            // Turn consideration of any candidate min value for 
            // the user data-range ON/OFF.  C-tor inits to FALSE.
            // Calls to dataRangeInclMinCandidate( double ); 
            // (immediately above) reset this state to TRUE!
BOOL
    cf_ChartModel
        ::dataRangeInclMinFloor()
            { return this->bRangeMinLimitUsesFloor; }
void
    cf_ChartModel
        ::dataRangeInclMinFloor( BOOL bNewState )
            { this->bRangeMinLimitUsesFloor = bNewState; }
            // If set TRUE, uses math.h floor() to round user data
            // ranges DOWN to an integer value.  C-tor inits to FALSE.


//----------------------------------------------------------
// Methods to deal with Range/Origin Inclusion/Exclusion:
//----------------------------------------------------------
BOOL
    cf_ChartModel
        ::dataRangeInclOrigAlways()
            { return this->bDataRangeInclOrigAlways; }
void
    cf_ChartModel
        ::dataRangeInclOrigAlways( BOOL bNewState )
            { this->bDataRangeInclOrigAlways = bNewState; }

double
    cf_ChartModel
        ::dataRangeInclOrigFromDelta()
            { return this->dNumDataDeltasForOrigInclus; }
void
    cf_ChartModel
        ::dataRangeInclOrigFromDelta( double dNumDataDeltasAway )
            { this->dNumDataDeltasForOrigInclus = dNumDataDeltasAway; }


//-----------------------------------------------------
// Query the active data groups for min/max/range
// values.  For use in engine axis/tick ops, etc...
//-----------------------------------------------------
double
    cf_ChartModel
        ::dataXItemsMax()
{
    double  dTestVal = 0.0;
    double  dReturnVal = this->p_cf_dgDomainItems->dataXMax();
            // First *candidate* is the domain-item max value

    if ( this->iIndexID == CF_ID_YVALS_ON_NAT_NS )
        // Natural-N Indexing implies a unit increment.
        dReturnVal += 1.0;

    // Now let's scan the rest of the data groups...
    for ( i = iMinGroupIndex; i <= iMaxGroupIndex; i++ )
    {
        dTestVal = ( (cf_DataGroup*) 
                     this->da_p_cf_dgDataGroups[ i ] )->dataXMax();
        if ( dTestVal > dReturnVal ) dReturnVal = dTestVal;
    }

    return dReturnVal;
}       

double
    cf_ChartModel
        ::dataXItemsMin()
{
    double  dTestVal = 0.0;
    double  dReturnVal = 0.0;
    
    // The X-Min MAY depend on the current index method of this chart-model:
    if ( this->iIndexID == CF_ID_YVALS_AT_INT_IS )
        dReturnVal = 0.0;   // We're already DONE for Int-I Indexing
    if ( this->iIndexID == CF_ID_YVALS_ON_NAT_NS )
        dReturnVal = 0.0;   // Nat-N Indexing is also easy!
    else 
    {   // Otherwise, we'll need to scan the data groups:
        dReturnVal = this->p_cf_dgDomainItems->dataXMin();

        for ( i = iMinGroupIndex; i <= iMaxGroupIndex; i++ )
        {
            dTestVal = ( (cf_DataGroup*) 
                        this->da_p_cf_dgDataGroups[ i ] )->dataXMin();
            if ( dTestVal < dReturnVal ) dReturnVal = dTestVal;
        }
    }

    return dReturnVal;
}

double
    cf_ChartModel
        ::dataXItemsExtent()
            { return ( this->dataXItemsMax() - this->dataXItemsMin() ); }

double
    cf_ChartModel
        ::dataYValuesMax()
{
    // First look for the Max:
    double  dTestVal = 0.0;
    double  dReturnVal = this->p_cf_dgRangeValues->dataYMax();
            // First *candidate* is the data-range group max value

    for ( i = iMinGroupIndex; i <= iMaxGroupIndex; i++ )
    {
        dTestVal = ( (cf_DataGroup*) 
                     this->da_p_cf_dgDataGroups[ i ] )->dataYMax();
        if ( dTestVal > dReturnVal ) dReturnVal = dTestVal;
    }                     //...OK, we've got our Max value.
    
    // Now consider what to do if this Max value is NEGATIVE:
    if ( dReturnVal < 0.0 )
    {    // Did the user FORCE the Max Value to be zero?
        if ( this->bDataRangeInclOrigAlways == TRUE ) dReturnVal = 0.0;
        else // Consider origin inclusion w.r.t delta of the data range.
        {    // To get the delta, we'll first need to find the Min...
            double dMinVal = ( (cf_DataGroup*) 
                                this->da_p_cf_dgDataGroups
                                        [ iMinGroupIndex ] )->dataYMin();
            for ( i = this->iMinGroupIndex + 1; i <= iMaxGroupIndex; i++ )
            {
                dTestVal = ( (cf_DataGroup*) 
                             this->da_p_cf_dgDataGroups[ i ] )->dataYMin();
                if ( dTestVal < dMinVal ) dMinVal = dTestVal;
            }               //...OK, we've got our Min value.
            
            double  dDataDeltaLimit = ( dReturnVal - dMinVal ) 
                                    * this->dNumDataDeltasForOrigInclus;
            // Compare the data delta against our negative Max value...
            if ( dDataDeltaLimit >= -dReturnVal ) dReturnVal = 0.0;
        }           //...and bump the negative Max value up to zero
    }   //...negative dMax values now dealt with.

    // Now consider any USER-specified candidates for the max:
    if ( this->bRangeMaxLimitUsesCand == TRUE )
        if ( this->dCandidateRangeMaxLimit > dReturnVal )
            dReturnVal = this->dCandidateRangeMaxLimit;

    // Add any current amount of range buffering:
    dReturnVal += this->dBufferRangeMaxLimitBy;
    if ( this->bRangeMaxLimitUsesCeil ) dReturnVal = ceil( dReturnVal );

    return dReturnVal;
}

double
    cf_ChartModel
        ::dataYValuesMin()
{
    // First look for the Min:
    double  dTestVal;
    double  dReturnVal
            = ( (cf_DataGroup*)
                this->da_p_cf_dgDataGroups[ iMinGroupIndex ] )->dataYMin();
            // First *candidate* is the data-range group min value.
    dTestVal = dReturnVal;  // Synchronize values prior to testing!

    for ( i = ( iMinGroupIndex + 1 ); i <= iMaxGroupIndex; i++ )
    {
        dTestVal = ( (cf_DataGroup*)
                     this->da_p_cf_dgDataGroups[ i ] )->dataYMin();
        if ( dTestVal < dReturnVal ) dReturnVal = dTestVal;
    }                     //...OK, we've got our Min value.
    
    // Now consider what to do if this Min value is POSITIVE:
    if ( dReturnVal > 0.0 )
    {    // Did the user FORCE the Min Value to be zero?
        if ( this->bDataRangeInclOrigAlways == TRUE ) dReturnVal = 0.0;
        else // Consider origin inclusion w.r.t delta of the data range.
        {    // To get the delta, we'll first need to find the Max...
            double dMaxVal = ( (cf_DataGroup*) 
                                this->da_p_cf_dgDataGroups
                                        [ iMinGroupIndex ] )->dataYMax();
            for ( i = this->iMinGroupIndex + 1; i <= iMaxGroupIndex; i++ )
            {
                dTestVal = ( (cf_DataGroup*) 
                             this->da_p_cf_dgDataGroups[ i ] )->dataYMax();
                if ( dTestVal > dMaxVal ) dMaxVal = dTestVal;
            }               //...OK, we've got our Max value.
            
            double  dDataDeltaLimit = ( dMaxVal - dReturnVal ) 
                                    * this->dNumDataDeltasForOrigInclus;
            // Compare the data delta against our positive Min value...
            if ( dDataDeltaLimit >= dReturnVal ) dReturnVal = 0.0;
        }           //...and bump the positive Min value down to zero
    }   //...positive dMin values now dealt with.

    // Now consider any USER-specified candidates for the min:
    if ( this->bRangeMinLimitUsesCand == TRUE )
        if ( this->dCandidateRangeMinLimit < dReturnVal )
            dReturnVal = this->dCandidateRangeMinLimit;

    // Add any current amount of range buffering:
    dReturnVal -= this->dBufferRangeMinLimitBy;
    if ( this->bRangeMinLimitUsesFloor ) dReturnVal = floor( dReturnVal );

    return dReturnVal;
}

double
    cf_ChartModel
        ::dataYValuesExtent()
            { return ( this->dataYValuesMax() - this->dataYValuesMin() ); }

double
    cf_ChartModel
        ::dataYValSumsMax()
{
    double  dTestVal = 0.0;
    double  dReturnVal = this->p_cf_dgRangeValues->dataYMax();
            // First *candidate* is the data-range group max value

    // Determine the number of domain items we will sample:
    int iNumItemsInGroup = 0;
    int iMaxNumItemsInAnyGroup = 0;
    for ( i = iMinGroupIndex; i <= iMaxGroupIndex; i++ )
    {
        iNumItemsInGroup = ( (cf_DataGroup*) 
                             this->da_p_cf_dgDataGroups[ i ] )->numItems();
        if ( iNumItemsInGroup > iMaxNumItemsInAnyGroup )
            iMaxNumItemsInAnyGroup = iNumItemsInGroup;
    }
    
    // Cycle through the domain item indexes and sum the range vals:
    cf_DataGroup    *p_cf_dgSample = NULL;
    cf_XYZ          *p_cf_xyzSample = NULL;
    double      dSampleAtIndex = 0.0;
    double      dSumAtIndex = 0.0;
    for ( j = 1; j <= iMaxNumItemsInAnyGroup; j++ )
    {
        dSumAtIndex = 0.0;
        for ( k = iMinGroupIndex; k <= iMaxGroupIndex; k++ )
        {
            p_cf_dgSample = this->activateGroup( k );
            
            // Protect ourselves from bogus access requests:
            if ( p_cf_dgSample->numItems() >= j )
            {
                // Sample the item:
                p_cf_xyzSample = p_cf_dgSample->activateXYZValAt( j );
                dSampleAtIndex = p_cf_xyzSample->y();
                
                // Will the sample contribute to the sum?
                if ( dSampleAtIndex > 0 )
                    dSumAtIndex += dSampleAtIndex;
            }
        }
        // Hey!  Is this a max sum value?
        if ( dSumAtIndex > dReturnVal )
            dReturnVal = dSumAtIndex;       
    }

    // Now consider any USER-specified candidates for the max:
    if ( this->bRangeMaxLimitUsesCand == TRUE )
        if ( this->dCandidateRangeMaxLimit > dReturnVal )
            dReturnVal = this->dCandidateRangeMaxLimit;

    // Add any current amount of range buffering:
    dReturnVal += this->dBufferRangeMaxLimitBy;
    if ( this->bRangeMaxLimitUsesCeil ) dReturnVal = ceil( dReturnVal );

    return dReturnVal;
}


double
    cf_ChartModel
        ::dataYValSumsMin()
{
    double  dTestVal = 0.0;
    double  dReturnVal = this->p_cf_dgRangeValues->dataYMin();
            // First *candidate* is the data-range group min value

    // Determine the number of domain items we will sample:
    int iNumItemsInGroup = 0;
    int iMaxNumItemsInAnyGroup = 0;
    for ( i = iMinGroupIndex; i <= iMaxGroupIndex; i++ )
    {
        iNumItemsInGroup = ( (cf_DataGroup*) 
                             this->da_p_cf_dgDataGroups[ i ] )->numItems();
        if ( iNumItemsInGroup > iMaxNumItemsInAnyGroup )
            iMaxNumItemsInAnyGroup = iNumItemsInGroup;
    }
    
    // Cycle through the domain item indexes and sum the range vals:
    cf_DataGroup    *p_cf_dgSample = NULL;
    cf_XYZ          *p_cf_xyzSample = NULL;
    double      dSampleAtIndex = 0.0;
    double      dSumAtIndex = 0.0;
    for ( j = 1; j <= iMaxNumItemsInAnyGroup; j++ )
    {
        dSumAtIndex = 0.0;
        for ( k = iMinGroupIndex; k <= iMaxGroupIndex; k++ )
        {
            p_cf_dgSample = this->activateGroup( k );
            
            // Protect ourselves from bogus access requests:
            if ( p_cf_dgSample->numItems() >= j )
            {
                // Sample the item:
                p_cf_xyzSample = p_cf_dgSample->activateXYZValAt( j );
                dSampleAtIndex = p_cf_xyzSample->y();
                
                // Will the sample contribute to the sum?
                if ( dSampleAtIndex < 0 )
                    dSumAtIndex += dSampleAtIndex;
            }
        }
        // Hey!  Is this a min sum value?
        if ( dSumAtIndex < dReturnVal )
            dReturnVal = dSumAtIndex;       
    }

    // Now consider any USER-specified candidates for the min:
    if ( this->bRangeMinLimitUsesCand == TRUE )
        if ( this->dCandidateRangeMinLimit < dReturnVal )
            dReturnVal = this->dCandidateRangeMinLimit;

    // Add any current amount of range buffering:
    dReturnVal -= this->dBufferRangeMinLimitBy;
    if ( this->bRangeMinLimitUsesFloor ) dReturnVal = floor( dReturnVal );

    return dReturnVal;
}

double
    cf_ChartModel
        ::dataYValSumsExtent()
        { return ( this->dataYValSumsMax() - this->dataYValSumsMin() ); }

double
    cf_ChartModel
        ::dataZGroupsMax()
{
    double  dTestVal = 0.0;
    double  dReturnVal = ( (cf_DataGroup*) 
                            this->da_p_cf_dgDataGroups
                                    [ iMinGroupIndex ] )->dataZMax();

    for ( i = iMinGroupIndex + 1; i <= iMaxGroupIndex; i++ )
    {
        dTestVal = ( (cf_DataGroup*) 
                     this->da_p_cf_dgDataGroups[ i ] )->dataZMax();
        if ( dTestVal > dReturnVal ) dReturnVal = dTestVal;
    }
    return dReturnVal;
}       

double
    cf_ChartModel
        ::dataZGroupsMin()
{
    double  dTestVal = 0.0;
    double  dReturnVal = ( (cf_DataGroup*) 
                           this->da_p_cf_dgDataGroups
                                    [ iMinGroupIndex ] )->dataZMin();

    for ( i = iMinGroupIndex + 1; i <= iMaxGroupIndex; i++ )
    {
        dTestVal = ( (cf_DataGroup*) 
                     this->da_p_cf_dgDataGroups[ i ] )->dataZMin();
        if ( dTestVal < dReturnVal ) dReturnVal = dTestVal;
    }
    return dReturnVal;
}

double
    cf_ChartModel
        ::dataZGroupsExtent()
            { return ( this->dataZGroupsMax() - this->dataZGroupsMin() ); }


void
    cf_ChartModel
        ::draw( bf_Display *p_bf_dspUseThisDisplay )
        { this->p_cf_engChartEngine->draw( p_bf_dspUseThisDisplay ); }


void
    cf_ChartModel
        ::initiateDrawOp()
{
    if ( this->p_bf_cviewParent != NULL )
        this->p_bf_cviewParent->getDisplay()->dsp_setDirty(); 
}       // which will cause the chart view to perform a draw(),
        // which will then call this->draw(), and hence this
        // chart model's chart-engine->draw() method... 

void
    cf_ChartModel
        ::initiateDisplayMetricsOp()
//-------------------------------------------------------------------------
// We'l set up a call-back to:
//      this->updateDisplayMetrics( bf_Display *p_bf_dspUseThisDisplay );
// ...where p_bf_dspUseThisDisplay = this->bf_cviewParent->getDisplay();
//-------------------------------------------------------------------------
{ if ( this->p_bf_cviewParent != NULL ) this->p_bf_cviewParent->setMetrics(); }


cf_ChartEngine*
    cf_ChartModel
        ::modelEngine()
{ return this->p_cf_engChartEngine; }


//
//=================================================================
//
// NOTE that the chart-model member method:
//
//  void
//      cf_ChartModel
//          ::modelEngine(  int     iChartTypeID,
//                          int     iIndexMethodID,
//                          BOOL    bDeleteOldEngine )
//
// Is implemented in ONE of these SEPARATE source files --
// (depending upon the version of the ChartFolio Framework):
// "c2_cmodl.cpp", or "c3_cmodl.cpp".
//
//=================================================================
//


void
    cf_ChartModel
        ::modelEngine(    cf_ChartEngine  *p_cf_engCustomized,
                          BOOL    bDeleteOldEngine )
{
    // Don't step into this if something is FUBAR!
    if ( p_cf_engCustomized == NULL ) return;

    // IF any kind of new/malloc has already been made for this chart-
    // model's engine, we're gonna hafta do some kind of delete op...
    cf_ChartEngine  *p_cf_engOldEngine = this->p_cf_engChartEngine;

    this->p_cf_engChartEngine = p_cf_engCustomized;

    // IF requested (and by default) if there was
    // an old engine already allocated, get rid of it:
    if( bDeleteOldEngine )
            if( p_cf_engOldEngine != NULL )
                        delete p_cf_engOldEngine;

    // Synchronize this new engine to the chart-model's data-state:
    this->p_cf_engChartEngine->updateFromCurrentModel();
    //this->p_cf_engChartEngine->autoConfigure();

    // Make a note via the indexing method that we have a custom chart engine:
    this->iIndexID = this->p_cf_engChartEngine->indexMethodIs();

    return;

}   // end of cf_ChartModel::modelEngine( cf_ChartEngine*, BOOL )


cf_Kernel*
    cf_ChartModel
        ::modelKernel()
            { return this->p_cf_kChartKernel; }



void
    cf_ChartModel
        ::modelKernel(  cf_Kernel   *p_cf_kNewKernel,
                        BOOL        bDeleteOldKernel   )
{ 
    //
    // First, reset this chart's kernel:
    //
    this->p_cf_kChartKernel = p_cf_kNewKernel; 

    //
    // Then, reset the kernel dependencies of other kernel-dependent
    // objects contained in this class:
    //
    if ( this->p_cf_engChartEngine != NULL )
            this->p_cf_engChartEngine
                    ->engineKernel( this->p_cf_kChartKernel );

    //
    // Now, delete the new/malloc of the old kernel if requested:
    //
    if ( bDeleteOldKernel == TRUE ) 
        if ( this->p_cf_kChartKernel != NULL )
            delete this->p_cf_kChartKernel;

}   // end of cf_ChartModel::modelKernel(...)



bf_ChartView*
    cf_ChartModel
        ::modelView()
            { return this->p_bf_cviewParent; }

void
    cf_ChartModel
        ::modelView(    bf_ChartView        *p_bf_cviewNewView,
                        BOOL                bDeleteOldView )
{
    //
    // Only delete the current display if called for -- the user
    // MIGHT wanna keep it around for later use (e.g. a display
    // context push/pop)...
    //
    if ( bDeleteOldView == TRUE ) 
        if ( this->p_bf_cviewParent != NULL )
            delete this->p_bf_cviewParent;

    this->p_bf_cviewParent = p_bf_cviewNewView;

    this->initiateDisplayMetricsOp();
    this->p_cf_engChartEngine->applyProjXform();
    
    return;
}



bf_Color
    cf_ChartModel
        ::paletteColor( int iModIntoAvailColors )
{
    bf_Color   bf_clrReturnThisVal;
    
    if ( iModIntoAvailColors > 0 ) 
        // Return the color value...
        bf_clrReturnThisVal
            = this->a_bf_clrChartPalette
                //...at this index...
                [ (iModIntoAvailColors - 1) % C1_CMODL_NUM_PALETTE_COLORS ];
    else
        bf_clrReturnThisVal =  CF_BLACK;

    return bf_clrReturnThisVal;
}


void    
    cf_ChartModel
        ::paletteColor( int     iModIntoAvailColors, 
                        bf_Color   bf_clrNew )
{
    // Screen for bogus index values:
    if ( iModIntoAvailColors > 0 ) 
        this->a_bf_clrChartPalette
            // The color at this index...
            [ (iModIntoAvailColors - 1) % C1_CMODL_NUM_PALETTE_COLORS ] 
                    = bf_clrNew;
                    // gets updated w/ this value.
}


void
    cf_ChartModel
        ::paletteColors()
{
    this->a_bf_clrChartPalette[ 0 ] = CF_RED;
    this->a_bf_clrChartPalette[ 1 ] = CF_LITE_BLUE;
    this->a_bf_clrChartPalette[ 2 ] = CF_YELLOW;
    this->a_bf_clrChartPalette[ 3 ] = CF_GREEN;
    this->a_bf_clrChartPalette[ 4 ] = CF_DARK_BLUE;
    this->a_bf_clrChartPalette[ 5 ] = CF_LITE_PURPLE;
    this->a_bf_clrChartPalette[ 6 ] = CF_BLUE;
    this->a_bf_clrChartPalette[ 7 ] = CF_SLATE_GRAY;
    this->a_bf_clrChartPalette[ 8 ] = CF_LITE_GREEN;
    this->a_bf_clrChartPalette[ 9 ] = CF_SLATE_BLUE;
    this->a_bf_clrChartPalette[ 10 ] = CF_MED_PURPLE;
    this->a_bf_clrChartPalette[ 11 ] = CF_LITE_GRAY;
    this->a_bf_clrChartPalette[ 12 ] = CF_LITE_RED;
    this->a_bf_clrChartPalette[ 13 ] = CF_ORANGE;
    this->a_bf_clrChartPalette[ 14 ] = CF_CREAM;
    this->a_bf_clrChartPalette[ 15 ] = CF_DARK_PURPLE;
}



bf_BrushHatch
    cf_ChartModel
        ::paletteBrushHatch( int iModIntoAvailHatches )
{
    bf_BrushHatch  bf_bhatReturnThisVal;
    
    if ( iModIntoAvailHatches > 0 ) 
        // Return the brush-hatch value...
        bf_bhatReturnThisVal
            = this->a_bf_bhatChartPalette
                //...at this index...
                [ (iModIntoAvailHatches - 1) % C1_CMODL_NUM_BRUSH_HATCHES ];
    else
        bf_bhatReturnThisVal = CF_BHAT_NONE; // try to do no harm...

    return bf_bhatReturnThisVal;
}


void    
    cf_ChartModel
        ::paletteBrushHatch(    int     iModIntoAvailHatches, 
                                bf_BrushHatch  bf_bhatNew )
{
    // Screen for bogus index values:
    if ( iModIntoAvailHatches > 0 ) 
        this->a_bf_bhatChartPalette
            // The brush-hatch at this index...
            [ (iModIntoAvailHatches - 1) % C1_CMODL_NUM_BRUSH_HATCHES ] 
                = bf_bhatNew;
                // gets updated w/ this value.
}


void
    cf_ChartModel
        ::paletteBrushHatches(  bf_BrushHatch bf_bhat01,
                                bf_BrushHatch bf_bhat02,
                                bf_BrushHatch bf_bhat03 )
{
    this->a_bf_bhatChartPalette[ 0 ] = bf_bhat01;
    this->a_bf_bhatChartPalette[ 1 ] = bf_bhat02;
    this->a_bf_bhatChartPalette[ 2 ] = bf_bhat03;
    // See the hpp file for default init settings...
}



//------------------------------------------------------------
// Manipulate 2D/3D Point-Marker Render Attribs of this chart:
//------------------------------------------------------------
BOOL
    cf_ChartModel
        ::paletteMarkers2D3D(   int     i2For2D_3For3D )
{
    BOOL    bStatus = FALSE;

    if ( i2For2D_3For3D == 2 )
        if ( this->bDraw2DPointMarkers == TRUE )
            bStatus = TRUE;

    else if ( i2For2D_3For3D == 3 )
        if ( this->bDraw3DPointMarkers == TRUE )
            bStatus = TRUE;

    return bStatus;
}


void
    cf_ChartModel
        ::paletteMarkers2D3D(   BOOL    bDisp2DMarkers,
                                BOOL    bDisp3DMarkers,
                                BOOL    bResetCurrentGroups )
{
    // Keep track of what we're doing:
    this->bDraw2DPointMarkers = bDisp2DMarkers;
    this->bDraw3DPointMarkers = bDisp3DMarkers;

    // Now update the active groups:
    if ( bResetCurrentGroups == TRUE )
    {
        cf_DispAttribs      *p_cf_datbCurrGroups = NULL;
        for ( i = 1; i <= this->iNumAllocatedGroups; i++ )
        {
            p_cf_datbCurrGroups = ( (cf_DataGroup*) 
                        this->da_p_cf_dgDataGroups[ i ] )->dispAttribs();
            p_cf_datbCurrGroups->markWhen2D( bDisp2DMarkers );
            p_cf_datbCurrGroups->markWhen3D( bDisp3DMarkers );
        }
    }
}                               



bf_PenStyle
    cf_ChartModel
        ::palettePenStyle( int iModIntoAvailStyles )
{
    bf_PenStyle    bf_pstyReturnThisVal;

    if ( iModIntoAvailStyles > 0 ) 
        // Return the pen-style value...
        bf_pstyReturnThisVal
            = this->a_bf_pstyChartPalette
                //...at this index...
                [ (iModIntoAvailStyles - 1) % C1_CMODL_NUM_PEN_STYLES ];  
    else
        bf_pstyReturnThisVal = CF_PS_SOLID;  // try to do no harm...

    return bf_pstyReturnThisVal;
}


void    
    cf_ChartModel
        ::palettePenStyle(  int     iModIntoAvailStyles, 
                            bf_PenStyle    bf_pstyNew )
{
    // Screen for bogus index values:
    if ( iModIntoAvailStyles > 0 ) 
        this->a_bf_pstyChartPalette
            // The pen-style at this index...
            [ (iModIntoAvailStyles - 1) % C1_CMODL_NUM_PEN_STYLES ] 
                = bf_pstyNew;
                // ...gets updated w/ this value.
}


void
    cf_ChartModel
        ::palettePenStyles( bf_PenStyle bf_psty01,
                            bf_PenStyle bf_psty02,
                            bf_PenStyle bf_psty03 )
{
    this->a_bf_pstyChartPalette[ 0 ] = bf_psty01;
    this->a_bf_pstyChartPalette[ 1 ] = bf_psty02;
    this->a_bf_pstyChartPalette[ 2 ] = bf_psty03;
    // See the hpp file for default init settings...
}



void
    cf_ChartModel
        ::updateDisplayMetrics( bf_Display *p_bf_dspHasChanged )
{
    // Let the child chart-engine use this display context to make
    // any adjustments IT (or it's encapsulated objects) will need:
    this->p_cf_engChartEngine->updateDisplayMetrics( p_bf_dspHasChanged );
}



void
    cf_ChartModel
        ::updateEngineObjects(  BOOL    bForceDataUpdate,
                                BOOL    bForceReConfig,
                                BOOL    bApplyXforms )
{
    // Protect ourselves from bogosities:
    if ( this->p_cf_engChartEngine == NULL ) return;

    // Do we need to re-generate a new set of chart-engine objects?
    BOOL    bPerformDataUpdate = FALSE;
    if ( bForceDataUpdate == TRUE )  bPerformDataUpdate = TRUE;
    if ( this->bUpdatedChartData == TRUE )  bPerformDataUpdate = TRUE;

    if ( bPerformDataUpdate == TRUE )
    {   //...then do so...
        this->p_cf_engChartEngine->updateFromCurrentModel();
        this->bUpdatedChartData = FALSE;
    }

    // Do we need to re-(auto)configure the engine and its objects?
    if ( ( bPerformDataUpdate == TRUE ) || ( bForceReConfig == TRUE ) )
        this->p_cf_engChartEngine->autoConfigure();

    // Were Geom/Projection xforms called for, or are they now implied?
    if ( ( bPerformDataUpdate == TRUE ) || ( bApplyXforms == TRUE ) )
    {
        this->p_cf_engChartEngine->applyGeomXform();
        this->p_cf_engChartEngine->applyProjXform();
    }

    return;
}

    
//
// END of cf_ChartModel Implementation!
//    

//////////////////////////////////////
// END of File: C1_CMODL.CPP
//////////////////////////////////////






