////////////////////////////////////////
// START of File: C1_ENGIN.CPP
////////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/c1_engin.hpp>
#include    <math.h>

//==========================================
// Implement class cf_ChartEngine
//==========================================


cf_ChartEngine
    ::cf_ChartEngine(   cf_ChartModel   *p_cf_cmodlOwner,
                        int             iChartTypeID,
                        int             iIndexMethodID )

    :   // Let's *guarantee* the init state of important components:
        cf_3dipEngObjVirtMin( 0.0, 0.0, 0.0 ),
        cf_3dipEngObjVirtMax( 0.0, 0.0, 0.0 ),

        // Make sure that we can track our *first* draw op,
        // so we can gather any neccessary display-metrics, etc.
        bHasEverRendered_( FALSE )
            // ...will be reset to TRUE upon first draw() op.
{                       
    //------------------------------------------------------
    // Init "critical (sometimes friend) pointer" vars:
    //------------------------------------------------------
    this->p_cf_cmodlUserData = p_cf_cmodlOwner;
            // We'll need to be able to use ANY of the 
            // parent chart model's member var or methods...

    this->p_cf_kEngineKernel = this->p_cf_cmodlUserData->modelKernel();
            // The kernel controls the state of this 
            // chart/engine's tramsformation geometry...

    //----------------------------
    // Init Chart Identity vars:
    //----------------------------
    this->iEngineID = iChartTypeID;
    this->indexMethod( iIndexMethodID );
                    // Should preceede any chart-type 
                    // and/or index-method dependent configs!

    this->bIsEngineCartesian    = FALSE;
    this->bIsEnginePolar        = FALSE;
    this->bIsEngineBarColStack  = FALSE;
                                // we WILL derive into one of these,
                                // but we do NOT know which -- yet...

    //----------------------------------
    // Perform modular initializations:
    //----------------------------------
    this->initObjectDynamicStorage();
            // Init the dynamic storage for the chart's objects:
            // (In this implementation, Dynamic Arrays of 
            // Pointers to cf_Virt3DObjs -- or really, 
            // objects DERIVED from cf_Virt3DObjs)...

    this->init3DBoxETC();
            // Init the "3D Box" to the current kernel dimensions!
    
    this->initDrawingAttribs();
            // New and Init the chart/engine's object's drawing attribs.
    
    this->preferProportions(    1.0,    // Init Height-to-Width
                                1.0,    // and Depth-to-Height Ratios
                                FALSE );// but DELAY auto-configuration...
                                        //... until immediately below...
    this->measureDataMinMax();
            // Initialize chart/engine dynamic scaling parameters.

    this->measureVirtMinMax();
            // Used to calc data and virtual ranges for scaling
            // and or tick marking ops used in auto-configuration

    this->initGridTickSeries();
    this->initTextMarkers();
            // Make em all explicitly and *test-ably* NULL!

    this->autoConfigure();
            // Based on the intializations performed above...

}   // end of c-tor



//
// Destructor must clean up any extra resource allocations.
//
cf_ChartEngine
    ::~cf_ChartEngine()
{
    // Delete ANYTHING we new'ed -- but be CAREFUL !!!
    if ( this->p_cf_3dbmpProjectedBitmap != NULL )
                    delete this->p_cf_3dbmpProjectedBitmap;
                    // from when we made a chart bitmap wall...

    // We new()-ed our 3d Box display attribs, so loose 'em
    if ( this->p_cf_datbBackFace != NULL )
                    delete this->p_cf_datbBackFace;
    if ( this->p_cf_datbLeftFace != NULL )
                    delete this->p_cf_datbLeftFace;
    if ( this->p_cf_datbBottomFace != NULL )
                    delete this->p_cf_datbBottomFace;

    // We also new()-ed our window background attribs --loose 'em!
    if ( this->p_cf_datbDispBack != NULL )
                    delete this->p_cf_datbDispBack;

    //
    // NOW... the heavy stuff!  Delete the pointer allocations 
    // of any dynamic engine objects we are responsible for!
    //
    for ( i = 1; i <= this->iNumCompoObjects; i++ )
        if ( this->da_p_cf_objCompoObjects[ i ] != NULL )
            delete ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] );

    for ( i = 1; i <= this->iNumFaceObjects; i++ )
        if ( this->da_p_cf_objFaceObjects[ i ] != NULL )
            delete ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] );

    for ( i = 1; i <= this->iNumSpaceObjects; i++ )
        if ( this->da_p_cf_objSpaceObjects[ i ] != NULL )
            delete ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] );

    for ( i = 1; i <= this->iNumMarkerObjects; i++ )
        if ( this->da_p_cf_objMarkerObjects[ i ] != NULL )
            delete ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] );

    // Well... if we haven't crashed (grin) -- we're DONE!
    return;
}



void
    cf_ChartEngine
        ::addCompoObject( cf_Virt3DObj *p_cf_objAddThisObject ) 
{
    p_cf_objAddThisObject->applyTranslateOp(    this->dChartXOrigin,
                                                this->dChartYOrigin,
                                                this->dChartZOrigin  );
    this->iNumCompoObjects++;
    this->da_p_cf_objCompoObjects[ iNumCompoObjects ] = p_cf_objAddThisObject; 
    return;
}   // end of cf_ChartEngine::addCompoObject( cf_Virt3DObj* )
                    


void
    cf_ChartEngine
        ::addFaceObject( cf_Virt3DObj *p_cf_objAddThisObject ) 
{
    p_cf_objAddThisObject->applyTranslateOp(    this->dChartXOrigin,
                                                this->dChartYOrigin,
                                                this->dChartZOrigin  );
    this->iNumFaceObjects++;
    this->da_p_cf_objFaceObjects[ iNumFaceObjects ] = p_cf_objAddThisObject; 
    return;
}   // end of cf_ChartEngine::addFaceObject( cf_Virt3DObj* ) 


void
    cf_ChartEngine
        ::addSpaceObject( cf_Virt3DObj *p_cf_objAddThisObject ) 
{
    p_cf_objAddThisObject->applyTranslateOp(    this->dChartXOrigin,
                                                this->dChartYOrigin,
                                                this->dChartZOrigin  );
    this->iNumSpaceObjects++;
    this->da_p_cf_objSpaceObjects[ iNumSpaceObjects ] = p_cf_objAddThisObject; 
    return;
}   // end of cf_ChartEngine::addSpaceObject( cf_Virt3DObj* ) 


void
    cf_ChartEngine
        ::addMarkerObject( cf_Virt3DObj *p_cf_objAddThisObject ) 
{
    p_cf_objAddThisObject->applyTranslateOp(    this->dChartXOrigin,
                                                this->dChartYOrigin,
                                                this->dChartZOrigin  );
    this->iNumMarkerObjects++;
    this->da_p_cf_objMarkerObjects[ iNumMarkerObjects ] = p_cf_objAddThisObject;
    return;
}   // end of cf_ChartEngine::addMarkerObject( cf_Virt3DObj* )



//==========================================================================
// Use the Given Chart Framework Kernel to apply various transformation(s)
// to this cf_ChartEngine and its components (by using our engine-kernel):
//==========================================================================
void
    cf_ChartEngine
        ::applyDispMetrics()
{
    // Update the Chart Faces:
    this->cf_3dpgLeftFace.applyDispMetrics( p_cf_kEngineKernel );
    this->cf_3dpgBackFace.applyDispMetrics( p_cf_kEngineKernel );
    this->cf_3dpgBottomFace.applyDispMetrics( p_cf_kEngineKernel );

    // Update the Face Component List ( tick marks, etc):
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                            ->applyDispMetrics( p_cf_kEngineKernel );

    // Update the Face Object List ( control lines, etc):
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                            ->applyDispMetrics( p_cf_kEngineKernel );

    // Update the Space Object List ( ribbons, etc):
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                            ->applyDispMetrics( p_cf_kEngineKernel );

    // Update the Marker Object List:
    for ( i = 1; i <= iNumMarkerObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                            ->applyDispMetrics( p_cf_kEngineKernel );

    // Note it's NOT neccessary to update the invis-Min/Max Tracking points:
}


void
    cf_ChartEngine
        ::applyGeomXform()
{
    // Process the Chart Faces:
    this->cf_3dpgLeftFace.applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpgBackFace.applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpgBottomFace.applyGeomXform( this->p_cf_kEngineKernel );

    // Process the Face Component List ( tick marks, etc):
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                            ->applyGeomXform( this->p_cf_kEngineKernel );

    // Process the Face Object List ( control lines, etc):
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                            ->applyGeomXform( this->p_cf_kEngineKernel );

    // Process the Space Object List ( ribbons, etc):
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                            ->applyGeomXform( this->p_cf_kEngineKernel );

    // Process the Marker Object List ( XY Text Markers, etc):
    for ( i = 1; i <= iNumMarkerObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                    ->applyGeomXform( this->p_cf_kEngineKernel );

    // Process the invisible virtual/data-limit points:
    this->cf_3dipEngObjVirtMin.applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dipEngObjVirtMax.applyGeomXform( this->p_cf_kEngineKernel );

}   // end of cf_ChartEngine::applyGeomXform(...);


void    cf_ChartEngine::
        applyProjXform()
{
    // Project the Chart Faces:
    this->cf_3dpgLeftFace.applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpgBackFace.applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpgBottomFace.applyProjXform( this->p_cf_kEngineKernel );

    // Project the Face Component List ( tick marks, etc):
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                            ->applyProjXform( this->p_cf_kEngineKernel );

    // Project the Face Object List ( control lines, etc):
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                            ->applyProjXform( this->p_cf_kEngineKernel );

    // Project the Space Object List ( ribbons, etc):
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                            ->applyProjXform( this->p_cf_kEngineKernel );

    // Project the Marker Object List:
    for ( i = 1; i <= iNumMarkerObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                            ->applyProjXform( this->p_cf_kEngineKernel );

    // Process the invisible virtual/data-limit points:
    this->cf_3dipEngObjVirtMin.applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dipEngObjVirtMax.applyProjXform( this->p_cf_kEngineKernel );

}   // end of cf_ChartEngine::applyProjXform(...);


void
    cf_ChartEngine
        ::applyVirtScalings()
            {}

void
    cf_ChartEngine
        ::applyVirtTranslations()
            {}


//
// This member method modularizes the calculation of the
// essential chart dynamic scaling parameter via the other
// chart engine member methods...
//
void
    cf_ChartEngine
        ::autoConfigure()
{
    // Set up the 3D Box's "reference points" -- Notice that 
    // we will use any DERIVED sub-class object's method(s)...
    this->configOrigins();
    this->config3DBoxETC();

    // Compute and Apply any neccessary 
    // Object Scalings & Translations:
    this->measureVirtMinMax();
    this->calcVirtScaleFactors();
    this->calcVirtTransFactors();   // ...computed!

    this->applyVirtScalings();
    this->applyVirtTranslations();  // ...applied!

    return;

}   // end of cf_ChartEngine::autoConfigure()



cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_X_BackFace()
{ 
    if ( this->p_cf_3dgtsXonBackFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsXonBackFace 
                    = new cf_3DGTS_AutoMarkBackFaceX( this );
        this->addCompoObject( this->p_cf_3dgtsXonBackFace );
        this->p_cf_3dgtsXonBackFace->on();
    }
    return  this->p_cf_3dgtsXonBackFace; 
}           // which SHOULD be A-OK to go with!...


cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_X_BottomFace()
{
    if ( this->p_cf_3dgtsXonBottomFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsXonBottomFace 
                    = new cf_3DGTS_AutoMarkBottomFaceX( this );
        this->addCompoObject( this->p_cf_3dgtsXonBottomFace );
        this->p_cf_3dgtsXonBottomFace->on();
    }
    return  this->p_cf_3dgtsXonBottomFace; 
}           // which SHOULD be A-OK to go with!...
    


cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_Y_LeftFace()
{
    if ( this->p_cf_3dgtsYonLeftFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsYonLeftFace 
                        = new cf_3DGTS_AutoMarkLeftFaceY( this );
        this->addCompoObject( this->p_cf_3dgtsYonLeftFace );
        this->p_cf_3dgtsYonLeftFace->on();
    }
    return  this->p_cf_3dgtsYonLeftFace; 
}           // which SHOULD be A-OK to go with!...



cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_Y_BackFace()
{
    if ( this->p_cf_3dgtsYonBackFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsYonBackFace 
                        = new cf_3DGTS_AutoMarkBackFaceY( this );
        this->addCompoObject( this->p_cf_3dgtsYonBackFace );
        this->p_cf_3dgtsYonBackFace->on();
    }
    return  this->p_cf_3dgtsYonBackFace; 
}           // which SHOULD be A-OK to go with!...
    

cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_Z_LeftFace()
{
    if ( this->p_cf_3dgtsZonLeftFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsZonLeftFace = new cf_3DGTS_AutoMarkLeftFaceZ( this );
        this->addCompoObject( this->p_cf_3dgtsZonLeftFace );
        this->p_cf_3dgtsZonLeftFace->on();
    }
    return  this->p_cf_3dgtsZonLeftFace; 
}           // which SHOULD be A-OK to go with!...



cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_Z_BottomFace()
{
    if ( this->p_cf_3dgtsZonBottomFace == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsZonBottomFace 
                        = new cf_3DGTS_AutoMarkBottomFaceZ( this );
        this->addCompoObject( this->p_cf_3dgtsZonBottomFace );
        this->p_cf_3dgtsZonBottomFace->on();
    }
    return  this->p_cf_3dgtsZonBottomFace; 
}           // which SHOULD be A-OK to go with!...


    
cf_3DGridTickSeries*
    cf_ChartEngine
        ::axis_Polar_Floor()
{
    if ( this->p_cf_3dgtsPolarOnFloor == NULL )
    // Which it WILL the first time an access op is run!...
    {
        this->p_cf_3dgtsPolarOnFloor 
                        = new cf_3DGTS_AutoMarkPolarFloor( this );
        this->addCompoObject( this->p_cf_3dgtsPolarOnFloor );
        this->p_cf_3dgtsPolarOnFloor->on();
    }
    return  this->p_cf_3dgtsPolarOnFloor; 
}           // which SHOULD be A-OK to go with!...

    
    
//
// Manipulate the Chart's Bitmap Wall Source File:
//
BOOL
    cf_ChartEngine
        ::bitmapChartWallFile() 
            { return this->p_cf_3dbmpProjectedBitmap->isFileSet(); }

void
    cf_ChartEngine
        ::bitmapChartWallFileName( const char *a_cFileNameString ) 
            { this->p_cf_3dbmpProjectedBitmap
                        ->sourceFileName( a_cFileNameString ); }    

const char*  
    cf_ChartEngine
        ::bitmapChartWallFileName()
            { return this->p_cf_3dbmpProjectedBitmap->sourceFileName(); }

//
// Manipulate the Chart's Bitmap Background Wall Paper Source File:
//
BOOL
    cf_ChartEngine
        ::bitmapWallPaperFile() 
{
    if ( this->cf_strWallPaperFile.cf_strlen() == 0 )
        return FALSE;
    else 
        return TRUE;
}

void
    cf_ChartEngine
        ::bitmapWallPaperFileName( const char *a_cFileNameString )
            { this->cf_strWallPaperFile = a_cFileNameString; }

const char*  
    cf_ChartEngine
        ::bitmapWallPaperFileName()
            { return this->cf_strWallPaperFile; }

            


void
    cf_ChartEngine
        ::calcVirtScaleFactors()
            {}
    
void
    cf_ChartEngine
        ::calcVirtTransFactors()
            {}

bf_Color
    cf_ChartEngine
        ::colorDispBack()   
            { return this->p_cf_datbDispBack->colorBrush(); }
            // Note that we don't even WANT to get the chart-view/
            // display-device's current color, cause if it's been 
            // turned white, as in chart control dialogs, we'll get 
            // an incorrect value.  The chart-engine will hold our
            // "background-color" value for us...

void
    cf_ChartEngine
        ::colorDispBack( const bf_Color &cr_bf_clrNew )
        {   
            this->p_cf_datbDispBack->colorBrush( cr_bf_clrNew );
            this->updateTextBackColors( cr_bf_clrNew ); 
        }

bf_Color   
    cf_ChartEngine
        ::colorFaceBack()       
            { return this->p_cf_datbBackFace->colorBrush(); }
void
    cf_ChartEngine
        ::colorFaceBack( const bf_Color &cr_bf_clrNew )
            { this->p_cf_datbBackFace->colorBrush( cr_bf_clrNew ); }

bf_Color
    cf_ChartEngine
        ::colorFaceBottom() 
            { return this->p_cf_datbBottomFace->colorBrush(); }
void
    cf_ChartEngine
        ::colorFaceBottom( const bf_Color &cr_bf_clrNew )
            { this->p_cf_datbBottomFace->colorBrush( cr_bf_clrNew ); }

bf_Color
    cf_ChartEngine
        ::colorFaceLeft()   
            { return this->p_cf_datbLeftFace->colorBrush(); }
void
    cf_ChartEngine
        ::colorFaceLeft( const bf_Color &cr_bf_clrNew )
            { this->p_cf_datbLeftFace->colorBrush( cr_bf_clrNew ); }
                                

//
// This private member modularizes the engine calculations:
//
// If we change coordinates within the chart, or change the
// dimensions of the kernel, we should recalc the chart.
//
void
    cf_ChartEngine
        ::config3DBoxETC()
{
    // Make a record of the LAST set of 3D Box dimensions
    // for upcoming rescaling ops on chart components:
    double dLastXDim = this->d3DBoxXMax - this->d3DBoxXMin;
    double dLastYDim = this->d3DBoxYMax - this->d3DBoxYMin;
    double dLastZDim = this->d3DBoxZMax - this->d3DBoxZMin;

    // RESET vars which hold the kernel's CURRENT 3D Box Coords:
    this->d3DBoxXMin = this->p_cf_kEngineKernel->getXMin();
    this->d3DBoxXMax = this->p_cf_kEngineKernel->getXMax();

    this->d3DBoxYMin = this->p_cf_kEngineKernel->getYMin();
    this->d3DBoxYMax = this->p_cf_kEngineKernel->getYMax();

    this->d3DBoxZMin = this->p_cf_kEngineKernel->getZMin();
    this->d3DBoxZMax = this->p_cf_kEngineKernel->getZMax();

    // Measure the NEW 3D Box dimensions for upcoming
    // rescale ops on chart components:
    double dCurrXDim = this->d3DBoxXMax - this->d3DBoxXMin;
    double dCurrYDim = this->d3DBoxYMax - this->d3DBoxYMin;
    double dCurrZDim = this->d3DBoxZMax - this->d3DBoxZMin;

    // Calc local re-scale-ing factors to apply to the chart components:
    double  dCXSF = 1.0;    // CXSF = Component X Scale Factor
    double  dCYSF = 1.0;    // CYSF = Component Y Scale Factor
    double  dCZSF = 1.0;    // CZSF = Component Z Scale Factor
    BOOL    bUpdateScaleFacs = FALSE;

    if ( dCurrXDim != dLastXDim )
    {
        bUpdateScaleFacs = TRUE;
        // Avoid numeric overflows caused by small divisors:
        // Was originally: "if ( dLastXDim != 0.0 )"
        if ( fabs( dLastXDim ) > CF_NUM_ZERO_IF_LT )
            dCXSF = dCurrXDim / dLastXDim;
    }   
    if ( dCurrYDim != dLastYDim )
    {
        bUpdateScaleFacs = TRUE;
        if ( fabs( dLastYDim ) > CF_NUM_ZERO_IF_LT ) // ditto as above...
            dCYSF = dCurrYDim / dLastYDim;// Otherwise = 1.0 per above.
    }   
    if ( dCurrZDim != dLastZDim )
    {
        bUpdateScaleFacs = TRUE;
        if ( fabs( dLastZDim ) > CF_NUM_ZERO_IF_LT ) // ditto as above...
            dCZSF = dCurrZDim / dLastZDim;// Otherwise = 1.0 per above.
    }

    //--------------------------------------------------
    // Rescale Chart Components (e.g. tick marks, etc)
    // to match the new 3D Box dimensions:
    //--------------------------------------------------
    if ( bUpdateScaleFacs == TRUE )
    {
        // First re-scale the 3D Box Faces:
        this->cf_3dpgLeftFace.applyScaleOp( dCXSF, dCYSF, dCZSF ); 
        this->cf_3dpgBackFace.applyScaleOp( dCXSF, dCYSF, dCZSF ); 
        this->cf_3dpgBottomFace.applyScaleOp( dCXSF, dCYSF, dCZSF ); 

        // Then re-scale other components, such as tick marks, etc:
        for ( i = 1; i <= iNumCompoObjects; i++ )
            ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                            ->applyScaleOp( dCXSF, dCYSF, dCZSF ); 

        for ( i = 1; i <= iNumMarkerObjects; i++ )
            ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                            ->applyScaleOp( dCXSF, dCYSF, dCZSF ); 
    }

    return;

}   // end of cf_ChartEngine::config3DBoxETC()


void
    cf_ChartEngine
        ::configOrigins()
{
    //
    // This BASE "generic" method sets the origin
    // as the MIN X/Y/Z of the virtual 3D Box:
    //
    this->p_cf_kEngineKernel->copy3DBoxOrigin(  &this->dChartXOrigin,
                                                &this->dChartYOrigin,
                                                &this->dChartZOrigin );

}   // end of cf_ChartEngine::configOrigins()

    


//
// These member methods gets current chart engine 
// data values for user classes that need them, 
// e.g. many of the grid/tick mark and
// grid/tick-mark series classes of objects. use these
// values to calc Scale Factors for their virtual objects
// based on the range of the engine's virtual X, Y, & Z 
// mins/maxes.
//
double
    cf_ChartEngine
        ::dataVirtXOrigin()
            { return this->dChartXOrigin; }
double
    cf_ChartEngine
        ::dataVirtYOrigin()
            { return this->dChartYOrigin; }
double
    cf_ChartEngine
        ::dataVirtZOrigin()
            { return this->dChartZOrigin; }


double  
    cf_ChartEngine
        ::dataChartXRangeMin()
            { return this->dChartXRangeMin; }

double  
    cf_ChartEngine
        ::dataChartXRangeMax()
            { return this->dChartXRangeMax; }
    
double  
    cf_ChartEngine
        ::dataChartYRangeMin()
            { return this->dChartYRangeMin; }

double  
    cf_ChartEngine
        ::dataChartYRangeMax()
            { return this->dChartYRangeMax; }

double
    cf_ChartEngine
        ::dataChartZRangeMin()
            { return this->dChartZRangeMin; }

double
    cf_ChartEngine
        ::dataChartZRangeMax()
            { return this->dChartZRangeMax; }

double
    cf_ChartEngine
        ::dataVirtXRangeMin()
            { return this->dVirtXRangeMin; }

double
    cf_ChartEngine
        ::dataVirtXRangeMax()
            { return this->dVirtXRangeMax; }

double
    cf_ChartEngine
        ::dataVirtYRangeMin()
            { return this->dVirtYRangeMin; }

double
    cf_ChartEngine
        ::dataVirtYRangeMax()
            { return this->dVirtYRangeMax; }

double
    cf_ChartEngine
        ::dataVirtZRangeMin()
            { return this->dVirtZRangeMin; }

double
    cf_ChartEngine
        ::dataVirtZRangeMax()
            { return this->dVirtZRangeMax; }


BOOL
    cf_ChartEngine
        ::disp2DMode()
            { return !( this->b3DMode ); }
void
    cf_ChartEngine
        ::disp2DMode( BOOL  bNewState )
            { this->disp3DMode( !bNewState ); }
BOOL
    cf_ChartEngine
        ::disp3DMode()
            { return this->b3DMode; }
void
    cf_ChartEngine
        ::disp3DMode( BOOL  bNewState )
{
    if ( bNewState == FALSE )
    // The user wants to go FROM 3D Mode TO 2D Mode
    {
        // But don't switch if we're already in 2D Mode!
        if ( this->b3DMode == TRUE )
        {
            // Save current view params for subsequent recall:
            this->dSaveXAngle = this->p_cf_kEngineKernel->getXAngle();
            this->dSaveYAngle = this->p_cf_kEngineKernel->getYAngle();
            this->dSaveZAngle = this->p_cf_kEngineKernel->getZAngle();
            this->dSavePRatio = this->p_cf_kEngineKernel->getPRatio();

            // Make the switch
            this->p_cf_kEngineKernel->preferViewAngle( 0.0, 0.0, FALSE );
            this->p_cf_kEngineKernel->preferPerspective( 0.0, TRUE );
            this->b3DMode = FALSE;
        }   // End of screen to insure we were IN 3D Mode.
            // Now, of course, we are in 2D Mode.
    }
    else
    // bNewState == TRUE implies that the user wants
    // to go FROM 2D Mode TO 3D Mode:
    {
        // But don't switch if we're already in 3D Mode!
        if ( this->b3DMode == FALSE )
        {
            // Restore any saved view params as the current state:
            this->p_cf_kEngineKernel->preferViewAngle(  this->dSaveXAngle,
                                                        this->dSaveYAngle,
                                                        FALSE );
            this->p_cf_kEngineKernel->preferPerspective( this->dSavePRatio,
                                                         TRUE );
            this->b3DMode = TRUE;
        }   // End of screen to insure we were IN 2D Mode.
            // Now, of course, we are in 3D Mode.
    }

    // Force a geometry update to occur...
    this->autoConfigure();
    this->applyGeomXform();

    // We USED to do this...
    /*
    this->p_cf_cmodlUserData->initiateDrawOp();
     */
    // But now we leave it up to the managing object to
    // decinde when it wants to signal a draw op...

    return;
    
}   // end of cf_ChartEngine::disp3DMode( BOOL  bNewState )


void
    cf_ChartEngine
        ::dispBitmapChartWall( BOOL bNewState ) 
            { this->bDoWeDrawBitmapWall = bNewState; }

BOOL
    cf_ChartEngine
        ::dispBitmapChartWall() 
            { return this->bDoWeDrawBitmapWall; }

BOOL
    cf_ChartEngine
        ::dispBitmapWallPaper()
            { return this->bDoWeDrawWallPaper; }

void
    cf_ChartEngine
        ::dispBitmapWallPaper( BOOL bNewState ) 
            { this->bDoWeDrawWallPaper = bNewState; }

void
    cf_ChartEngine
        ::dispFaceBack( BOOL bNewState ) 
            { this->bDoWeDrawBackFace = bNewState; }
BOOL
    cf_ChartEngine
        ::dispFaceBack()
            { return this->bDoWeDrawBackFace; }
void
    cf_ChartEngine
        ::dispFaceBottom( BOOL bNewState ) 
            { this->bDoWeDrawBottomFace = bNewState; }
BOOL
    cf_ChartEngine
        ::dispFaceBottom()
            { return this->bDoWeDrawBottomFace; }
void
    cf_ChartEngine
        ::dispFaceLeft( BOOL bNewState ) 
            { this->bDoWeDrawLeftFace = bNewState; }
BOOL
    cf_ChartEngine
        ::dispFaceLeft()
            { return this->bDoWeDrawLeftFace; }


void
    cf_ChartEngine
        ::dispForceBackBorder( BOOL bNewState )
            { this->bForceRenderOfBackBorder = bNewState; }
BOOL
    cf_ChartEngine
        ::dispForceBackBorder()
            { return this->bForceRenderOfBackBorder; }

void
    cf_ChartEngine
        ::dispForceBackColor( BOOL bNewState )
            { this->bForceRenderOfBackColorRect = bNewState; }
BOOL
    cf_ChartEngine
        ::dispForceBackColor()
            { return this->bForceRenderOfBackColorRect; }


void
    cf_ChartEngine
        ::dispWireFrameMode( BOOL   bNewState )
            { this->bInWireFrameMode = bNewState; }
BOOL
    cf_ChartEngine
        ::dispWireFrameMode()
            { return this->bInWireFrameMode; }

void
    cf_ChartEngine
        ::dispZoomMode( BOOL bNewState )
            { this->p_cf_kEngineKernel->setZoomMode( bNewState ); }

BOOL
    cf_ChartEngine
        ::dispZoomMode()
            { return this->p_cf_kEngineKernel->setZoomMode(); }                         


void
    cf_ChartEngine
        ::draw( bf_Display *p_bf_dspOutputDisplay )
{
    // get a LOCK on the Device Context for Rendering !!!
    p_bf_dspOutputDisplay->dsp_lock();

    // If this is the FIRST time we are rendering...
    if ( bHasEverRendered_ == FALSE )
    {   // ...let's make SURE our objects are synched with the display!
        this->applyDispMetrics();
        bHasEverRendered_ = TRUE;
    }

    //----------------------------------------------------------
    // If we are not in "wire-frame" mode" we will see if we  
    // need to render any display "background" items, such as:
    //  (1) a bitmap for background "wall paper",
    //  (2) a FORCED display-background filled-rectangle or,
    //  (3) a FORCED display-background hollow-border polyline.
    //----------------------------------------------------------
    if ( this->bInWireFrameMode == FALSE )
    {
        // Measure the current display, because almost anything
        // we do here will want to know these dimensions...
        cf_2DRect  cf_2drDisplaySize;
        p_bf_dspOutputDisplay->dsp_getVisible( cf_2drDisplaySize ); 
                    // Our "background" rectangle is now defined...

        // Shall we render a "background" bitmap?
        if ( this->bDoWeDrawWallPaper == TRUE ) 
        {
            // If we've GOT one, then load it and render it
            if ( this->bitmapWallPaperFile() == TRUE ) 
            {
                int iWidth = cf_2drDisplaySize.width();
                int iHeight = cf_2drDisplaySize.height();
    
                this->p_bf_2dbmpWallPaper 
                    = new bf_2DBitmap( p_bf_dspOutputDisplay, 
                                        this->cf_strWallPaperFile );
                // Show it!...
                p_bf_dspOutputDisplay
                    ->dsp_bitmap(   this->p_bf_2dbmpWallPaper, 
                                    cf_2DPoint( 0,0 ),// placement
                                    cf_2DRect(  0, 0, // Size to top/left  X/Y,
                                                iWidth,
                                                iHeight) );
                                                // Size to Bottom Right X/Y
                //...then Blow it!
                delete this->p_bf_2dbmpWallPaper;
                this->p_bf_2dbmpWallPaper = NULL;
                        // reset to a measurable state...
         
            }  // End of screen for existance of bitmap file.
        }   // End of screen for bitmap wallpaper to be drawn.

        // Force the rendering of a background color-filled rectangle?
        else if ( this->bForceRenderOfBackColorRect == TRUE )
                // (only if we haven't already rendered a bitmap.)
        {
            //...our background rect is already defined... so render it!
            p_cf_datbDispBack->pushPenAttribs( p_bf_dspOutputDisplay );
            p_cf_datbDispBack->pushBrushAttribs( p_bf_dspOutputDisplay );
            p_bf_dspOutputDisplay->dsp_rectangle( cf_2drDisplaySize );
            p_cf_datbDispBack->popBrushAttribs( p_bf_dspOutputDisplay );
            p_cf_datbDispBack->popPenAttribs( p_bf_dspOutputDisplay );
        }

        // Force the rendering of a background non-filled polyline?
        if ( this->bForceRenderOfBackBorder == TRUE )
        {
            // We need to tweak the bottom-right dimension of the
            // background rect to avoid clipping!
            int iRight = cf_2drDisplaySize.right() - 1;
            int iBottom = cf_2drDisplaySize.bottom() - 1;
            cf_2DRect  cf_2drRenderSize( 0, 0, iRight, iBottom );
            
            p_cf_datbDispBack->pushPenAttribs( p_bf_dspOutputDisplay );
            p_bf_dspOutputDisplay->dsp_polyline( cf_2drRenderSize );
            p_cf_datbDispBack->popPenAttribs( p_bf_dspOutputDisplay );
        }
        
    }   // End of screen for wire-frame mode.

    // Now let the rest of the engine components do their tricks !!!
    if ( this->b3DMode == TRUE )
        this->draw3D( p_bf_dspOutputDisplay );
    else
        this->draw2D( p_bf_dspOutputDisplay );

    // We're done -- Release the lock !!!
    p_bf_dspOutputDisplay->dsp_unlock();
    return;

}   // end of cf_ChartEngine::draw( bf_Display * )



void
    cf_ChartEngine
        ::draw2D( bf_Display *p_bf_dspOutputDisplay )
{
    //
    // If we're in Wire Frame Mode -- Do minimal draw ops,
    // and Bail Out ASAP!
    // 
    if ( this->bInWireFrameMode == TRUE ) 
    {
        if ( this->bDoWeDrawLeftFace == TRUE ) 
            this->cf_3dpgLeftFace.drawBorder( p_bf_dspOutputDisplay );
        if ( this->bDoWeDrawBackFace == TRUE ) 
            this->cf_3dpgBackFace.drawBorder( p_bf_dspOutputDisplay );
        if ( this->bDoWeDrawBottomFace == TRUE ) 
            this->cf_3dpgBottomFace.drawBorder( p_bf_dspOutputDisplay );
    }
    else
    {
        //
        // Draw The Chart's faces -- giving the user something
        // to look at whilst any derived classes do their rendering...
        //
        if ( this->bDoWeDrawLeftFace == TRUE ) 
            this->cf_3dpgLeftFace.draw3D( p_bf_dspOutputDisplay );
        if ( this->bDoWeDrawBackFace == TRUE ) 
            this->cf_3dpgBackFace.draw3D( p_bf_dspOutputDisplay );
        if ( this->bDoWeDrawBottomFace ==  TRUE ) 
            this->cf_3dpgBottomFace.draw3D( p_bf_dspOutputDisplay );
    }
    
    return;

}   // end of cf_ChartEngine::draw2D(...);


void
    cf_ChartEngine
        ::draw3D( bf_Display *p_bf_dspOutputDisplay )
{
    //
    // Avoid duplicating code by using existing functionality...
    //
    cf_ChartEngine::draw2D( p_bf_dspOutputDisplay );
    
    //
    // If we're in 3D Mode, we might also want 
    // to draw the ChartBitmapWall wall:
    //
    if ( this->bDoWeDrawBitmapWall == TRUE ) 
    {
        if ( this->p_cf_3dbmpProjectedBitmap->isFileSet() == TRUE ) 
        {
            this->p_cf_3dbmpProjectedBitmap
                                ->draw3D( p_bf_dspOutputDisplay );
            this->cf_3dpgLeftFace.drawBorder( p_bf_dspOutputDisplay );
        }
    }
    return;
    
}   // end of cf_ChartEngine::draw3D(...);



//
// Member maintenance function:
//
void
    cf_ChartEngine
        ::engineKernel( cf_Kernel *p_cf_kUseThisKernel )
{ 
    this->p_cf_kEngineKernel = p_cf_kUseThisKernel; 

    // Pass the new pointer to any other kernel-dependent 
    // objects we control!...
    if ( this->p_cf_3dbmpProjectedBitmap != NULL )
            this->p_cf_3dbmpProjectedBitmap
                    ->setKernel( this->p_cf_kEngineKernel );
                    // from directly above!

}   // Notice we do NOT do a delete on the kernel pointer,
    // unlike the model, because we are not the KEEPERS
    // of the kernel, we are only the USERS of it
    // (via this pointer).

    
cf_Kernel*
     cf_ChartEngine
        ::engineKernel()
            { return this->p_cf_kEngineKernel; }

cf_ChartModel*
    cf_ChartEngine
        ::engineModel()
            { return this->p_cf_cmodlUserData; }

//
// Get/set Index Method IDs as follows:
//
void
    cf_ChartEngine
        ::indexMethod( int iIndexMethodID )
{
    switch ( iIndexMethodID )
    {
        case CF_ID_YVALS_ON_NAT_NS:
        case CF_ID_YVALS_AT_INT_IS:
        case CF_ID_YVALS_AT_USR_XS:
            this->iIndexID = iIndexMethodID;
            break;
        
        default:
            this->iIndexID = CF_ID_YVALS_ON_NAT_NS;
            break;
    }
}

int
    cf_ChartEngine
        ::indexMethodIs()
            { return this->iIndexID; }


//----------------------------------------------------
// Protected virtual member function called by c-tor.
// Inits chart "3D Box" Geometry objects/member vars:
//----------------------------------------------------
void
    cf_ChartEngine
        ::init3DBoxETC() 
{
    double      a_d4Xs[ 4 ];
    double      a_d4Ys[ 4 ];
    double      a_d4Zs[ 4 ];
    cf_3DPoint  *p_cf_3dpNew = NULL;
                // Used to generate the faces of the 3D Box, 
    
    // Set up the LEFT Face:
    this->p_cf_kEngineKernel->copyLeftFaceCoords( a_d4Xs, a_d4Ys, a_d4Zs );
    for ( i = 0; i < 4; i++ ) 
    {
        p_cf_3dpNew = new cf_3DPoint( a_d4Xs[i], a_d4Ys[i], a_d4Zs[i] );
        this->cf_3dpgLeftFace.addPoint( p_cf_3dpNew );
    }
    this->cf_3dpgLeftFace.applyGeomXform( this->p_cf_kEngineKernel );
                            // Projection xform will wait until AFTER
                            // we know projection state and params...

    // Set up the BACK Face:
    this->p_cf_kEngineKernel->copyBackFaceCoords( a_d4Xs, a_d4Ys, a_d4Zs );
    for ( i = 0; i < 4; i++ )
    {
        p_cf_3dpNew = new cf_3DPoint( a_d4Xs[i], a_d4Ys[i], a_d4Zs[i] );
        this->cf_3dpgBackFace.addPoint( p_cf_3dpNew );
    }
    this->cf_3dpgBackFace.applyGeomXform( this->p_cf_kEngineKernel );
                            // As above, Proj/xform waits for later...

    // Set up the BOTTOM Face:
    this->p_cf_kEngineKernel->copyBottomFaceCoords( a_d4Xs, a_d4Ys, a_d4Zs );
    for ( i = 0; i < 4; i++ )
    {
        p_cf_3dpNew = new cf_3DPoint( a_d4Xs[i], a_d4Ys[i], a_d4Zs[i] );
        this->cf_3dpgBottomFace.addPoint( p_cf_3dpNew );
    }
    this->cf_3dpgBottomFace.applyGeomXform( this->p_cf_kEngineKernel );
                            // As above, Proj/xform waits for later...

    // Now init members vars which hold the kernel's 3D Box XYZ
    // Min Max Coords -- for later use in re-proportion ops, etc:
    this->d3DBoxXMin = this->p_cf_kEngineKernel->getXMin();
    this->d3DBoxXMax = this->p_cf_kEngineKernel->getXMax();

    this->d3DBoxYMin = this->p_cf_kEngineKernel->getYMin();
    this->d3DBoxYMax = this->p_cf_kEngineKernel->getYMax();

    this->d3DBoxZMin = this->p_cf_kEngineKernel->getZMin();
    this->d3DBoxZMax = this->p_cf_kEngineKernel->getZMax();

    return;

}   // end of cf_ChartEngine::init3DBoxETC()


//
// Protected virtual member function called by c-tor
// to Init chart grid/tick mark series.
//
void
    cf_ChartEngine
        ::initDrawingAttribs()
{
    // Init overall Draw-Mode attribs:
    this->b3DMode                       = TRUE;
    this->bInWireFrameMode              = FALSE;

    this->bForceRenderOfBackColorRect   = FALSE;
    this->bForceRenderOfBackBorder      = FALSE;

    this->bDoWeDrawLeftFace             = TRUE;
    this->bDoWeDrawBackFace             = TRUE;
    this->bDoWeDrawBottomFace           = TRUE;

    // Allocate and Init the 3D Box Face Display Attribs:
    this->p_cf_datbBackFace = new cf_DispAttribs();
    this->p_cf_datbBackFace->stylePenWidth( 1 );
    this->p_cf_datbBackFace->colorBrush( CF_WHITE );
    this->cf_3dpgBackFace.dispAttribsPointTo( this->p_cf_datbBackFace );

    this->p_cf_datbBottomFace = new cf_DispAttribs();
    this->p_cf_datbBottomFace->stylePenWidth( 1 );
    this->p_cf_datbBottomFace->colorBrush( CF_GRAY );
    this->cf_3dpgBottomFace.dispAttribsPointTo( this->p_cf_datbBottomFace );

    this->p_cf_datbLeftFace = new cf_DispAttribs();
    this->p_cf_datbLeftFace->stylePenWidth( 1 );
    this->p_cf_datbLeftFace->colorBrush( CF_WHITE );
    this->cf_3dpgLeftFace.dispAttribsPointTo( this->p_cf_datbLeftFace );

    this->p_cf_datbDispBack = new cf_DispAttribs();
    this->p_cf_datbDispBack->stylePenWidth( 1 );
    this->p_cf_datbDispBack->colorBrush( CF_LITE_GRAY );

    // Default is NOT to display background bitmaps:
    this->cf_strWallPaperFile = "";
    this->bDoWeDrawWallPaper = FALSE;
    this->p_bf_2dbmpWallPaper = NULL;  // a measurable state!...

    // Default is NOT to display chart 3D bitmap/wall(s):
    this->bDoWeDrawBitmapWall = FALSE;
    this->p_cf_3dbmpProjectedBitmap
            = new cf_3DBitmap( this->p_cf_kEngineKernel );
            // But we will NEW the pointer -- for now...
            // NB: The 3D Bitmap Wall source bitmap file name
            // is kept INSIDE the cf_3dbmpProjectedBitmap object.
    return;

}   // end of cf_ChartEngine::initDrawingAttribs()



//
// Protected virtual member function called by c-tor
// to Init chart grid/tick mark series.
//
void
    cf_ChartEngine::
        initGridTickSeries()
{
    //
    // This BASE chart engine class will instantiate ALL
    // the possible Grid/Tick Series to NULL.
    //
    // Derived Chart classes will instantiate the Grid/Tick
    // series that they need, and if a user calls an access
    // methid on an un-instantiated Grid/Tick series, the
    // access method SHOULD instantiate a NEW Grid/Tick
    // series so the user will NOT access a NULL pointer...
    //
    this->p_cf_3dgtsXonBackFace     = NULL;
    this->p_cf_3dgtsXonBottomFace   = NULL;
    this->p_cf_3dgtsYonLeftFace     = NULL;
    this->p_cf_3dgtsYonBackFace     = NULL;
    this->p_cf_3dgtsZonLeftFace     = NULL;
    this->p_cf_3dgtsZonBottomFace   = NULL;
    this->p_cf_3dgtsPolarOnFloor    = NULL;
    return;

}   // end of cf_ChartEngine::initGridTickSeries()



//
// This protected member method is called by the c-tor
// and modularizes the initialization of the chart engine's
// dynamic data arrays of pointers to pictObejcts,
// and associated member vars.
//
void
    cf_ChartEngine::
    initObjectDynamicStorage()
{
    this->iNumCompoObjects = 0;
    this->da_p_cf_objCompoObjects.growBy( 10 );

    this->iNumFaceObjects = 0;
    this->da_p_cf_objFaceObjects.growBy( 100 );

    this->iNumSpaceObjects = 0;
    this->da_p_cf_objSpaceObjects.growBy( 100 );

    this->iNumMarkerObjects = 0;
    this->da_p_cf_objMarkerObjects.growBy( 100 );

}   // end of cf_ChartEngine::initObjectDynamicStorage()


void
    cf_ChartEngine
        ::initTextMarkers()
            { this->p_cf_tmmTitlesAxes  = NULL;
              this->p_cf_3dtmsUserLegends  = NULL; }


cf_TitleMarkerMgr*
    cf_ChartEngine
        ::markerMgrTitlesAxes()
{
    if ( this->p_cf_tmmTitlesAxes == NULL )
    {   // Which it WILL the first time an access op is run!...
        this->p_cf_tmmTitlesAxes = new cf_TitleMarkerMgr( this );
        this->addMarkerObject( this->p_cf_tmmTitlesAxes );
    }
    return this->p_cf_tmmTitlesAxes;
}           // which SHOULD be A-OK to go with!...


cf_3DTextMarkerSeries*
    cf_ChartEngine
        ::markerMgrUserLegends()
{
    if ( this->p_cf_3dtmsUserLegends == NULL )
    {   // Which it WILL the first time an access op is run!...
        this->p_cf_3dtmsUserLegends = new cf_3DTextMarkerSeries( this );
        this->addMarkerObject( this->p_cf_3dtmsUserLegends );
    }
    return this->p_cf_3dtmsUserLegends;
}           // which SHOULD be A-OK to go with!...



void
    cf_ChartEngine
        ::measureDataMinMax()
{
    this->dChartXRangeMin = 0.0;
    this->dChartXRangeMax = 0.0;

    this->dChartYRangeMin = 0.0;
    this->dChartYRangeMax = 0.0;

    this->dChartZRangeMin = 0.0;
    this->dChartZRangeMax = 0.0;

}   // end of cf_ChartEngine::measureDataMinMax()

void
    cf_ChartEngine
        ::measureVirtMinMax()
{
    this->dVirtXRangeMin = this->p_cf_kEngineKernel->getXMin();
    this->dVirtYRangeMin = this->p_cf_kEngineKernel->getYMin();

    this->dVirtZRangeMin = this->p_cf_kEngineKernel->getZMin();
    this->dVirtXRangeMax = this->p_cf_kEngineKernel->getXMax();

    this->dVirtYRangeMax = this->p_cf_kEngineKernel->getYMax();
    this->dVirtZRangeMax = this->p_cf_kEngineKernel->getZMax();

}   // end of cf_ChartEngine::measureVirtMinMax()



//----------------------------------------------------------------
// Allow user classes to over-ride any default positioning of
// a 3D Point and specify a new position as an X/Y/Z percentage 
// away from the CENTER of the 3D Box of the chart.  Positive 
// OR negative percentages are allowed.
//----------------------------------------------------------------
BOOL
    cf_ChartEngine
        ::positPointAt( cf_3DPoint  *p_cf_3dpTargetPoint,
                        double      dPctWidthFromCtr,
                        double      dPctHeightFromCtr,
                        double      dPctDepthFromCtr,
                        BOOL        bCorrectOriginOffset )
{
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( p_cf_3dpTargetPoint != NULL )
    {
        // Measure the Current Chart's 3D Box Height and Width:
        double  dChartXMax = this->p_cf_kEngineKernel->getXMax();
        double  dChartYMax = this->p_cf_kEngineKernel->getYMax();
        double  dChartZMax = this->p_cf_kEngineKernel->getZMax();

        // Calc XY placement values from the user's percent param:
        double dNewX = dChartXMax * dPctWidthFromCtr / 100.0;
        double dNewY = dChartYMax * dPctHeightFromCtr / 100.0;
        double dNewZ = dChartZMax * dPctDepthFromCtr / 100.0;
        
        // Re-position and transform the point:
        p_cf_3dpTargetPoint->dataXYZ( dNewX, dNewY, dNewZ );

        if ( bCorrectOriginOffset == TRUE )
            p_cf_3dpTargetPoint
                ->applyTranslateOp( - this->dChartXOrigin,
                                    - this->dChartYOrigin,
                                    - this->dChartZOrigin );
                                    // point now re-positioned...
                                                
        //p_cf_3dpTargetPoint->applyGeomXform( this->p_cf_kEngineKernel );
        //p_cf_3dpTargetPoint->applyProjXform( this->p_cf_kEngineKernel );
                                    // point now transformed!
                                        
        bStatus = TRUE;
    }
    return bStatus;

}   // end of cf_ChartEngine::positPointAt( point, 3 XYZ params )


void
    cf_ChartEngine
        ::preferMarginTop( int iMarginInTwips )
    { this->p_cf_kEngineKernel->setTopMarginTwips( iMarginInTwips ); }

void
    cf_ChartEngine
        ::preferMarginBottom(   int iMarginInTwips )
    { this->p_cf_kEngineKernel->setBottomMarginTwips( iMarginInTwips ); }

void
    cf_ChartEngine
        ::preferMarginLeft( int iMarginInTwips )
    { this->p_cf_kEngineKernel->setLeftMarginTwips( iMarginInTwips ); }

void
    cf_ChartEngine
        ::preferMarginRight( int iMarginInTwips )
    { this->p_cf_kEngineKernel->setRightMarginTwips( iMarginInTwips ); }

void
    cf_ChartEngine
        ::preferMargins(    int iTopMarginInTwips,
                            int iBottomMarginInTwips,
                            int iLeftMarginInTwips,
                            int iRightMarginInTwips )
{
    this->p_cf_kEngineKernel->setTopMarginTwips( iTopMarginInTwips );
    this->p_cf_kEngineKernel->setBottomMarginTwips( iBottomMarginInTwips );
    this->p_cf_kEngineKernel->setLeftMarginTwips( iLeftMarginInTwips );
    this->p_cf_kEngineKernel->setRightMarginTwips( iRightMarginInTwips );
}


void
    cf_ChartEngine
        ::preferPerspective( double dPercentPerspective )
{ this->p_cf_kEngineKernel->preferPerspective( dPercentPerspective, TRUE ); }

void
    cf_ChartEngine
        ::preferPerspectiveMax()
{
    this->p_cf_kEngineKernel
                ->preferPerspective( CF_MAX_P_RATIO_PERCENT/100.0, TRUE );
}

void
    cf_ChartEngine
        ::preferPerspectiveNormal()
{
    this->p_cf_kEngineKernel
                ->preferPerspective( CF_NORMAL_P_RATIO_PERCENT/100.0, TRUE );
}

void
    cf_ChartEngine
        ::preferPerspectiveZero()
{
    this->p_cf_kEngineKernel
                ->preferPerspective( CF_ZERO_P_RATIO_PERCENT/100.0, TRUE );
}


void
    cf_ChartEngine
        ::preferProportions(    double  dPrefHWPercentRatio,
                                double  dPrefDHPercentRatio,
                                BOOL    bPerformAutoConfigure )
                                        // default param = TRUE );
//--------------------------------------------------------------------
// Set PREFERRED (i.e. you might NOT get it IF it's gonna make
// a problem!) 3D Box Proportions -- HW is Height/Width Ratio,
// and DH is Depth/Height Ratio:
//--------------------------------------------------------------------
{
    this->dPrefChartHWRatio = dPrefHWPercentRatio;
    this->dPrefChartDHRatio = dPrefDHPercentRatio;

    // Re-set the preferred Y and Z dimensions of the kernel
    // from the proportion ratios applied to the X dimension.
    double  dPrefChartXDim  = this->p_cf_kEngineKernel->getXMax() * 2.0;
                            // ...from simple 3D Box symmetry.

    double  dPrefChartYDim  = dPrefChartXDim * this->dPrefChartHWRatio;
                            // I.e. via the Height-to-Width Ratio.

    double  dPrefChartZDim  = dPrefChartYDim * this->dPrefChartDHRatio;
                            // I.e. via the Depth-to-Height Ratio.

    this->p_cf_kEngineKernel->preferDimensions( dPrefChartXDim,
                                                dPrefChartYDim,
                                                dPrefChartZDim,
                                                TRUE  );
                                                // re-calcs geom-matrix
    // Should we auto-configure?
    if ( bPerformAutoConfigure == TRUE ) this->autoConfigure();
                                         // OK -- make it so !!! ...
    return;
}


void
    cf_ChartEngine
        ::preferViewAngle(  double dSetXAngle,
                            double dSetYAngle )
{ this->p_cf_kEngineKernel->preferViewAngle( dSetXAngle, dSetYAngle, TRUE ); }


int
    cf_ChartEngine
        ::typeID()
{ return this->iEngineID; }

BOOL
    cf_ChartEngine
        ::typeBarColStack()
{ return this->bIsEngineBarColStack; }

BOOL
    cf_ChartEngine
        ::typeCartesian()
{ return this->bIsEngineCartesian; }

BOOL
    cf_ChartEngine
        ::typePolar()
{ return this->bIsEnginePolar; }

void
    cf_ChartEngine
        ::updateAxisGridTickSeries()
{}// All "real" work takes place in the derived engine types


void
    cf_ChartEngine
        ::updateAxisRangeBufferPoints()
{}// All "real" work takes place in the derived engine types



void
    cf_ChartEngine
        ::updateDisplayMetrics( bf_Display *p_bf_dspHasChanged )
{
    // Update our engine-kernel according to the new display-state:
    p_bf_dspHasChanged->dsp_lock();
            // Establish a stable device-context for our ops:
    this->p_cf_kEngineKernel->updateDisplayMetrics( p_bf_dspHasChanged );
            // Kernel display projection info is now synched w/ display.
    p_bf_dspHasChanged->dsp_unlock();
            // Let's release the lock we established as SOON as possible!

    // Now update our collection of objects...
    this->applyProjXform();
            //...with use the kernel's re-calc'ed projection state:
    this->applyDispMetrics();
            //...or the kernel's re-measured display-area state.
}


void
    cf_ChartEngine
        ::updateFromCurrentModel()
//--------------------------------------------------------------------------
// This BASE update-method implements modular functionality that is almost
// ALWAYS called from other DERIVED update-methods (based on type ID, etc.).
//--------------------------------------------------------------------------
{
    // Get rid of any FACE objects we generated!
    if ( this->iNumFaceObjects >= 1 )
    {
        for ( i = 1; i <= this->iNumFaceObjects; i++ )
            if ( this->da_p_cf_objFaceObjects[ i ] != NULL )
                delete ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] );
        this->iNumFaceObjects = 0;
    }

    // Get rid of any SPACE objects we generated!
    if ( this->iNumSpaceObjects >= 1 )
    {
        for ( i = 1; i <= this->iNumSpaceObjects; i++ )
            if ( this->da_p_cf_objSpaceObjects[ i ] != NULL )
                delete ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] );
        this->iNumSpaceObjects = 0;
    }

    // Reset any ACTIVE Grid/Tick Mark Series:
    if ( this->p_cf_3dgtsXonBackFace != NULL )
        this->p_cf_3dgtsXonBackFace->reset();

    if ( this->p_cf_3dgtsXonBottomFace != NULL )
        this->p_cf_3dgtsXonBottomFace->reset();

    if ( this->p_cf_3dgtsYonLeftFace != NULL )
        this->p_cf_3dgtsYonLeftFace->reset();

    if ( this->p_cf_3dgtsYonBackFace != NULL )
        this->p_cf_3dgtsYonBackFace->reset();

    if ( this->p_cf_3dgtsZonLeftFace != NULL )
        this->p_cf_3dgtsZonLeftFace->reset();

    if ( this->p_cf_3dgtsZonBottomFace != NULL )
        this->p_cf_3dgtsZonBottomFace->reset();

    if ( this->p_cf_3dgtsPolarOnFloor != NULL )
        this->p_cf_3dgtsPolarOnFloor->reset();

    // Reset our virtual scaling metrics:
    this->measureDataMinMax();
    this->measureVirtMinMax();

    return;

}   // end of cf_ChartEngine::updateFromCurrentModel()


void
    cf_ChartEngine
        ::updateYValsOnNatNs()
            {}// All "real" work takes place in the derived engine types
void    
    cf_ChartEngine
        ::updateYValsAtIntIs()
            {}// All "real" work takes place in the derived engine types


void
    cf_ChartEngine
        ::updateTextBackColors( const bf_Color &cr_bf_clrNewColor )
{
    // Update the Engine's Grid/Tick Mark Series:
    if ( this->p_cf_3dgtsXonBackFace != NULL )
        this->p_cf_3dgtsXonBackFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsXonBottomFace != NULL )
        this->p_cf_3dgtsXonBottomFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsYonLeftFace != NULL )
        this->p_cf_3dgtsYonLeftFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsYonBackFace != NULL )
        this->p_cf_3dgtsYonBackFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsZonLeftFace != NULL )
        this->p_cf_3dgtsZonLeftFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsZonBottomFace != NULL )
        this->p_cf_3dgtsZonBottomFace->colorBack( cr_bf_clrNewColor ); 

    if ( this->p_cf_3dgtsPolarOnFloor != NULL )
        this->p_cf_3dgtsPolarOnFloor->colorBack( cr_bf_clrNewColor ); 

    // Update the Engine's Title-Marker-Manager:
    if ( this->p_cf_tmmTitlesAxes != NULL )
        this->p_cf_tmmTitlesAxes->colorBack( cr_bf_clrNewColor );  

    // Update the Engine's "User-Legend" 3D-Text-Marker-Series:
    if ( this->p_cf_3dtmsUserLegends != NULL )
        this->p_cf_3dtmsUserLegends->colorBack( cr_bf_clrNewColor );  

    return;
        
}   // end of updateTextBackColors( bf_Color bf_clrNewColor )
        


//
// End of class cf_ChartEngine Implementation!
//    


//////////////////////////////////////
// END of File: C1_ENGIN.CPP
//////////////////////////////////////





