//////////////////////////////////////
// START of File: CO1_VOBJ.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/c1_vobj.hpp> 
            // relative to the IDE's (global) include path: ~/cfrz

//
// Implement the Chart FrameWork virtual "Object" base class.  
//
cf_Virt3DObj
    ::cf_Virt3DObj() 
{
    this->p_cf_datbForDrawOps = NULL;
    this->bWereDispAttribsSet   = FALSE;
    this->bWereDispAttribsNewed = FALSE;

    this->p_cf_tatbForTextOps = NULL;
    this->bWereTextAttribsSet   = FALSE;
    this->bWereTextAttribsNewed = FALSE;

    this->bIgnorePushBrush  = FALSE;
    this->bIgnorePushFont   = FALSE;
    this->bIgnorePushPen    = FALSE;

    this->bDispUsingFSORT   = FALSE;
    this->bDispUsingGSORT   = FALSE;
    this->bDispUsingHSE     = FALSE;

    this->bDispWireFrame            = FALSE;
    this->bAppliedKernelDispMetrics = FALSE;
}


cf_Virt3DObj
    ::cf_Virt3DObj( cf_DispAttribs *p_attrToDrawWith )
{
    this->p_cf_datbForDrawOps = p_attrToDrawWith;
    this->bWereDispAttribsSet = TRUE;
    this->bWereDispAttribsNewed = FALSE;

    this->p_cf_tatbForTextOps = NULL;
    this->bWereTextAttribsSet   = FALSE;
    this->bWereTextAttribsNewed = FALSE;

    this->bIgnorePushBrush = FALSE;
    this->bIgnorePushFont = FALSE;
    this->bIgnorePushPen = FALSE;

    this->bDispUsingFSORT   = FALSE;
    this->bDispUsingGSORT   = FALSE;
    this->bDispUsingHSE     = FALSE;

    this->bDispWireFrame            = FALSE;
    this->bAppliedKernelDispMetrics = FALSE;
}



cf_Virt3DObj
    ::~cf_Virt3DObj()
{
    if ( this->bWereDispAttribsSet == TRUE )
        if ( this->bWereDispAttribsNewed == TRUE )
            if ( this->p_cf_datbForDrawOps != NULL )
                delete this->p_cf_datbForDrawOps;
}


cf_Virt3DObj&
    cf_Virt3DObj
        ::operator=( const cf_Virt3DObj &cr_cf_vobjSource )
{
    //----------------------------------------------------
    // Copy this virtual object's DISPLAY Attribute State:
    //----------------------------------------------------
    if ( cr_cf_vobjSource.bWereDispAttribsSet == TRUE )
    // I.e. -- does the target even HAVE display attributes set?
    {
        // If the target new()ed its OWN "display attrib" object,
        // (i.e. it's NOT using someone else's display attribute)...
        if ( cr_cf_vobjSource.bWereDispAttribsNewed == TRUE )
        {
            // Make ourselves ready to do a CONTENT copy
            if ( this->bWereDispAttribsNewed == FALSE )
            {   // we need to allocate our OWN new disp-attribs!
                this->p_cf_datbForDrawOps = new cf_DispAttribs();
                this->bWereDispAttribsNewed = TRUE;
                // Keep track of what we just did (for the d-tor!);
            }
            // ...now we're ready to do a CONTENT copy:
            *(this->p_cf_datbForDrawOps)
                        = *(cr_cf_vobjSource.p_cf_datbForDrawOps);
            this->bWereDispAttribsSet = TRUE;
        }
        else
        // The target did NOT new() its own display attribs -- it's
        // just pointing to someone else's -- let's point there, too!
        {
            this->p_cf_datbForDrawOps
                        = cr_cf_vobjSource.p_cf_datbForDrawOps;
            this->bWereDispAttribsNewed = FALSE;
            this->bWereDispAttribsSet = TRUE;
            // Keep track of what we just did (for the d-tor!);
        }
    }
    else
    // The source object's display attribs were NEVER SET!
    {
        this->p_cf_datbForDrawOps = NULL;
        this->bWereDispAttribsSet = FALSE;
        this->bWereDispAttribsNewed = FALSE;
        // We assume the default c-tor init state...
    }

    //----------------------------------------------------
    // Copy this virtual object's Text Attribute State:
    //----------------------------------------------------
    if ( cr_cf_vobjSource.bWereTextAttribsSet == TRUE )
    // I.e. -- does the target even HAVE text attributes set?
    {
        // If the target new()ed its OWN "text attrib" object,
        // (i.e. it's NOT using someone else's text attribute)...
        if ( cr_cf_vobjSource.bWereTextAttribsNewed == TRUE )
        {
            // Make ourselves ready to do a CONTENT copy
            if ( this->bWereTextAttribsNewed == FALSE )
            {   // we need to allocate our OWN new disp-attribs!
                this->p_cf_tatbForTextOps = new cf_TextAttribs();
                this->bWereTextAttribsNewed = TRUE;
                // Keep track of what we just did (for the d-tor!);
            }
            // ...now we're ready to do a CONTENT copy:
            *(this->p_cf_tatbForTextOps)
                        = *(cr_cf_vobjSource.p_cf_tatbForTextOps);
            this->bWereTextAttribsSet = TRUE;
        }
        else
        // The target did NOT new() its own text attribs -- it's
        // just pointing to someone else's -- let's point there, too!
        {
            this->p_cf_tatbForTextOps
                        = cr_cf_vobjSource.p_cf_tatbForTextOps;
            this->bWereTextAttribsNewed = FALSE;
            this->bWereTextAttribsSet = TRUE;
            // Keep track of what we just did (for the d-tor!);
        }
    }
    else
    // The source object's text attribs were NEVER SET!
    {
        this->p_cf_tatbForTextOps = NULL;
        this->bWereTextAttribsSet = FALSE;
        this->bWereTextAttribsNewed = FALSE;
        // We assume the default c-tor init state...
    }

    //-----------------------------------------
    // Finish by copying miscelenaous attribs:
    //-----------------------------------------
    this->bIgnorePushBrush = cr_cf_vobjSource.bIgnorePushBrush;
    this->bIgnorePushFont = cr_cf_vobjSource.bIgnorePushFont;
    this->bIgnorePushPen = cr_cf_vobjSource.bIgnorePushPen;

    this->bDispUsingFSORT   = cr_cf_vobjSource.bDispUsingFSORT;
    this->bDispUsingGSORT   = cr_cf_vobjSource.bDispUsingGSORT;
    this->bDispUsingHSE     = cr_cf_vobjSource.bDispUsingHSE;

    this->bDispWireFrame    = cr_cf_vobjSource.bDispWireFrame;

    this->bAppliedKernelDispMetrics
                            = cr_cf_vobjSource.bAppliedKernelDispMetrics;
    return *this;

}   // end of cf_Virt3DObj::operator=( const cf_Virt3DObj& )



void
    cf_Virt3DObj
        ::applyDispMetrics( cf_Kernel  * /*p_cf_kUseThisKernel*/ )
//-------------------------------------------------------------------------
// Called by the parent chart-engine configuration in response to display
// context changes (e.g. re-size events) imposed by the chart-engine's
// parent chart-model/chart-view.
//-------------------------------------------------------------------------
{
    // Make a note to anyone who wants to know later on...
    this->bAppliedKernelDispMetrics = TRUE;
            //... we've run this routine at least once!...
}

void
    cf_Virt3DObj
        ::applyGeomXform( cf_Kernel  * /*p_cf_kUseThisKernel*/ )
{}

void
    cf_Virt3DObj
        ::applyProjXform( cf_Kernel *p_cf_kUseThisKernel )
{
    // If we haven't yet done a synchronization to the
    // kernel's latest display metrics, let's *force* one!
    if ( this->bAppliedKernelDispMetrics == FALSE )
            this->applyDispMetrics( p_cf_kUseThisKernel );
}

void
    cf_Virt3DObj
        ::applyScaleOp( double /*dXScaleFactor*/,
                        double /*dYScaleFactor*/,
                        double /*dZScaleFactor*/ )
{}

void
    cf_Virt3DObj
        ::applyTranslateOp( double /*dXTrans*/,
                            double /*dYTrans*/,
                            double /*dZTrans*/ )
{}



//
// Get whatever we're pointing at. Note that it's wise
// to preceed this with a call to this->dispAttribsValid().
//
cf_DispAttribs*
    cf_Virt3DObj
        ::dispAttribsCurrent()
{
    if ( this->bWereDispAttribsSet == FALSE )
            this->dispAttribsNewSet();
            // which yields the valid pointer return value below:
    return this->p_cf_datbForDrawOps;
}

//
// Create a New set of "Display Attribs" to 
// be the target for this object's "Display
// Attribs" pointer! Any current new()ed
// allocation will be deleted.  D-tor will
// be notified to delete these current
// new()-ed "Disp Attribs" object.
//
cf_DispAttribs*
    cf_Virt3DObj
        ::dispAttribsNewSet()
{
    // If we made an earlier allocation, get rid of it!
    if ( this->bWereDispAttribsNewed == TRUE )
        if ( this->p_cf_datbForDrawOps != NULL )
                delete this->p_cf_datbForDrawOps;

    // Now -- we make a new set of display attribs:
    this->p_cf_datbForDrawOps = new cf_DispAttribs();
    this->bWereDispAttribsNewed = TRUE;
    this->bWereDispAttribsSet = TRUE;
        // Keep track of what we just did (for the d-tor!);

    return this->p_cf_datbForDrawOps;
}


//
// Set this object's display attribute pointer!
//
void
    cf_Virt3DObj
        ::dispAttribsPointTo( cf_DispAttribs *p_cf_datbTarget )
{
    // TRY to screen out bogosities!
    if ( p_cf_datbTarget != NULL )
    {
        // If we made an earlier allocation, get rid of it!
        if ( this->bWereDispAttribsNewed == TRUE )
            if ( this->p_cf_datbForDrawOps != NULL )
                    delete this->p_cf_datbForDrawOps;

        // Now -- all we can do is point to the new target!
        this->p_cf_datbForDrawOps = p_cf_datbTarget;
        this->bWereDispAttribsNewed = FALSE;
        this->bWereDispAttribsSet = TRUE;
            // Keep track of what we just did (for the d-tor!);
    }
    else
    // Don't set to a BOGUS pointer value!  Just IGNORE it!
    {
        this->p_cf_datbForDrawOps = NULL;
        this->bWereDispAttribsSet = FALSE;
        this->bWereDispAttribsNewed = FALSE;
        // We'll assume the (hopefully harmless)
        // default c-tor init state...
    }

    return;

}   // end of cf_Virt3DObj::dispAttribsSetTo( cf_DispAttribs * )


BOOL
    cf_Virt3DObj
        ::dispAttribsValid()
            { return this->bWereDispAttribsSet; }


BOOL
    cf_Virt3DObj
        ::dispIgnorePushBrush()
            { return this->bIgnorePushBrush; }
void
    cf_Virt3DObj
        ::dispIgnorePushBrush( BOOL bNewState )
            { this->bIgnorePushBrush = bNewState; }

BOOL
    cf_Virt3DObj
        ::dispIgnorePushFont()
            { return this->bIgnorePushFont; }
void    
    cf_Virt3DObj
        ::dispIgnorePushFont( BOOL bNewState )
            { this->bIgnorePushFont = bNewState; }

BOOL    
    cf_Virt3DObj
        ::dispIgnorePushPen()
            { return this->bIgnorePushPen; }
void    
    cf_Virt3DObj
        ::dispIgnorePushPen( BOOL bNewState )
            { this->bIgnorePushPen = bNewState; }


void
    cf_Virt3DObj
        ::dispPushBrushAttribs( bf_Display *p_bf_dspActive ) 
{
    if ( this->bDispWireFrame == FALSE )
    {
        if ( this->bIgnorePushBrush == FALSE )
        {
            if ( this->bWereDispAttribsSet == TRUE )
                p_bf_dspActive->dsp_pushBrush
                ( 
                    new bf_Brush( this->p_cf_datbForDrawOps
                                                ->colorBrush() ) 
                );
            else
                p_bf_dspActive->dsp_pushBrush
                ( 
                    new bf_Brush( CF_WHITE ) 
                );
        }
    }
    return;
}
        

void
    cf_Virt3DObj
        ::dispPushPenAttribs( bf_Display *p_bf_dspActive ) 
{
    if ( this->bDispWireFrame == FALSE )
    {
        if ( this->bIgnorePushPen == FALSE )
        {
            if ( this->bWereDispAttribsSet == TRUE )
            {
                p_bf_dspActive->dsp_pushPen
                ( 
                  new bf_Pen(this->p_cf_datbForDrawOps->colorPen(), 
                              this->p_cf_datbForDrawOps->stylePen(), 
                              this->p_cf_datbForDrawOps->stylePenWidth() )
                );
                 
            }
            else
                p_bf_dspActive->dsp_pushPen
                ( 
                    new bf_Pen(    CF_BLACK,       // color
                                    CF_PS_SOLID,    // style
                                    1 )             // width 
                );
        }
    }
    return;
}


void
    cf_Virt3DObj
        ::dispPopBrushAttribs( bf_Display *p_bf_dspTarget ) 
{
    if ( this->bDispWireFrame == FALSE )
        if ( this->bIgnorePushBrush == FALSE ) 
                delete p_bf_dspTarget->dsp_popBrush(); 
                // I.e. ONLY pop & delete IF we newed & pushed!
}


void
    cf_Virt3DObj
        ::dispPopPenAttribs( bf_Display *p_bf_dspTarget ) 
{ 
    if ( this->bDispWireFrame == FALSE )
        if ( this->bIgnorePushPen == FALSE ) 
                delete p_bf_dspTarget->dsp_popPen(); 
                // I.e. ONLY pop & delete IF we newed & pushed!
}


// Get/set Facet-Based Depth-Sorting calcs ON/OFF.
BOOL    
    cf_Virt3DObj
        ::dispUsingFSORT()
            { return this->bDispUsingFSORT; }
void    
    cf_Virt3DObj
        ::dispUsingFSORT( BOOL bNewState )
            { this->bDispUsingFSORT = bNewState; }

// Get/set Geometry-Based Depth-Sorting calcs ON/OFF.
BOOL    
    cf_Virt3DObj
        ::dispUsingGSORT()
            { return this->bDispUsingGSORT; }
void    
    cf_Virt3DObj
        ::dispUsingGSORT( BOOL bNewState )
            { this->bDispUsingGSORT = bNewState; }

// Get/set Hidden-Surface-Elimination calcs ON/OFF.
BOOL    
    cf_Virt3DObj
        ::dispUsingHSE()
            { return this->bDispUsingHSE; }
void    
    cf_Virt3DObj
        ::dispUsingHSE( BOOL bNewState )
            { this->bDispUsingHSE = bNewState; }


void
    cf_Virt3DObj
        ::dispWireFrame( BOOL   bNewState )
            { this->bDispWireFrame = bNewState; }
BOOL
    cf_Virt3DObj
        ::dispWireFrame()
            { return this->bDispWireFrame; }


//
// Define "nop" implementations of the DRAW function
// for this virtual BASE objct:
//
void
    cf_Virt3DObj
        ::draw( bf_Display * /*p_bf_dspUseThisDisplay*/ )
{}

void
    cf_Virt3DObj
        ::draw2D( bf_Display * /*p_bf_dspUseThisDisplay*/ )
{}

void
    cf_Virt3DObj
        ::draw3D( bf_Display * /*p_bf_dspUseThisDisplay*/ )
{}


//
//  Get Information on the Data Dimensions 
//  (e.g. for Tick Marking Ops) of cf_Virt3DObjs:
//
double
    cf_Virt3DObj
        ::dataXMin()
            const   
                { return 0.0; }
            
double
    cf_Virt3DObj
        ::dataXMax()
            const   
                { return 0.0; }

double
    cf_Virt3DObj
        ::dataYMin()
            const
                { return 0.0; }
            
double
    cf_Virt3DObj
        ::dataYMax() 
            const   
                { return 0.0; }

double
    cf_Virt3DObj
        ::dataZMin() 
            const   
                { return 0.0; }
            
double
    cf_Virt3DObj
        ::dataZMax()
            const
                { return 0.0; }


//---------------------------------------------------------------------
//  Get Information on the Virtual Dimensions (e.g. for Scaling Ops):
//---------------------------------------------------------------------
double
    cf_Virt3DObj
        ::virtXMin()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::virtXMax()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::virtYMin()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::virtYMax()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::virtZMin()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::virtZMax()
            const
                { return 0.0; }


//-----------------------------------------------------------------------
//  Get Info on the TRANSFORMED Dimensions (e.g. for Depth-Sorting Ops):
//-----------------------------------------------------------------------
double
    cf_Virt3DObj
        ::xformXMin()
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::xformXMax()   
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::xformYMin()   
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::xformYMax()   
            const
                { return 0.0; }

double
    cf_Virt3DObj
        ::xformZMin()   
            const
                { return 0.0; }
            
double
    cf_Virt3DObj
        ::xformZMax()   
            const
                { return 0.0; }
//
// END of cf_Virt3DObj implementation.
//

//////////////////////////////////////
// END of File: CO1_VOBJ.CPP
//////////////////////////////////////

