//////////////////////////////////////
// START of File: C2_CMODL.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/c1_cmodl.hpp>

//
// Include any other files which reference 
// neccessary #includes, #defines, defintions, 
// typedefs, class declarations, etc...
//
// The class specifications of the various
// chart-engines (which are version depeendent)
// are of PARTICULAR interest to this source file!
//
class       cf_eng_Polar;
#include    <cf_api/ce1_pcor.hpp>

class       cf_eng_Pie2D;
#include    <cf_api/ce2_pie.hpp>

class       cf_eng_CartXYZ;
#include    <cf_api/ce1_ccor.hpp>

class       cf_eng_Bar2D;
#include    <cf_api/ce2_bar.hpp>

class       cf_eng_Col2D;
#include    <cf_api/ce2_colm.hpp>

class       cf_eng_Stack2D;
#include    <cf_api/ce2_stac.hpp>

class       cf_eng_PointSer2D;
#include    <cf_api/ce2_pser.hpp>

class       cf_eng_Line2D;
#include    <cf_api/ce2_line.hpp>

class       cf_eng_Area2D;
#include    <cf_api/ce2_area.hpp>


//
//==========================================================================
// Implement 2D AND 3D chart-engine selection for the ChartFolio Framework:
//==========================================================================
//
void    
    cf_ChartModel
        ::modelEngine(  int     iChartTypeID,
                        int     iIndexMethodID,
                        BOOL    bDeleteOldEngine )
{
    // IF any kind of new/malloc has already been made for this chart-
    // model's engine, we're gonna hafta do some kind of delete op...
    cf_ChartEngine  *p_cf_engOldEngine
                    = this->p_cf_engChartEngine;
                    // We'll be testing whether or not
                    // to delete this pointer value below...

    // Keep track of the indexing method:
    this->iIndexID = iIndexMethodID;

    // The engine controls the "display model" of this chart's data...
    switch( iChartTypeID )
    {
        // Note that these cases parallel the IDs in "id_charts.h":

        // Chart Family IDs:
        case    CF_ID_BASE_CHART:
                this->p_cf_engChartEngine 
                        = new cf_ChartEngine(   this, 
                                                iChartTypeID,
                                                iIndexMethodID );
                break;

        // Polar Coordinate Engines:
        case    CF_ID_POLAR_CHART:
                this->p_cf_engChartEngine 
                        = new cf_eng_Polar( this, 
                                            iChartTypeID,
                                            iIndexMethodID );
                break;

        case    CF_ID_CARTESIAN_CHART:
                this->p_cf_engChartEngine 
                        = new cf_eng_CartXYZ(   this, 
                                                iChartTypeID,
                                                iIndexMethodID );
                break;

        case    CF_ID_COMBO_CHART:
                this->p_cf_engChartEngine 
                        = new cf_eng_CartXYZ(   this, 
                                                iChartTypeID,
                                                iIndexMethodID );
                break;

        case    CF_ID_CUSTOM_CHART:
                this->p_cf_engChartEngine 
                        = new cf_eng_CartXYZ(   this, 
                                                iChartTypeID,
                                                iIndexMethodID );
                break;

        // Polar Chart IDs:
        case    CF_ID_PIE:
        case    CF_ID_PIE_2D:
        case    CF_ID_PIE_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Pie2D( this, iIndexMethodID );
                break;

                // Note that an "index method ID" has no "real"
                // meaning for Pie Charts, so default c-tor ID 
                // value would work just as well as any....

        // Cartesian Coordinate Chart IDs:
        case    CF_ID_BAR:
        case    CF_ID_BAR_2D:
        case    CF_ID_BAR_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Bar2D( this, iIndexMethodID );
                break;

        case    CF_ID_COL:
        case    CF_ID_COL_2D:
        case    CF_ID_COL_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Col2D( this, iIndexMethodID );
                break;

        case    CF_ID_STACK:
        case    CF_ID_STACK_2D:
        case    CF_ID_STACK_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Stack2D( this, iIndexMethodID );
                break;

        case    CF_ID_POINTS:
        case    CF_ID_POINTS_2D:
        case    CF_ID_POINTS_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_PointSer2D( this, iIndexMethodID );
                break;

        case    CF_ID_LINE:
        case    CF_ID_LINE_2D:
        case    CF_ID_LINE_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Line2D( this, iIndexMethodID );
                break;

        case    CF_ID_AREA:
        case    CF_ID_AREA_2D:
        case    CF_ID_AREA_3D:
                this->p_cf_engChartEngine
                    = new cf_eng_Area2D( this, iIndexMethodID );
                break;

        case    CF_ID_RIBBON:
        case    CF_ID_RIBBON_2D:
        case    CF_ID_RIBBON_3D:
        default:
                this->p_cf_engChartEngine
                    = new cf_eng_PointSer2D( this, iIndexMethodID );
                break;

    }   // end of switch on Engine ID


    // IF requested (and by default) if there was 
    // an old engine already allocated, get rid of it:
    if( bDeleteOldEngine )
            if( p_cf_engOldEngine != NULL )   
                        delete p_cf_engOldEngine;

    // Synchronize this new engine to the chart-model's data-state:
    this->p_cf_engChartEngine->updateFromCurrentModel();
    //this->p_cf_engChartEngine->autoConfigure();

    return;

}   // end of cf_ChartModel::modelEngine( int, int, BOOL )


//    
//===============================================================
// End of SEPARATE Implementation of version-dependent
// "chart-model" modelEngine( int, int, BOOL ) Member Method.
//===============================================================
//    

//////////////////////////////////////
// END of File: C2_CMODL.CPP
//////////////////////////////////////






