//////////////////////////////////////
// START of File: CE1_CCOR.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

// Include this source file's header file:
#include    <cf_api/ce1_ccor.hpp>

// Specify any other file usage:
#include    <math.h>
            // We'll use fabs() functionality below...
            
cf_eng_CartXYZ
    ::cf_eng_CartXYZ(   cf_ChartModel   *p_cf_cmodlOwner,
                        int             iChartTypeID,
                        int             iIndexMethodID )

    : cf_ChartEngine( p_cf_cmodlOwner, iChartTypeID, iIndexMethodID )

{
    // Init the Category vars:
    this->bIsEngineCartesian = TRUE;
    this->bIsEnginePolar = FALSE;

    //
    // The "base" engine class uses an origin at the
    // "center" of the 3D Box. These "cartesian XYZ" engines
    // use an origin at the minimal X/Y/Z coords of the 3B Box:
    //
    this->configOrigins();

    //
    // Init the Kernel to a favorable state for cartesian 
    // chart views -- like an XY Plot with a little depth:
    //
    this->p_cf_kEngineKernel->preferPerspective( 0.0, FALSE );
    this->p_cf_kEngineKernel->preferViewAngle( 20, -30, FALSE );
    this->preferProportions(    1.0,    // Height-to-Width ratio
                                0.5,    // Depth-to-Height ratio
                                TRUE ); // which will also call:
                                // this->p_cf_kEngineKernel->calcGeometry();
    this->autoConfigure();

}   // end of cartesian c-tor

cf_eng_CartXYZ
    ::~cf_eng_CartXYZ() 
        {}


void
    cf_eng_CartXYZ
        ::applyVirtScalings()
{
    // Apply any neccessary Scalings to Chart Objects:
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                    ->applyScaleOp( this->dChartObjectXScaleFactor,
                                    this->dChartObjectYScaleFactor, 
                                    this->dChartObjectZScaleFactor ); 
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                    ->applyScaleOp( this->dChartObjectXScaleFactor,
                                    this->dChartObjectYScaleFactor, 
                                    this->dChartObjectZScaleFactor ); 
    return;
    
}   // end of cf_ChartEngine::applyVirtScalings()



void
    cf_eng_CartXYZ
        ::applyVirtTranslations()
{
    // Apply any neccessary Translations to chart objects:
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                ->applyTranslateOp( this->dChartObjectXTransFactor,
                                    this->dChartObjectYTransFactor, 
                                    this->dChartObjectZTransFactor ); 
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                ->applyTranslateOp( this->dChartObjectXTransFactor,
                                    this->dChartObjectYTransFactor, 
                                    this->dChartObjectZTransFactor ); 
    return;
    
}   // end of cf_ChartEngine::applyVirtTranslations()
    



void
    cf_eng_CartXYZ
        ::calcVirtScaleFactors()
{
    // Once we have MAX & MIN values established, (from this 
    // chart/engine measureVirtMinMax() member method) we can
    // calc the DELTA values, and thence the Range Ratios and
    // resulting Scale Factors...
    double  dVirtDeltaX = this->dVirtXRangeMax - this->dVirtXRangeMin; 
    double  dVirtDeltaY = this->dVirtYRangeMax - this->dVirtYRangeMin; 
    double  dVirtDeltaZ = this->dVirtZRangeMax - this->dVirtZRangeMin; 

    double  d3DBoxDeltaX = this->d3DBoxXMax - this->d3DBoxXMin;
    double  d3DBoxDeltaY = this->d3DBoxYMax - this->d3DBoxYMin;
    double  d3DBoxDeltaZ = this->d3DBoxZMax - this->d3DBoxZMin;

    this->dChartObjectXScaleFactor = 1.0;   // Unless updated below...
    #ifdef CF_ASSERT_DVALS
    //assert( fabs( dVirtDeltaX ) > CF_NUM_ZERO_IF_LT );
    #endif
    if ( fabs( dVirtDeltaX ) > CF_NUM_ZERO_IF_LT )
			this->dChartObjectXScaleFactor = d3DBoxDeltaX / dVirtDeltaX;

    this->dChartObjectYScaleFactor = 1.0;   // ditto...
    #ifdef CF_ASSERT_DVALS
    //assert( fabs( dVirtDeltaY ) > CF_NUM_ZERO_IF_LT );
    #endif
    if ( fabs( dVirtDeltaY ) > CF_NUM_ZERO_IF_LT )
    		this->dChartObjectYScaleFactor = d3DBoxDeltaY / dVirtDeltaY;

    this->dChartObjectZScaleFactor = 1.0;   // and ditto...
    #ifdef CF_ASSERT_DVALS
    //assert( fabs( dVirtDeltaZ ) > CF_NUM_ZERO_IF_LT );
    #endif
    if ( fabs( dVirtDeltaZ ) > CF_NUM_ZERO_IF_LT )
    		this->dChartObjectZScaleFactor = d3DBoxDeltaZ / dVirtDeltaZ;
    return;

}   // end of cf_eng_CartXYZ::calcVirtScaleFactors()

void
    cf_eng_CartXYZ
        ::calcVirtTransFactors()
{
    this->dChartObjectXTransFactor 
        = ( this->dChartXOrigin - this->dVirtXRangeMin )
            * this->dChartObjectXScaleFactor;

    this->dChartObjectYTransFactor 
        = ( this->dChartYOrigin - this->dVirtYRangeMin )
            * this->dChartObjectYScaleFactor;

    this->dChartObjectZTransFactor 
        = ( this->dChartZOrigin - this->dVirtZRangeMin )
            * this->dChartObjectZScaleFactor;
    return;

}   // end of cf_eng_CartXYZ::calcVirtTransFactors()



void
    cf_eng_CartXYZ
        ::configOrigins()
{
    //
    // BEFORE we configure new Chart Origins, 
    // we NEED to calculate a correction for current 
    // Chart Objects to line up with the new origin:
    //
    double  dOldXOrigin = this->dChartXOrigin;
    double  dOldYOrigin = this->dChartYOrigin;
    double  dOldZOrigin = this->dChartZOrigin;

    //
    // Update our relative chart origin w.r.t. the kernel's
    // absolute origin & dimensions:
    //
    this->dChartXOrigin = this->p_cf_kEngineKernel->getXMin();
    this->dChartYOrigin = this->p_cf_kEngineKernel->getYMin();
    this->dChartZOrigin = this->p_cf_kEngineKernel->getZMin();

    //
    // See if it's neccessary to make any adjustments:
    //
    BOOL    bIsTransNeccessary = FALSE; // For now...
    
    double  dXlateAlongX = this->dChartXOrigin - dOldXOrigin;
    double  dXlateAlongY = this->dChartYOrigin - dOldYOrigin;
    double  dXlateAlongZ = this->dChartZOrigin - dOldZOrigin;

    if ( dXlateAlongX != 0.0 
                        || dXlateAlongY != 0.0 
                                        || dXlateAlongZ != 0.0 )
            bIsTransNeccessary = TRUE;

    if ( bIsTransNeccessary == TRUE )
    // Apply neccessary translation corrections to chart objects:
    {
        for ( i = 1; i <= iNumCompoObjects; i++ )
            ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                        ->applyTranslateOp( dXlateAlongX,
                                            dXlateAlongY, 
                                            dXlateAlongZ ); 
        for ( i = 1; i <= iNumFaceObjects; i++ )
            ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                        ->applyTranslateOp( dXlateAlongX,
                                            dXlateAlongY, 
                                            dXlateAlongZ ); 
        for ( i = 1; i <= iNumSpaceObjects; i++ )
            ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                        ->applyTranslateOp( dXlateAlongX,
                                            dXlateAlongY, 
                                            dXlateAlongZ ); 
    }
    
    return;

}   // end of cf_eng_CartXYZ::configOrigins()




void
    cf_eng_CartXYZ
        ::draw2D( bf_Display *p_bf_dspFromChartView ) 
{
    //
    // Render the 3D Box using the base class:
    //
    cf_ChartEngine::draw2D( p_bf_dspFromChartView );

    //
    // Process the Face Component List ( tick marks, etc):
    //
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                                ->draw2D( p_bf_dspFromChartView );
    
    //
    // Process the Face Object List ( control lines, etc):
    //
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                                ->draw2D( p_bf_dspFromChartView );
    
    //
    // Process the Space Object List ( ribbons, etc):
    //
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                                ->draw2D( p_bf_dspFromChartView );

    //
    // Process the Chart Marker List ( ribbons, etc):
    //
    for ( i = 1; i <= iNumMarkerObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                                ->draw2D( p_bf_dspFromChartView );

    return;

}   // end of cf_eng_CartXYZ::draw2D(...)



void
    cf_eng_CartXYZ
        ::draw3D( bf_Display *p_bf_dspFromChartView ) 
{
    //
    // Render the 3D Box using the base class:
    //
    cf_ChartEngine::draw3D( p_bf_dspFromChartView );

    //
    // Process the Face Component List ( tick marks, etc):
    //
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                                ->draw3D( p_bf_dspFromChartView );
    
    //
    // Process the Face Object List ( control lines, etc):
    //
    for ( i = 1; i <= iNumFaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objFaceObjects[ i ] )
                                ->draw3D( p_bf_dspFromChartView );
    
    //
    // Process the Space Object List ( ribbons, etc):
    //
    for ( i = 1; i <= iNumSpaceObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objSpaceObjects[ i ] )
                                ->draw3D( p_bf_dspFromChartView );

    //
    // Process the Chart Marker List ( ribbons, etc):
    //
    for ( i = 1; i <= iNumMarkerObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objMarkerObjects[ i ] )
                                ->draw3D( p_bf_dspFromChartView );

    return;

}   // end of cf_eng_CartXYZ::draw3D(...)



void
    cf_eng_CartXYZ
        ::initGridTickSeries()
{
    // Activate a Target "Domain" Axis:
    cf_3DGridTickSeries *p_cf_3dgtsTargetDomain 
                        = this->axis_X_BottomFace();
                        // DERIVED from base grid/tick/series obj.

    p_cf_3dgtsTargetDomain->on();
    p_cf_3dgtsTargetDomain->useDomainItems( TRUE );
                        // Tell it how to act on chart model data!
                        
    p_cf_3dgtsTargetDomain->freqTextsToTicks( 1 );
    p_cf_3dgtsTargetDomain->freqGridsToTicks( 1 );
                        // Set attributes for "auto-update" behavior

    // Activate a Target "Range" Axis:
    cf_3DGridTickSeries *p_cf_3dgtsTargetRange 
                        = this->axis_Y_LeftFace();
                        // DERIVED from base grid/tick/series object.
                        
    p_cf_3dgtsTargetRange->on();
    p_cf_3dgtsTargetRange->useDataRange( TRUE );
    p_cf_3dgtsTargetRange->useRangeValues( TRUE );
                        // We want to tick across the data -- AND --
                        // have user range ticks show up as well...
                                                
    p_cf_3dgtsTargetRange->freqTextsToTicks( 2 );
    p_cf_3dgtsTargetRange->freqGridsToTicks( 2 );
                        // Set attributes for "auto-update" behavior
    return;
}

void
    cf_eng_CartXYZ
        ::measureDataMinMax()
{
    this->dChartXRangeMin = this->p_cf_cmodlUserData->dataXItemsMin();
    this->dChartXRangeMax = this->p_cf_cmodlUserData->dataXItemsMax();

    this->dChartYRangeMin = this->p_cf_cmodlUserData->dataYValuesMin();
    this->dChartYRangeMax = this->p_cf_cmodlUserData->dataYValuesMax();

    this->dChartZRangeMin = this->p_cf_cmodlUserData->dataZGroupsMin();
    this->dChartZRangeMax = this->p_cf_cmodlUserData->dataZGroupsMax();

}   // end of cf_eng_CartXYZ::measureDataMinMax()

void
    cf_eng_CartXYZ
        ::measureVirtMinMax()
{
    // Declare vars used to Determine the Min/Max X, Y, & Z
    // DATA vals in this cf_ChartModel by "scanning over" the data
    // values of this cf_ChartModel's objects:
    double  dXMin = this->p_cf_kEngineKernel->getXMin();
    double  dYMin = this->p_cf_kEngineKernel->getYMin();
    double  dZMin = this->p_cf_kEngineKernel->getZMin();

    double  dXMax = this->p_cf_kEngineKernel->getXMax();
    double  dYMax = this->p_cf_kEngineKernel->getYMax();
    double  dZMax = this->p_cf_kEngineKernel->getZMax();
            // Before we start our scan, set some harmless
            // default.  If we have no data objects to scan,
            // these will remain the min/max values...

    //---------------------------------------------------------
    // Otherwise, start by scanning the chart space objects:
    //---------------------------------------------------------
    cf_Virt3DObj    *p_cf_objDefault = NULL;
    cf_Virt3DObj    *p_cf_objTestVal = NULL;

    if ( this->iNumSpaceObjects >= 1 )
    {
        p_cf_objDefault = ( (cf_Virt3DObj*)
                            this->da_p_cf_objSpaceObjects[ 1 ] );

        dXMin = p_cf_objDefault->virtXMin();
        dYMin = p_cf_objDefault->virtYMin();
        dZMin = p_cf_objDefault->virtZMin();

        dXMax = p_cf_objDefault->virtXMax();
        dYMax = p_cf_objDefault->virtYMax();
        dZMax = p_cf_objDefault->virtZMax();
    }   // Min/Max vals DEFAULT to the Min/Max of the 1st space object.

    for ( i = 2; i <= this->iNumSpaceObjects; i++ )
    {
        p_cf_objTestVal = ( (cf_Virt3DObj*)
                            this->da_p_cf_objSpaceObjects[ i ] );

        // Look for an X or Y or Z Minima
        if ( p_cf_objTestVal->virtXMin() < dXMin )
                dXMin = p_cf_objTestVal->virtXMin();
        if ( p_cf_objTestVal->virtYMin() < dYMin )
                dYMin = p_cf_objTestVal->virtYMin();
        if ( p_cf_objTestVal->virtZMin() < dZMin )
                dZMin = p_cf_objTestVal->virtZMin();

        // Look for an X or Y or Z Maxima
        if ( p_cf_objTestVal->virtXMax() > dXMax )
                dXMax = p_cf_objTestVal->virtXMax();
        if ( p_cf_objTestVal->virtYMax() > dYMax )
                dYMax = p_cf_objTestVal->virtYMax();
        if ( p_cf_objTestVal->virtZMax() > dZMax )
                dZMax = p_cf_objTestVal->virtZMax();

    }   // End of scanning the SpaceObjects for Data Min/Max vals.

    //----------------------------------------------
    // Next, scan the chart face objects:
    //----------------------------------------------
    for ( i = 1; i <= this->iNumFaceObjects; i++ )
    {
        // Is the first face object the SOLE object in this engine?
        if ( i == 1 )
        {   // IF there are no space objects to used for default values:
            if ( ( this->iNumSpaceObjects < 1 )
                && ( this->iNumFaceObjects >= 1 ) )
            {   // ...let the Min & Max vals be initially set to
                // the Min & Max of the 1st face object:
                p_cf_objDefault = ( (cf_Virt3DObj*)
                                    this->da_p_cf_objFaceObjects[ 1 ] );

                dXMin = p_cf_objDefault->virtXMin();
                dYMin = p_cf_objDefault->virtYMin();
                dZMin = p_cf_objDefault->virtZMin();

                dXMax = p_cf_objDefault->virtXMax();
                dYMax = p_cf_objDefault->virtYMax();
                dZMax = p_cf_objDefault->virtZMax();
            }
        }   // end of special consideration for first face object.

        p_cf_objTestVal = ( (cf_Virt3DObj*)
                            this->da_p_cf_objFaceObjects[ i ] );

        // Otherwise, continue to look for X or Y or Z Minima
        if ( p_cf_objTestVal->virtXMin() < dXMin )
                dXMin = p_cf_objTestVal->virtXMin();
        if ( p_cf_objTestVal->virtYMin() < dYMin )
                dYMin = p_cf_objTestVal->virtYMin();
        if ( p_cf_objTestVal->virtZMin() < dZMin )
                dZMin = p_cf_objTestVal->virtZMin();

        // ...and X or Y or Z Maxima
        if ( p_cf_objTestVal->virtXMax() > dXMax )
                dXMax = p_cf_objTestVal->virtXMax();
        if ( p_cf_objTestVal->virtYMax() > dYMax )
                dYMax = p_cf_objTestVal->virtYMax();
        if ( p_cf_objTestVal->virtZMax() > dZMax )
                dZMax = p_cf_objTestVal->virtZMax();

    }   // End of scanning the FaceObject array for Data Min/Max vals.

    //-------------------------------------------------------
    // After the scan(s), update this cf_ChartModel Engine's
    // X, Y & Z Min/Max "virtual" DATA Vars:
    //-------------------------------------------------------
    this->dVirtXRangeMin = dXMin;
    this->dVirtXRangeMax = dXMax;
    this->dVirtYRangeMin = dYMin;
    this->dVirtYRangeMax = dYMax;
    this->dVirtZRangeMin = dZMin;
    this->dVirtZRangeMax = dZMax;
        // This is the FIRST approximation...  Now make updates...
        // We'll move from scanning the actual virtual data
        // to determining the total RANGE of the virtual data...

    // Next approx. in/ex-cludes origins, -- as appropriate...
    // Start by considering the X-Origin:
    BOOL    bIncludeXAxisOrigin = FALSE;    // Tentatively...

    if ( ( this->dChartXRangeMin <= 0.0 )
            && ( this->dChartXRangeMax >= 0.0 ) )
                    bIncludeXAxisOrigin = TRUE;

    if ( bIncludeXAxisOrigin == TRUE )
    {   // Deal with virtual data sets that lie
        // entirely RIGHT OF the X-Origin...
        if ( this->dChartXRangeMin >= 0.0 )
            // By updating the Virt-Range-Min to be ZERO -- if needed.
            if ( this->dChartXOrigin < this->dVirtXRangeMin )
                this->dVirtXRangeMin = this->dChartXOrigin;
        // Deal with virtual data that lies RIGHT OF the X-Origin...
        if ( this->dChartXRangeMax <= 0.0 )
            // By updating the Virt-Range-Max to be ZERO -- if needed.
            if ( this->dChartXOrigin > this->dVirtXRangeMax )
                this->dVirtXRangeMax = this->dChartXOrigin;
    }

    // Next consider the Y-Origin:
    BOOL    bIncludeYAxisOrigin = FALSE;    // Tentatively...

    if ( ( this->dChartYRangeMin <= 0.0 )
            && ( this->dChartYRangeMax >= 0.0 ) )
        bIncludeYAxisOrigin = TRUE;

    if ( bIncludeYAxisOrigin == TRUE )
    {   // Deal with virtual data that lies ABOVE the Y-Origin...
        if ( this->dChartYRangeMin >= 0.0 )
            // By updating the Virt-Range-Min to be ZERO -- if needed.
            if ( this->dChartYOrigin < this->dVirtYRangeMin )
                this->dVirtYRangeMin = this->dChartYOrigin;
        // Deal with virtual data that lies BELOW the Y-Origin...
        if ( this->dChartYRangeMax <= 0.0 )
            // By updating the Virt-Range-Max to be ZERO -- if needed.
            if ( this->dChartYOrigin > this->dVirtYRangeMax )
                this->dVirtYRangeMax = this->dChartYOrigin;
    }

    // Finally consider the Z-Origin:
    BOOL    bIncludeZAxisOrigin = FALSE;    // Tentatively...

    if ( ( this->dChartZRangeMin <= 0.0 )
            && ( this->dChartZRangeMax >= 0.0 ) )
                    bIncludeZAxisOrigin = TRUE;

    if ( bIncludeZAxisOrigin == TRUE )
    {   // Deal with virtual data that lies ABOVE the Y-Origin...
        if ( this->dChartZRangeMin >= 0.0 )
            // By updating the Virt-Range-Min to be ZERO -- if needed.
            if ( this->dChartZOrigin < this->dVirtZRangeMin )
                this->dVirtZRangeMin = this->dChartZOrigin;
        // Deal with virtual data that lies BELOW the Y-Origin...
        if ( this->dChartZRangeMax <= 0.0 )
            // By updating the Virt-Range-Max to be ZERO -- if needed.
            if ( this->dChartZOrigin > this->dVirtZRangeMax )
                this->dVirtZRangeMax = this->dChartZOrigin;
    }

    return; // VIRTUAL data/ranges of this engine have now been set,
            // with consideration to data and origin inclusions.

}   // end of cf_eng_CartXYZ::measureVirtMinMax()


void
    cf_eng_CartXYZ
        ::updateAxisGridTickSeries()
{
    //--------------------------------------------------------
    // If an Axis Grid/Tick Mark Series is NOT NULL, then it
    // HAS been activated, and let's consider updating it.
    //--------------------------------------------------------

    if ( this->p_cf_3dgtsXonBottomFace != NULL )
        // ( Notice we do NOT use the engine's axis<Whatever>()
        // method to determine if the series is active, because
        // this would only instantiate and activate it!  Instead,
        // we wait until we KNOW it's non-NULL before we poll it! )
        if ( this->axis_X_BottomFace()->isActive() )
                this->p_cf_3dgtsXonBottomFace->updateFromEngineModel();
                //...according to the "update method state" the user set:

    if ( this->p_cf_3dgtsXonBackFace != NULL )
        if ( this->axis_X_BackFace()->isActive() )
                this->p_cf_3dgtsXonBackFace->updateFromEngineModel();
                // Bottom and left-face X-axes now updated...
                    

    if ( this->p_cf_3dgtsYonLeftFace != NULL )
        if ( this->axis_Y_LeftFace()->isActive() )
                this->p_cf_3dgtsYonLeftFace->updateFromEngineModel();

    if ( this->p_cf_3dgtsYonBackFace != NULL )
        if ( this->axis_Y_BackFace()->isActive() )
                this->p_cf_3dgtsYonBackFace->updateFromEngineModel();
                // Left and back-face Y-axes now updated...
                    

    // Consider whether or not we are in 2D Mode BEFORE letting 
    // the Z-Axis Grid/Tick Mark Series mark the Z-Axis Ranges:
    if ( this->disp3DMode() != TRUE )
    {
        if ( this->p_cf_3dgtsZonLeftFace != NULL )
            if ( this->axis_Z_LeftFace()->isActive() )
                this->p_cf_3dgtsZonLeftFace->updateFromEngineModel();

        if ( this->p_cf_3dgtsZonBottomFace != NULL )
            if ( this->axis_Z_BottomFace()->isActive() )
                this->p_cf_3dgtsZonBottomFace->updateFromEngineModel();
                // Left and bottom-face Z-axes now updated...
    }

    // Note that we do NOT consider updating ANY "polar" 
    // grid/ticks.  This is a "cartesian" show here!
    return;

}   // end of cf_eng_CartXYZ::updateAxisGridTickSeries()


void
    cf_eng_CartXYZ
        ::updateAxisRangeBufferPoints()
{
    double          dAxisLimVal = 0.0;
    cf_3DInvisPoint *p_cf_3dpAxisMinMax = NULL; 

    // Add "invisible" 3D-points at axis range buffer/limits:
    dAxisLimVal = this->p_cf_cmodlUserData->dataXItemsMin();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( dAxisLimVal, 0.0, 0.0 );
    this->addFaceObject( p_cf_3dpAxisMinMax );

    dAxisLimVal = this->p_cf_cmodlUserData->dataXItemsMax();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( dAxisLimVal, 0.0, 0.0 );
    this->addFaceObject( p_cf_3dpAxisMinMax ); // Virtual X-Axis now set.
    
    dAxisLimVal = this->p_cf_cmodlUserData->dataYValuesMin();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( 0.0, dAxisLimVal, 0.0 );
    this->addFaceObject( p_cf_3dpAxisMinMax );

    dAxisLimVal = this->p_cf_cmodlUserData->dataYValuesMax();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( 0.0, dAxisLimVal, 0.0 );
    this->addFaceObject( p_cf_3dpAxisMinMax ); // Virtual Y-Axis now set.

    return;
}


void
    cf_eng_CartXYZ
        ::updateFromCurrentModel()
{
    // We need to discard old chart-engine objects
    // if our data has been changed.
    cf_ChartEngine::updateFromCurrentModel();
                    //Use the functionality of the BASE member method
                    //to reset any existing chart-geometry-objects.

    // Generate a set of "new" Chart Engine Objects:
    switch ( this->iIndexID )
    {
        case    CF_ID_YVALS_AT_INT_IS:

                // Generate Virtual Objects AT Integer I indices:
                this->updateYValsAtIntIs();
                break;

        case    CF_ID_YVALS_ON_NAT_NS:
        case    CF_ID_YVALS_AT_USR_XS:
        default:
                // Generate Virtual Objects ON "Natural N" indices:
                this->updateYValsOnNatNs();
                break;
    }

    // Generate domain and range axis grid/tick/series objects:
    this->updateAxisGridTickSeries();

    // Generate range axis buffering (by adding "invisible" 3D-points):
    this->updateAxisRangeBufferPoints();
    
    return;

}   // end of cf_eng_CartXYZ::updateFromCurrentModel()




// END of cf_eng_CartXYZ Implementation

//////////////////////////////////////
// END of File: CE1_CCOR.CPP
//////////////////////////////////////





