//////////////////////////////////////
// START of File: CE1_PCOR.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/ce1_pcor.hpp> 

//
// Include any additional classe defintions:
//
class       cf_2DPoint;
class       cf_2DRect;
#include    <cf_api/c0_dimen.hpp>

//
// Use this ctor to set up an "empty" cf_eng_Polar.
//
cf_eng_Polar
    ::cf_eng_Polar( cf_ChartModel   *p_cf_cmodlOwner,
                    int             iChartTypeID, 
                    int             iIndexMethodID )
                                
    : cf_ChartEngine( p_cf_cmodlOwner, iChartTypeID, iIndexMethodID )
                                
{
    // Init the Category vars:
    this->bIsEngineCartesian = FALSE;
    this->bIsEnginePolar = TRUE;

    // Init the Kernel to a favorable state for polar chart views:
    this->preferViewAngle(  30.0,   // x-view angle -- up at 30 deg.
                            0.0 );  // y-view angle -- head-on-front.
    this->preferPerspectiveZero();  // Go for ISOMETRIC projection

    // Pies should be "squashed" a little flatter
    // than Cartesian/XYZ charts:
    this->preferProportions( 0.05,       // New height-to-width
                             100.0,      // New depth-to-height
                             FALSE );    // But do NOT auto-configure

    // Configure this derived chart's origins, etc:
    this->configOrigins();

    //
    // The following lines can be uncommented for debugging:
    // (They set up marker point attributes for the XZ Plane)
    //
    /*
    this->cf_3dpXZPlaneEllipseCenter.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseCenter.setColor( CF_BLACK );

    this->cf_3dpXZPlaneEllipseProjXMax.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseProjXMax.setColor( CF_RED );
    this->cf_3dpXZPlaneEllipseProjXMin.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseProjXMin.setColor( CF_RED );

    this->cf_3dpXZPlaneEllipseVirtXMax.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseVirtXMax.setColor( cf_objRANGE );
    this->cf_3dpXZPlaneEllipseVirtXMin.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseVirtXMin.setColor( cf_objRANGE );
    
    this->cf_3dpXZPlaneEllipseProjZMax.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseProjZMax.setColor( CF_DARK_BLUE );
    this->cf_3dpXZPlaneEllipseProjZMin.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseProjZMin.setColor( CF_DARK_BLUE );

    this->cf_3dpXZPlaneEllipseVirtZMax.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseVirtZMax.setColor( CF_MED_PURPLE );
    this->cf_3dpXZPlaneEllipseVirtZMin.set3DMarker( TRUE );
    this->cf_3dpXZPlaneEllipseVirtZMin.setColor( CF_MED_PURPLE );
    */
}


cf_eng_Polar
    ::~cf_eng_Polar() 
        {}


void
    cf_eng_Polar
        ::applyGeomXform() 
{
    //
    // First, use the capabilities of the base class:
    //
    cf_ChartEngine::applyGeomXform();
    
    //
    // Now, perform specific polar chart functionality...
    // Xform the center of the XZ Plane:
    //
    this->cf_3dpXZPlaneEllipseCenter
            .applyGeomXform( this->p_cf_kEngineKernel );

    //
    // Xform the "virtual" minima/maxima:
    //
    this->cf_3dpXZPlaneEllipseVirtXMax
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtXMin
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtZMax
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtZMin
            .applyGeomXform( this->p_cf_kEngineKernel );

    // Counter any Y-Angle Rotation Effects
    // on the XZ plane (base) ellipse rectangle:
    double  dCurrXAngle = this->p_cf_kEngineKernel->getXAngle();
    double  dCurrYAngle = this->p_cf_kEngineKernel->getYAngle();
    double  dCurrPRatio = this->p_cf_kEngineKernel->getPRatio();

    this->p_cf_kEngineKernel->preferViewAngle( dCurrXAngle, 0, TRUE );

    this->cf_3dpXZPlaneEllipseProjXMax
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjXMin
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjZMax
            .applyGeomXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjZMin
            .applyGeomXform( this->p_cf_kEngineKernel );
            // Xforms the XZ plane ellipse marker points

    // Now restore the kernel's state:
    this->p_cf_kEngineKernel->preferViewAngle( dCurrXAngle, dCurrYAngle, TRUE);

}   // end of cf_eng_Polar::applyGeomXform()



void
    cf_eng_Polar
        ::applyProjXform() 
{
    //
    // First, use the capabilities of the base class:
    //
    cf_ChartEngine::applyProjXform();
    
    //
    // Now, perform specific polar chart functionality:
    //
    this->cf_3dpXZPlaneEllipseProjXMax
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjXMin
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjZMax
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseProjZMin
                .applyProjXform( this->p_cf_kEngineKernel );

    this->cf_3dpXZPlaneEllipseCenter
                .applyProjXform( this->p_cf_kEngineKernel );

    this->cf_3dpXZPlaneEllipseVirtXMax
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtXMin
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtZMax
                .applyProjXform( this->p_cf_kEngineKernel );
    this->cf_3dpXZPlaneEllipseVirtZMin
                .applyProjXform( this->p_cf_kEngineKernel );
            //
            // XZ Plane xform completed
            //

    return;

}   // end of cf_eng_Polar::applyProjXform()




//
// If we change coordinates within the chart, or change the
// dimensions of the kernel, we should recalc the chart.
//
void
    cf_eng_Polar
        ::config3DBoxETC() 
{
    // Use the functionality of the Base Class to 
    // Configure the "standard" components of the 3D Box:
    cf_ChartEngine::config3DBoxETC();

    // Verify that we have a "proper" square to calc the
    // projected recangular boundary of our "polar pie"
    double  dXDim  = this->d3DBoxXMax - this->d3DBoxXMin;
    double  dZDim  = this->d3DBoxZMax - this->d3DBoxZMin;
    
    if ( dXDim != dZDim )
    // Whooooaw! Our "width" does not equal our "depth" !!!
    {
        // Alter the chart dimensions to correct the problem:
        dZDim = dXDim;

        double  dYDim = this->d3DBoxYMax - this->d3DBoxYMin;
        this->p_cf_kEngineKernel->preferDimensions( dXDim, dYDim, dZDim, TRUE );

        // Update all the 3D Box dimension tracking vars:
        cf_ChartEngine::config3DBoxETC();

    }   // End of screen for non-square pie-circle 
        // bounding-box bogosity.

    // Set up the XZ Plane ellipse tracking points by using
    // the bounding limits of the Pie's Base Ellipse Rectangle:
    // These points mark the "border of the pie"'s projected
    // ellipse boundaries, after correction for rotation, etc
    this->cf_3dpXZPlaneEllipseVirtXMax
                        .dataXYZ(   this->d3DBoxXMax, 
                                    this->dChartYOrigin, 
                                    this->dChartZOrigin );
    this->cf_3dpXZPlaneEllipseProjXMax
                        =( this->cf_3dpXZPlaneEllipseVirtXMax );

    this->cf_3dpXZPlaneEllipseVirtXMin
                        .dataXYZ(   this->d3DBoxXMin, 
                                    this->dChartYOrigin, 
                                    this->dChartZOrigin );
    this->cf_3dpXZPlaneEllipseProjXMin
                        =( this->cf_3dpXZPlaneEllipseVirtXMin );

    this->cf_3dpXZPlaneEllipseVirtZMax
                        .dataXYZ(   this->dChartXOrigin, 
                                    this->dChartYOrigin, 
                                    this->d3DBoxZMax );
    this->cf_3dpXZPlaneEllipseProjZMax
                        =( this->cf_3dpXZPlaneEllipseVirtZMax );

    this->cf_3dpXZPlaneEllipseVirtZMin
                        .dataXYZ(   this->dChartXOrigin, 
                                    this->dChartYOrigin, 
                                    this->d3DBoxZMin );
    this->cf_3dpXZPlaneEllipseProjZMin
                        =( this->cf_3dpXZPlaneEllipseVirtZMin );

    this->cf_3dpXZPlaneEllipseCenter
                        .dataXYZ(   this->dChartXOrigin,
                                    this->dChartYOrigin,
                                    this->dChartZOrigin  );
    return;

}   // end of cf_eng_Polar::config3DBoxETC()
                            


void
    cf_eng_Polar
        ::configOrigins()
{
    //
    // For Polar Coordinate charts -- we need to over-ride
    // the base class functionality and set the chart's local
    // XYZ Origin to be the CENTER of the BOTTOM 3B Box Face:
    //
    this->dChartXOrigin = 0.0;
    this->dChartYOrigin = this->p_cf_kEngineKernel->getYMin();
    this->dChartZOrigin = 0.0;

    return;

}   // end of cf_eng_Polar::configOrigins()



void    
    cf_eng_Polar
        ::draw2D( bf_Display *p_bf_dspFromChartView ) 
            { cf_eng_Polar::draw3D( p_bf_dspFromChartView ); }


void    
    cf_eng_Polar
        ::draw3D( bf_Display *p_bf_dspFromChartView )
{
    //
    // Render the 3D Box using the base class:
    //
    cf_ChartEngine::draw3D( p_bf_dspFromChartView );

    //
    // Now, perform derived functionality...
    // Get a display context:
    //
    p_bf_dspFromChartView->dsp_lock();

    //
    // Use the Virtual Projection Points to determine the boundaries
    // of a rectangle that will define the XZ projected ellipse
    // on the XY Coordinate space of the current bf_Display:
    //
    cf_2DPoint cf_2dpProjXMin 
                = this->cf_3dpXZPlaneEllipseProjXMin.screenPoint();
    cf_2DPoint cf_2dpProjXMax 
                = this->cf_3dpXZPlaneEllipseProjXMax.screenPoint();
    cf_2DPoint cf_2dpProjZMin 
                = this->cf_3dpXZPlaneEllipseProjZMin.screenPoint();
    cf_2DPoint cf_2dpProjZMax 
                = this->cf_3dpXZPlaneEllipseProjZMax.screenPoint();

    cf_2DPoint cf_2dpVirtXMax 
                = this->cf_3dpXZPlaneEllipseVirtXMax.screenPoint();
    cf_2DPoint cf_2dpVirtZMax
                = this->cf_3dpXZPlaneEllipseVirtZMax.screenPoint();
    
    //
    // Test and update these default Min/Max Projection boundaries:
    //
    cf_2DPoint cf_2dpUpperLeft = cf_2dpProjXMin;
    cf_2dpUpperLeft = cf_2dpUpperLeft.yieldMin( cf_2dpProjXMax );
    cf_2dpUpperLeft = cf_2dpUpperLeft.yieldMin( cf_2dpProjZMin );
    cf_2dpUpperLeft = cf_2dpUpperLeft.yieldMin( cf_2dpProjZMax );

    cf_2DPoint cf_2dpLowerRight = cf_2dpProjXMin;
    cf_2dpLowerRight = cf_2dpLowerRight.yieldMax( cf_2dpProjXMax );
    cf_2dpLowerRight = cf_2dpLowerRight.yieldMax( cf_2dpProjZMin );
    cf_2dpLowerRight = cf_2dpLowerRight.yieldMax( cf_2dpProjZMax );

    cf_2dpLowerRight = cf_2dpLowerRight.yieldMax( cf_2dpVirtXMax );
    cf_2dpLowerRight = cf_2dpLowerRight.yieldMax( cf_2dpVirtZMax );

    cf_2DRect  cf_2drBoundingEllipseRect( cf_2dpUpperLeft, 
                                                cf_2dpLowerRight );

    p_bf_dspFromChartView->dsp_ellipse(    cf_2dpUpperLeft, 
                                            cf_2dpLowerRight );

    //
    // For debugging purposes it may be useful to uncomment
    // these display lines below...
    //
    /*
    this->cf_3dpXZPlaneEllipseProjXMax.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseProjXMin.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseProjZMax.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseProjZMin.draw3D( p_bf_dspFromChartView );

    this->cf_3dpXZPlaneEllipseVirtXMax.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseVirtXMin.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseVirtZMax.draw3D( p_bf_dspFromChartView );
    this->cf_3dpXZPlaneEllipseVirtZMin.draw3D( p_bf_dspFromChartView );

    this->cf_3dpXZPlaneEllipseCenter.draw3D( p_bf_dspFromChartView );

    p_zapp_dspTemp->dsp_polyline( cf_2drBoundingEllipseRect ); 
    */

    //
    // Process the Face Component List ( tick marks, etc):
    //
    for ( i = 1; i <= iNumCompoObjects; i++ )
        ( (cf_Virt3DObj*) this->da_p_cf_objCompoObjects[ i ] )
                                ->draw3D( p_bf_dspFromChartView );
    //
    // And -- We're done!...
    //
    p_bf_dspFromChartView->dsp_unlock();

    return;

}   // end of cf_eng_Polar::draw3D( ... )


void
    cf_eng_Polar
        ::disp2DMode( BOOL  bNewState )
            { this->disp3DMode( !bNewState ); }

void
    cf_eng_Polar
        ::disp3DMode( BOOL  bNewState )
{
    if ( bNewState == FALSE )
    //
    // The user wants to go FROM 3D Mode TO 2D Mode
    //
    {
        //
        // But don't switch if we're already in 2D Mode!
        //
        if ( this->b3DMode == TRUE )    
        {
            // Save current view params for subsequent recall:
            this->dSaveXAngle
                        = this->p_cf_kEngineKernel->getXAngle();
            this->dSaveYAngle
                        = this->p_cf_kEngineKernel->getYAngle();
            this->dSaveZAngle
                        = this->p_cf_kEngineKernel->getZAngle();
            this->dSavePRatio
                        = this->p_cf_kEngineKernel->getPRatio();

            // Make the switch
            this->p_cf_kEngineKernel->preferViewAngle( 90.0, 0.0, FALSE  );
            this->p_cf_kEngineKernel->preferPerspective( 0.0, TRUE );

            this->b3DMode = FALSE;
        }   //
            // End of screen to insure we were IN 3D Mode.
            // Now, of course, we are in 2D Mode.
            //
    }
    else
    // bNewState == TRUE implies that the user wants
    // to go FROM 2D Mode TO 3D Mode:
    {
        // But don't switch if we're already in 3D Mode!
        if ( this->b3DMode != TRUE )
        {
            // Restore any saved view params as the current state:
            this->p_cf_kEngineKernel
                            ->preferViewAngle(  this->dSaveXAngle,
                                                this->dSaveYAngle,
                                                FALSE );
            this->p_cf_kEngineKernel
                            ->preferPerspective( this->dSavePRatio, TRUE );
            this->b3DMode = TRUE;
        }   // End of screen to insure we were IN 2D Mode.
            // Now, of course, we are in 3D Mode.
    }

    // Force a geometry update to occur...
    this->autoConfigure();
    this->applyGeomXform();

    // We USED to do this...
    /*
    this->p_cf_cmodlUserData->initiateDrawOp();
     */
    // But now we leave it up to the managing object to
    // decinde when it wants to signal a draw op...

    return;
    
}   // end of cf_eng_Polar::disp3DMode( BOOL    bNewState )



//
// END of cf_eng_Polar Implementation
//
    

//////////////////////////////////////
// END of File: CE1_PCOR.CPP
//////////////////////////////////////




