//////////////////////////////////////
// START of File: CE2_BAR.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
// and any additional neccessary include statements.
//
#include    <cf_api/ce2_bar.hpp> 

class       cf_XYZ;
class       cf_DataGroup;
#include    <cf_api/c1_dgrou.hpp>
            // We'll access the data groups/items in our Chart Model..

class       cf_3DPoint;
class       cf_3DPolygon;
#include    <cf_api/co1_pgon.hpp>
            // We'll build 2D Polygons for the 2D Bars...
    
//
//=====================================
// Implement class cf_eng_Bar2D:
//=====================================
//
cf_eng_Bar2D
    ::cf_eng_Bar2D( cf_ChartModel   *p_cf_cmodlOwner,
                    int             iIndexMethodID )

        : cf_eng_CartXYZ(   p_cf_cmodlOwner,
                            CF_ID_BAR_2D,
                            iIndexMethodID )
{ 
    this->disp2DMode( TRUE );           // Make it a 2D Chart!...
    this->bIsEngineBarColStack = TRUE;  // Make axis indexing accurate...

    // The BASE chart engine class will have instantiated
    // ALL the possible Grid/Tick Series to NULL...
    // The DERIVED chart engine classes MUST set 
    // the series that they specifically want to be ON!...
    this->initGridTickSeries();
            // cf_engCartXYZ-derived charts can use either
            // the "default" XYZ/Cartesian method, which
            // activates the BOTTOM X-Axis as a "domain"
            // axis, and the LEFT Y-Axis as a "range over
            // chart model Y-Vals" axis, or define their
            // OWN version to over-ride the virtual XYZ/
            // Cartesian method...
}


cf_eng_Bar2D
    ::~cf_eng_Bar2D() 
        {}

void
    cf_eng_Bar2D
        ::initGridTickSeries()
{
    // Activate a Target "Domain" Axis:
    cf_3DGridTickSeries *p_cf_3dgtsTargetDomain 
                        = this->axis_Y_LeftFace();
                        // DERIVED from base grid/tick/series obj.

    p_cf_3dgtsTargetDomain->on();
    p_cf_3dgtsTargetDomain->useDomainItems( TRUE );
                        // Tell it how to act on chart model data!
                        
    p_cf_3dgtsTargetDomain->freqTextsToTicks( 1 );
    p_cf_3dgtsTargetDomain->freqGridsToTicks( 1 );
                        // Set attributes for "auto-update" behavior

    // Activate a Target "Range" Axis:
    cf_3DGridTickSeries *p_cf_3dgtsTargetRange 
                        = this->axis_X_BottomFace();
                        // DERIVED from base grid/tick/series object.
                        
    p_cf_3dgtsTargetRange->on();
    p_cf_3dgtsTargetRange->useDataRange( TRUE );
    p_cf_3dgtsTargetRange->useRangeValues( TRUE );
                        // We want to tick across the data -- AND --
                        // have user range ticks show up as well...
                                                
    p_cf_3dgtsTargetRange->freqTextsToTicks( 2 );
    p_cf_3dgtsTargetRange->freqGridsToTicks( 2 );
                        // Set attributes for "auto-update" behavior

    return;
}   


void
    cf_eng_Bar2D
        ::measureDataMinMax() 
{
    this->dChartXRangeMin = this->p_cf_cmodlUserData->dataYValuesMin();
    this->dChartXRangeMax = this->p_cf_cmodlUserData->dataYValuesMax();

    this->dChartYRangeMin = this->p_cf_cmodlUserData->dataXItemsMin();
    this->dChartYRangeMax = this->p_cf_cmodlUserData->dataXItemsMax();
    
    this->dChartZRangeMin = this->p_cf_cmodlUserData->dataZGroupsMin();
    this->dChartZRangeMax = this->p_cf_cmodlUserData->dataZGroupsMax();
    
}   // end of cf_eng_Bar2D::measureDataMinMax() 


void
    cf_eng_Bar2D
        ::updateAxisRangeBufferPoints()
{
    double          dAxisLimVal = 0.0;
    cf_3DInvisPoint *p_cf_3dpAxisMinMax = NULL; 

    // Add "invisible" 3D-points at the range buffer/limits
    // (along the X-AXIS for this) BAR chart:
    dAxisLimVal = this->p_cf_cmodlUserData->dataYValuesMin();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( dAxisLimVal, 0.0, 0.0 );
    this->addSpaceObject( p_cf_3dpAxisMinMax );

    dAxisLimVal = this->p_cf_cmodlUserData->dataYValuesMax();
    p_cf_3dpAxisMinMax = new cf_3DInvisPoint( dAxisLimVal, 0.0, 0.0 );
    this->addSpaceObject( p_cf_3dpAxisMinMax );
}



void    
    cf_eng_Bar2D
        ::updateYValsOnNatNs()
//
//====================================
// OK -- Here's the Plan:
//====================================
//
/*        
        N-| (continued on Natural N Indices...)
          
          [...]
          
          |++++++
        3-|**********
          |&&&&
         ~|
          |++++++++++++
        2-|***************
          |&&&&&&&&&&&&&&&&&&&&
         ~|
          |++++++++++++
        1-|********
          |&&&&
         ~|
          |
        0-|___________( Base Line )_______________

 */
{
    // We will pull data from the chart with these vars:
    cf_DataGroup    *p_cf_dgAccessGroup = NULL;
    cf_XYZ          *p_cf_xyzAccessItem = NULL;
    double      dUserYVal = 0.0;

    // We will generate "Virtual 2D Bar Points/Polygons" 
    // for each of the Chart's Data Items with these vars:
    cf_3DPoint      cf_3dpTemp( 0.0, 0.0, 0.0 );
    cf_3DPolygon    *p_cf_3dpgNewGon = NULL;
                    // After new()-ed and configured, this storage
                    // will get passed to our dyna-array of ptrs...

    cf_3DInvisPoint *p_cf_3dpAfterLastGon = NULL; 
                    // Adds space so last polygon is NOT flush
                    // to the ending border, and matches the 
                    // margin established by the first group.

    // Count the number of Active Groups:
    int     iStartAt = this->p_cf_cmodlUserData->dataGroupsStartAt();
    int     iEndAt = this->p_cf_cmodlUserData->dataGroupsEndAt();
    int     iNumGroups = iEndAt - iStartAt;
    int     iNumItemsPerPosit = iNumGroups + 2;
            // Cause each group will have a leading AND trailing space...
    
    // Calc the space required for each item:
    double  dSpacePerPosit = 1.0;
    double  dSpacePerItem = dSpacePerPosit / iNumItemsPerPosit;
    double  dItemHalfSpace = dSpacePerItem / 2.0;
                // Note that iNumItemsPerPosit should NEVER == 0.0! 

    // Compute the max number of axis items to iterate over:
    int iNumActiveInGroup = 0;
    int iNumDomainAxisItems = 0;
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );
        iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
        if ( iNumActiveInGroup > iNumDomainAxisItems ) 
                iNumDomainAxisItems = iNumActiveInGroup;
    }           // We now have our iteration limit...

    //
    //--------------------------------------------------
    // Loop over each axis index which represents the
    // indexed values from our collection of groups:
    //--------------------------------------------------
    //
    double  dPlaceItemAt = 0.5 + dItemHalfSpace;
            // Set the spot to (start to) lay down items:
                
    for ( i = 1; i <= iNumDomainAxisItems; i++ )
    {
        //--------------------------------------------------------  
        // Loop over the (available) groups to generate polygons:
        //--------------------------------------------------------  
        for ( j = iStartAt; j <= iEndAt; j++ )  // (D-group indices)
        {
            // Get data values and construct 3D Points that
            // define bar/col/stack vertices:
            p_cf_dgAccessGroup  // Grab a group...
                = this->p_cf_cmodlUserData->activateGroup( j );
            p_cf_xyzAccessItem  // Then an item....
                = p_cf_dgAccessGroup->activateXYZValAt( i );
            dUserYVal = p_cf_xyzAccessItem->y();
                // ... whose Y value will define a "length/height"...

            // New a polygon -- note that we wait till AFTER we have
            // a data group activated so we can use VALID disp attribs!
            p_cf_3dpgNewGon 
                = new cf_3DPolygon( p_cf_dgAccessGroup->dispAttribs() );
                // (which will become a bar/col/stack/etc):
                
            // Construct the "leading edge":
            cf_3dpTemp.dataXYZ( 0.0, dPlaceItemAt, 0.0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            cf_3dpTemp.dataXYZ( dUserYVal, dPlaceItemAt, 0.0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            dPlaceItemAt += dSpacePerItem;
                        // bump along to the trailing edge...
            
            // Construct the "trailing edge":
            cf_3dpTemp.dataXYZ( dUserYVal, dPlaceItemAt, 0.0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            cf_3dpTemp.dataXYZ( 0.0, dPlaceItemAt, 0.0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            // Done with this polygon -- don't loose it!
            this->addSpaceObject( p_cf_3dpgNewGon );

        }   // end of for-loop over Data Groups

        // Lay down a trailing space AFTER the grouped items:
        dPlaceItemAt += dSpacePerItem;

    }   // end of for-loop over iNumCurrentItems

    // Add an "invisible point" to an appropriate margin/space:
    dPlaceItemAt = (double) iNumDomainAxisItems + 1.0;
    p_cf_3dpAfterLastGon 
        = new cf_3DInvisPoint(  0.0, dPlaceItemAt, 0.0 );
        // makes a "trailing" margin matching the "leading" margin.
    this->addFaceObject( p_cf_3dpAfterLastGon );

    return;
    
}   // end of cf_eng_Bar2D::updateYValsOnNatNs()



void    
    cf_eng_Bar2D
        ::updateYValsAtIntIs()
//
//====================================
// OK -- Here's the Plan:
//====================================
//
/*        
        I-| (continued on Integer I Indices...)
          
          [...]
          
          |++++++
          |**********
        2-|&&&&
          |
          |++++++++++++
          |***************
        1-|&&&&&&&&&&&&&&&&&&&&
          |
          |++++++++++++
          |********
        0-|&&&&_______( Base Line )_______________

 */
{
    // We will pull data from the chart with these vars:
    cf_DataGroup    *p_cf_dgAccessGroup = NULL;
    cf_XYZ          *p_cf_xyzAccessItem = NULL;
    double      dUserYVal = 0.0;

    // We will generate "Virtual 2D Bar Points/Polygons" 
    // for each of the Chart's Data Items with these vars:
    cf_3DPoint      cf_3dpTemp( 0, 0, 0 );
    cf_3DPolygon    *p_cf_3dpgNewGon = NULL;
                    // After new()-ed and configured, this storage
                    // will get passed to our dyna-array of ptrs...

    cf_3DInvisPoint *p_cf_3dpAfterLastGon = NULL; 
                    // Adds space so last polygon is NOT flush
                    // to the ending border, and matches the 
                    // margin established by the first group.

    // Count the number of Active Groups:
    int     iStartAt = this->p_cf_cmodlUserData->dataGroupsStartAt();
    int     iEndAt = this->p_cf_cmodlUserData->dataGroupsEndAt();
    int     iNumGroups = iEndAt - iStartAt;
    int     iNumItemsPerPosit = iNumGroups + 1;
            // Note that nulike Natural N Indices, 
            // each group will Not have a trailing space...
    
    // Calc the space required for each item:
    double  dSpacePerPosit = 1.0;
    double  dSpacePerItem = dSpacePerPosit / iNumItemsPerPosit;
            // Note that iNumItemsPerPosit should NEVER == 0.0! 

    // Compute the max number of axis items to iterate over:
    int iNumActiveInGroup = 0;
    int iNumDomainAxisItems = 0;
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );
        iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
        if ( iNumActiveInGroup > iNumDomainAxisItems ) 
                iNumDomainAxisItems = iNumActiveInGroup;
    }           // We now have our iteration limit...

    //
    //--------------------------------------------------
    // Loop over each axis index which represents the
    // indexed values from our collection of groups:
    //--------------------------------------------------
    //
    double  dPlaceItemAt = 0.0;
                // Set the spot to (start to) lay down items:
                
    for ( i = 1; i <= iNumDomainAxisItems; i++ )
    {
        //--------------------------------------------------------  
        // Loop over the (available) groups to generate polygons:
        //--------------------------------------------------------  
        for ( j = iStartAt; j <= iEndAt; j++ )  // (D-group indices)
        {
            // Get data values and construct 3D Points that
            // define bar/col/stack vertices:
            p_cf_dgAccessGroup  // Grab a group...
                = this->p_cf_cmodlUserData->activateGroup( j );
            p_cf_xyzAccessItem  // Then an item....
                = p_cf_dgAccessGroup->activateXYZValAt( i );
            dUserYVal = p_cf_xyzAccessItem->y();
                // ... whose Y value will define a "length/height"...

            // New a polygon -- note that we wait till AFTER we have
            // a data group activated so we can use VALID disp attribs!
            p_cf_3dpgNewGon 
                = new cf_3DPolygon( p_cf_dgAccessGroup->dispAttribs() );
                // (which will become a bar/col/stack/etc):
                
            // Construct the "leading edge":
            cf_3dpTemp.dataXYZ( 0, dPlaceItemAt, 0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            cf_3dpTemp.dataXYZ( dUserYVal, dPlaceItemAt, 0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            dPlaceItemAt += dSpacePerItem;
                        // bump along to the trailing edge...
            
            // Construct the "trailing edge":
            cf_3dpTemp.dataXYZ( dUserYVal, dPlaceItemAt, 0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            cf_3dpTemp.dataXYZ( 0, dPlaceItemAt, 0 );
            p_cf_3dpgNewGon->addPoint( cf_3dpTemp );

            // Done with this polygon -- don't loose it!
            this->addSpaceObject( p_cf_3dpgNewGon );

        }   // end of for-loop over Data Groups

    }   // end of for-loop over iNumCurrentItems

    // Add an "invisible point" to an appropriate margin/space:
    dPlaceItemAt = (double) iNumDomainAxisItems;
    p_cf_3dpAfterLastGon 
        = new cf_3DInvisPoint(  0.0, dPlaceItemAt, 0.0 );
        // makes a "trailing" margin matching the "leading" margin.
    this->addFaceObject( p_cf_3dpAfterLastGon );

    return;
    
}   // end of cf_eng_Bar2D::updateYValsAtIntIs()

//
// END of class cf_eng_Bar2D Implementation
//

//////////////////////////////////////
// END of File: CE2_BAR.CPP
//////////////////////////////////////

