//////////////////////////////////////
// START of File: CE2_PSER.CPP
//////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
// and any additional neccessary include statements.
//
#include    <cf_api/ce2_pser.hpp> 

class       cf_XYZ;
class       cf_DataGroup;
#include    <cf_api/c1_dgrou.hpp>
            // We'll access the data groups/items in our Chart Model..

class       cf_3DPoint;
class       cf_3DPointSeries;
#include    <cf_api/co1_pser.hpp>
            // We'll add user data values into 2D/3D Point Series...
    
//
//==========================================
// Implement class cf_eng_PointSer2D:
//==========================================
//
cf_eng_PointSer2D
    ::cf_eng_PointSer2D(    cf_ChartModel   *p_cf_cmodlOwner,
                            int             iIndexMethodID )

        : cf_eng_CartXYZ(   p_cf_cmodlOwner,
                            CF_ID_POINTS_2D,
                            iIndexMethodID )
{ 
    this->disp2DMode( TRUE );   // Make it a 2D Chart!...

    // The BASE chart engine class will have instantiated
    // ALL the possible Grid/Tick Series to NULL...
    // The DERIVED chart engine classes MUST set 
    // the series that they specifically want to be ON!...
    this->initGridTickSeries();
            // cf_engCartXYZ-derived charts can use either
            // the "default" XYZ/Cartesian method, which
            // activates the BOTTOM X-Axis as a "domain"
            // axis, and the LEFT Y-Axis as a "range over
            // chart model Y-Vals" axis, or define their
            // OWN version to over-ride the virtual XYZ/
            // Cartesian method...
}


cf_eng_PointSer2D
    ::~cf_eng_PointSer2D() 
        {}

void
    cf_eng_PointSer2D
        ::updateYValsOnNatNs()
{
    // We will generate "Virtual 2D Line Points and PolyLines" 
    // for each of the Chart's Data Items with these vars:
    cf_DataGroup        *p_cf_dgAccessGroup;
    cf_XYZ              *p_cf_xyzAccessItem;
                        // We'll point along the various groups/items
    
    cf_3DPoint          cf_3dpNewPoint;
    double              dNewPointDepthZ = 0.0;
                        // and grab Y-vals to add as points to the...

    cf_3DPointSeries    *p_cf_3dpsNewSeries;
                        // ...series which represent the data groups...

    cf_3DInvisPoint     *p_cf_3dpAfterLastItem = NULL; 
                        // Adds space so last item is NOT flush
                        // to the ending border, and matches the 
                        // margin established by the first item.

    // We will iterate over the available groups and generate
    // 2D PolyLines from the data elements in the parent 
    // cf_ChartModel's data groups:
    int iStartAt = this->p_cf_cmodlUserData->dataGroupsStartAt();
    int iEndAt = this->p_cf_cmodlUserData->dataGroupsEndAt();
            
    // Compute the max number of axis items to iterate over:
    int iNumActiveInGroup = 0;
    int iNumDomainAxisItems = 0;
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );
        iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
        if ( iNumActiveInGroup > iNumDomainAxisItems ) 
                iNumDomainAxisItems = iNumActiveInGroup;
    }           // We now have our iteration limit...


    // Begin a for-loop over the active group range:
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        // Activate each chart group, and get a pointer to it:
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );

        // Only add a virtual chart object IF there is data to use:
        if ( p_cf_dgAccessGroup->hasData() )
        {
            // Define a new 2D/3D point-series:
            p_cf_3dpsNewSeries 
                = new cf_3DPointSeries( p_cf_dgAccessGroup->dispAttribs() );

            // Now we'll add points to this point-series:
            iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
            
            // for-loop over the current count of items:
            for ( j = 1; j <= iNumActiveInGroup; j++ )
            {
                // Get data values and construct the 2D/3D object:
                p_cf_xyzAccessItem 
                    = p_cf_dgAccessGroup->activateXYZValAt( j );
                    // This XY pair will define a new PolyLine point...
                
                cf_3dpNewPoint.dataXYZ( p_cf_xyzAccessItem->x(), 
                                        //was: j, // Indexing Natural Ns!
                                        p_cf_xyzAccessItem->y(), 
                                        dNewPointDepthZ );

                p_cf_3dpsNewSeries->addPoint( cf_3dpNewPoint );

            }   // end of for-loop over iNumCurrentItems

            // Add the group/series to the engine...
            this->addSpaceObject( p_cf_3dpsNewSeries );

            //... and set up for the NEXT group/series
            dNewPointDepthZ+= 1.0;

        }   // end of if-then screen for actual group data

        // Add a trainling margin (to match the leading 
        // margin) with an "invisible point":
        /*
        p_cf_3dpAfterLastItem
            = new cf_3DInvisPoint( iNumDomainAxisItems + 1.0, 0.0, 0.0 );
        this->addFaceObject( p_cf_3dpAfterLastItem );
          */
          // No longer neccess. w/ Auto-Grid/Tick-Series axis-indexing
          
    }   // end of for-loop over Data Groups

    return;

}   // end of cf_eng_PointSer2D::updateYValsAtNatNs()


void
    cf_eng_PointSer2D
        ::updateYValsAtIntIs()
{
    // We will generate "Virtual 2D Line Points and PolyLines" 
    // for each of the Chart's Data Items with these vars:
    cf_DataGroup        *p_cf_dgAccessGroup;
    cf_XYZ              *p_cf_xyzAccessItem;
                        // We'll point along the various groups/items
    
    cf_3DPoint          cf_3dpNewPoint;
    double              dNewPointDepthZ = 0.0;
                        // and grab Y-vals to add as points to the...

    cf_3DPointSeries    *p_cf_3dpsNewSeries;
                        // ...series which represent the data groups...

    // We will iterate over the available groups and generate
    // 2D PolyLines from the data elements in the parent 
    // cf_ChartModel's data groups:
    int iStartAt = this->p_cf_cmodlUserData->dataGroupsStartAt();
    int iEndAt = this->p_cf_cmodlUserData->dataGroupsEndAt();
            
    // Compute the max number of axis items to iterate over:
    int iNumActiveInGroup = 0;
    int iNumDomainAxisItems = 0;
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );
        iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
        if ( iNumActiveInGroup > iNumDomainAxisItems ) 
                iNumDomainAxisItems = iNumActiveInGroup;
    }           // We now have our iteration limit...

    // Begin a for-loop over the active group range:
    for ( i = iStartAt; i <= iEndAt; i++ )
    {
        // Activate each chart group, and get a pointer to it:
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );

        // Only add a virtual chart object IF there is data to use:
        if ( p_cf_dgAccessGroup->hasData() )
        {
            // Define a new 2D/3D point-series:
            p_cf_3dpsNewSeries 
                = new cf_3DPointSeries( p_cf_dgAccessGroup->dispAttribs() );

            // Now we'll add points to this point-series:
            iNumActiveInGroup = p_cf_dgAccessGroup->numItems();
            
            // for-loop over the current count of items:
            for ( j = 1; j <= iNumActiveInGroup; j++ )
            {
                // Get data values and construct the 2D/3D object:
                p_cf_xyzAccessItem 
                    = p_cf_dgAccessGroup->activateXYZValAt( j );
                    // This XY pair will define a new PolyLine point...
                
                cf_3dpNewPoint.dataXYZ( j - 1, // Index on Integer Is!
                                        p_cf_xyzAccessItem->y(), 
                                        dNewPointDepthZ );

                p_cf_3dpsNewSeries->addPoint( cf_3dpNewPoint );

            }   // end of for-loop over iNumCurrentItems

            // Add the group/series to the engine...
            this->addSpaceObject( p_cf_3dpsNewSeries );

            //... and set up for the NEXT group/series
            dNewPointDepthZ+= 1.0;

        }   // end of if-then screen for actual group data
        
    }   // end of for-loop over Data Groups

    return;

}   // end of cf_eng_PointSer2D::updateYValsAtIntIs()


void
    cf_eng_PointSer2D
        ::updateYValsAtUsrXs()
{
    // We will generate "Virtual 2D Line Points and PolyLines" 
    // for each of the Chart's Data Items with these vars:
    cf_DataGroup        *p_cf_dgAccessGroup;
    cf_XYZ              *p_cf_xyzAccessItem;
                        // We'll point along the various groups/items
    
    cf_3DPoint          cf_3dpNewPoint;
    double              dNewPointDepthZ = 0.0;
                        // and grab Y-vals to add as points to the...

    cf_3DPointSeries    *p_cf_3dpsNewSeries;
                        // ...point-series which represent 
                        // the data groups...

    // We will iterate over the available groups and generate
    // 2D PolyLines from the data elements in the parent 
    // cf_ChartModel's data groups:
    int     iDataGroupsStartAt
            = this->p_cf_cmodlUserData->dataGroupsStartAt();

    int     iDataGroupsEndAt
            = this->p_cf_cmodlUserData->dataGroupsEndAt();
            

    // Begin a for-loop over the active group range:
    for ( i = iDataGroupsStartAt; i <= iDataGroupsEndAt; i++ )
    {
        // Activate each chart group, and get a pointer to it:
        p_cf_dgAccessGroup = this->p_cf_cmodlUserData->activateGroup( i );

        // Only add a virtual chart object IF there is data to use:
        if ( p_cf_dgAccessGroup->hasData() )
        {
            // Define a new 2D/3D point-series:
            p_cf_3dpsNewSeries 
                = new cf_3DPointSeries( p_cf_dgAccessGroup->dispAttribs() );

            // Now we'll add points to this point-series:
            int     iMaxDataItemCount
                    = p_cf_dgAccessGroup->numItems();
            
            // for-loop over the current count of items:
            for ( j = 1; j <= iMaxDataItemCount; j++ )
            {
                // Get data values and construct the 2D/3D object:
                p_cf_xyzAccessItem 
                    = p_cf_dgAccessGroup->activateXYZValAt( j );
                    // This XY pair will define a new PolyLine point...
                
                cf_3dpNewPoint.dataXYZ( p_cf_xyzAccessItem->x(), 
                                        p_cf_xyzAccessItem->y(), 
                                        dNewPointDepthZ );

                p_cf_3dpsNewSeries->addPoint( cf_3dpNewPoint );

            }   // end of for-loop over iNumCurrentItems

            this->addSpaceObject( p_cf_3dpsNewSeries );

            dNewPointDepthZ+= 1.0;

        }   // end of if-then screen for actual group data
        
    }   // end of for-loop over Data Groups

    return;

}   // end of cf_eng_PointSer2D::updateYValsAtUsrXs()

//
// END of class cf_eng_PointSer2D Implementation
//

//////////////////////////////////////
// END of File: CE2_PSER.CPP
//////////////////////////////////////

