////////////////////////////////////////
// Start of File: CO1_TMS.CPP
////////////////////////////////////////

//===================================================================
//
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994, 1995, 1996, 1997 by DPC Technology.
// All rights reserved. 
//
// See files "cf_copyr.h" and "cf_licen.h" for details of 
// copyright and license specifications, and contact information.
//
//===================================================================

//
// Include this source file's header file:
//
#include    <cf_api/co1_tms.hpp> 
            // relative to the IDE's (global) include path: ~/cfrz
 
//
//=============================================
// Implement the cf_3DTextMarkerSeries object:
//=============================================
//
cf_3DTextMarkerSeries::
    cf_3DTextMarkerSeries( cf_ChartEngine *p_cf_engINowBelongTo )
        : cf_Virt3DObj()
{
    this->p_cf_engParentEngine = p_cf_engINowBelongTo;

    this->da_p_cf_3dtmMarkerArray.growBy( 25 );
    this->iNumMarkAllocs = 0;
    this->bIsActive = TRUE;

    // Init the "active marker" pointer to a KNOWN state:
    this->p_cf_3dtmActiveMarker = NULL; // Until set by activateMark( int )

}   // end of c-tor.


cf_3DTextMarkerSeries::
    ~cf_3DTextMarkerSeries()
{
    // Get RID of ANY allocated text/markers we new()ed!
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ )
        delete ( ( cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] );
}   // end of d-tor.



cf_3DTextMarker*
    cf_3DTextMarkerSeries
        ::activateMarker( int iMarkerNumber )
{
    // Screen (and correct to default value) for bogus index values:
    int iGetMarkerNumber = iMarkerNumber;

    if ( ( iGetMarkerNumber < 1 ) 
        || ( iGetMarkerNumber > this->iNumMarkAllocs ) )
            // IGNORE their bogus index request...
            iGetMarkerNumber = 1;
            //...and give them something we actually HAVE!

    // Wait a second... do we HAVE something to give?...
    if ( this->iNumMarkAllocs < 1 ) 
        // WHOOPS!  Then we had BETTER correct this!
        this->addTextMarker();  
            // makes a "default" (tail=head=text=0,0,0) marker.
    
    // OK... we should be good-to-go with either the user's 
    // ORIGINALLY indexed request, or with our "ignore-the-
    // user's-index-request" text-marker at index one:
    this->p_cf_3dtmActiveMarker = ( (cf_3DTextMarker*) 
                this->da_p_cf_3dtmMarkerArray[ iGetMarkerNumber ] );

    return  this->p_cf_3dtmActiveMarker; 
}

//---------------------------------------------------------
// Allow the user to over-ride the default positioning
// of the Head, Tail and Text of the Text Marker, and 
// specify a new position as an X/Y projection away from 
// the projected CENTER of the chart.  Positive OR 
// negative percentages are allowed.
//---------------------------------------------------------
BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositHead(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr,
                                    double  dPctDepthFromCtr )
{ 
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's HEAD point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToHead();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr,
                                            dPctDepthFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}                               

                                        
BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositHead(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr )
{ 
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's HEAD point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToHead();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}                               



BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositTail(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr,
                                    double  dPctDepthFromCtr )
{ 
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's TAIL point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToTail();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr,
                                            dPctDepthFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}

            
BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositTail(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr )
{ 
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's TAIL point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToTail();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}



BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositText(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr,
                                    double  dPctDepthFromCtr )
{
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's TEXT point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToText();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr,
                                            dPctDepthFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}                               


BOOL
    cf_3DTextMarkerSeries
        ::activeMarkerPositText(    double  dPctWidthFromCtr,
                                    double  dPctHeightFromCtr )
{
    BOOL bStatus = FALSE;   // Init to ERROR state to be safe...

    if ( this->p_cf_3dtmActiveMarker != NULL )
    {   // We'll reset the active marker's TEXT point:
        cf_3DPoint *p_cf_3dpTargetPoint 
                    = p_cf_3dtmActiveMarker->pointToText();
        bStatus = this->p_cf_engParentEngine
                            ->positPointAt( p_cf_3dpTargetPoint,
                                            dPctWidthFromCtr,
                                            dPctHeightFromCtr );
    }
    return bStatus; // Which (hopefully) is now TRUE!...
}                               



//----------------------------------------------
// Add a "default" 3D-text-marker (with the
// tail, head, and text points at 0,0,0, and
// return a pointer to this new marker.
//----------------------------------------------
cf_3DTextMarker*    
    cf_3DTextMarkerSeries
        ::addTextMarker()
{
    cf_3DPoint  *p_cf_3dpTail = new cf_3DPoint( 0, 0, 0 );
    cf_3DPoint  *p_cf_3dpHead = new cf_3DPoint( 0, 0, 0 );
    cf_3DPoint  *p_cf_3dpText = new cf_3DPoint( 0, 0, 0 );

    this->addTextMarker( p_cf_3dpTail, p_cf_3dpHead, p_cf_3dpText );
    // which sets this->p_cf_3dtmActiveMarker = p_cf_3dtmNewMarker;

    return this->p_cf_3dtmActiveMarker;
}

//----------------------------------------------
// Add a "user-specified" 3D-text-marker and
// return a pointer to this new marker.
//----------------------------------------------
cf_3DTextMarker*
    cf_3DTextMarkerSeries
        ::addTextMarker(    cf_3DPoint  *p_cf_3dpTailPoint,
                            cf_3DPoint  *p_cf_3dpHeadPoint,
                            cf_3DPoint  *p_cf_3dpTextPoint )
{
    // Create a new text-marker and add its address into 
    // our dynamic array of pointers to text-markers
    this->iNumMarkAllocs++;
    cf_3DTextMarker *p_cf_3dtmNewMarker
                    = new cf_3DTextMarker(  p_cf_3dpTailPoint,
                                            p_cf_3dpHeadPoint,
                                            p_cf_3dpTextPoint );

    cf_AutoScaleFontSpec *p_cf_asfsForMarker = new cf_AutoScaleFontSpec();
    p_cf_asfsForMarker->colorBack( this->p_cf_engParentEngine
                                                    ->colorDispBack() );
    p_cf_3dtmNewMarker->autoScaleFontSpecNew( p_cf_asfsForMarker );
    this->da_p_cf_3dtmMarkerArray[ iNumMarkAllocs ] = p_cf_3dtmNewMarker;
    return ( this->p_cf_3dtmActiveMarker = p_cf_3dtmNewMarker );
    
}   // end of addTextMarker(... 3 points ...)



//----------------------------------------------
// Add a "user-specified" (with tail and head
// points at 0,0,0, and the text-point posit 
// as a percentage projection away from the
// center of the chart-engine's 3D-box.
//----------------------------------------------
cf_3DTextMarker*    
    cf_3DTextMarkerSeries
        ::addTextMarker(    double  dPctWidthFromCtr,
                            double  dPctHeightFromCtr,
                            double  dPctDepthFromCtr )
{
    cf_3DPoint  *p_cf_3dpTail = new cf_3DPoint( 0, 0, 0 );
    cf_3DPoint  *p_cf_3dpHead = new cf_3DPoint( 0, 0, 0 );
    cf_3DPoint  *p_cf_3dpText = new cf_3DPoint( 0, 0, 0 );

    this->addTextMarker( p_cf_3dpTail, p_cf_3dpHead, p_cf_3dpText );
    // which sets this->p_cf_3dtmActiveMarker = p_cf_3dtmNewMarker;

    this->p_cf_engParentEngine
            ->positPointAt(   p_cf_3dpText,
                                dPctWidthFromCtr,
                                    dPctHeightFromCtr,
                                        dPctDepthFromCtr );
    return this->p_cf_3dtmActiveMarker;
}                            



//----------------------------------------------------------
// Use the Given Chart Framework Kernel to apply Geometry/
// Projection Transformations to this cf_3DGridTickSeries:
//----------------------------------------------------------
void    
    cf_3DTextMarkerSeries
        ::applyGeomXform( cf_Kernel  *p_cf_kUseThisKernel )
{
    // Apply the xform to the text marker line's vertices:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                                ->applyGeomXform( p_cf_kUseThisKernel );
                // Note that we do not perform DSORT update ops 
    return;     // as in class cf_3DPointSeries...
}

void
    cf_3DTextMarkerSeries
        ::applyProjXform( cf_Kernel *p_cf_kUseThisKernel ) 
{
    // Apply the xform to the text marker line's vertices:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                                ->applyProjXform( p_cf_kUseThisKernel );
    return;
}


//---------------------------------------------------------
// SCALE the cf_3DTextMarkerSeries's VIRTUAL Coordinates.
//---------------------------------------------------------
void
    cf_3DTextMarkerSeries
        ::applyScaleOp( double dXScaleFactor,
                        double dYScaleFactor, 
                        double dZScaleFactor ) 
{
    // In order to "properly" scale this object, we
    // need to translate it back to it's original coords.
    double  dSaveXTrans = this->dTransX;
    double  dSaveYTrans = this->dTransY;
    double  dSaveZTrans = this->dTransZ;
    
    this->applyTranslateOp( -dSaveXTrans, 
                                -dSaveYTrans, -dSaveZTrans );
    
    // Apply the xform to the text marker line's vertices:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                                            ->applyScaleOp( dXScaleFactor,
                                                            dYScaleFactor, 
                                                            dZScaleFactor );
    // Update this cf_Virt3DObj's "history":
    this->dScaleFacX = this->dScaleFacX * dXScaleFactor;
    this->dScaleFacY = this->dScaleFacY * dYScaleFactor;
    this->dScaleFacZ = this->dScaleFacZ * dZScaleFactor;

    this->dVirtXMax = this->dVirtXMax * dXScaleFactor;
    this->dVirtXMin = this->dVirtXMin * dXScaleFactor;
    this->dVirtYMax = this->dVirtYMax * dYScaleFactor;
    this->dVirtYMin = this->dVirtYMin * dYScaleFactor;
    this->dVirtZMax = this->dVirtZMax * dZScaleFactor;
    this->dVirtZMin = this->dVirtZMin * dZScaleFactor;

    // Now put our re-scaled cf_Virt3DObj "back" where it was:
    this->applyTranslateOp( dSaveXTrans, 
                                    dSaveYTrans, dSaveZTrans );
    return;

}   // end of cf_3DTextMarkerSeries::applyScaleOp(...)



//------------------------------------------------------------
// TRANSLATE the cf_3DTextMarkerSeries's VIRTUAL Coordinates.
//------------------------------------------------------------
void
    cf_3DTextMarkerSeries
        ::applyTranslateOp( double  dXTrans,
                            double  dYTrans,
                            double  dZTrans  ) 
{
    // Apply the xform to the text marker line's vertices:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                                        ->applyTranslateOp( dXTrans,    
                                                            dYTrans, 
                                                            dZTrans );
    // Update our translation counters, so we can always
    // reverse-calc the Initial XYZ coords of the polyline.
    this->dTransX = this->dTransX + dXTrans;
    this->dTransY = this->dTransY + dYTrans;
    this->dTransZ = this->dTransZ + dZTrans;
                                
    this->dVirtXMax = this->dVirtXMax + dXTrans;
    this->dVirtXMin = this->dVirtXMin + dXTrans;
    this->dVirtYMax = this->dVirtYMax + dYTrans;
    this->dVirtYMin = this->dVirtYMin + dYTrans;
    this->dVirtZMax = this->dVirtZMax + dZTrans;
    this->dVirtZMin = this->dVirtZMin + dZTrans;
    return;

}   // end of cf_3DTextMarkerSeries::applyTranslateOp(...)


//----------------------------------------------------------------
// (re)set text back color for any active (auto-scale) font spec:
//----------------------------------------------------------------
void
    cf_3DTextMarkerSeries
        ::colorBack( const bf_Color &cr_bf_clrNewVal )
{
    // Reset any allocated ASFS text-back-colors:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                    ->autoScaleFontSpec()->colorBack( cr_bf_clrNewVal );
    return;
}


//---------------------------------------------------
// DRAW this cf_3DTextMarkerSeries on a bf_Display.  
//---------------------------------------------------
void    
    cf_3DTextMarkerSeries::
        draw2D( bf_Display *p_bf_dspOutputDisplay ) 
            { this->draw3D( p_bf_dspOutputDisplay ); }
void
    cf_3DTextMarkerSeries::
        draw3D( bf_Display *p_bf_dspOutputDisplay )
{
    // Don't incur font calculation overhead unless
    // we're serious about this!...
    if ( this->bIsActive != TRUE ) return;

    // We'll use the cf_3DTextMarker::draw3D() member method 
    // which lets EACH cf_3DTextMarker object push onto the 
    // display its own appropriate font, pen, etc., to draw with:
    for ( int i = 1; i <= this->iNumMarkAllocs; i++ ) 
        ( (cf_3DTextMarker*) this->da_p_cf_3dtmMarkerArray[ i ] )
                                    ->draw3D( p_bf_dspOutputDisplay );

    return;
}


//-------------------------------
// Get/set activation status:
//-------------------------------
BOOL
    cf_3DTextMarkerSeries
        ::isActive()
            { return this->bIsActive; }

BOOL
    cf_3DTextMarkerSeries
        ::isActive( BOOL bNewState )
            { return( this->bIsActive = bNewState ); }


//----------------------------------------------------------
// Allow users to create new text markers placed at a
// specific location within an chart engine's 3D space:
//----------------------------------------------------------
cf_3DTextMarker*
    cf_3DTextMarkerSeries
        ::markerValueAt(    double  dPctWidthFromCtr,
                            double  dPctHeightFromCtr,
                            double  dPctDepthFromCtr,
                            double  dValueForLabel )
{
    // Create a new text-marker and add its address into 
    // our dynamic array of pointers to text-markers
    cf_3DTextMarker *p_cf_3dtmNewMarker
                    = this->addTextMarker(  new cf_3DPoint,
                                            new cf_3DPoint,
                                            new cf_3DPoint );
    // Set the new text marker's attributes:
    this->p_cf_3dtmActiveMarker = p_cf_3dtmNewMarker;
    this->p_cf_3dtmActiveMarker->labelNew( dValueForLabel );
    this->activeMarkerPositText(    dPctWidthFromCtr,
                                    dPctHeightFromCtr,
                                    dPctDepthFromCtr );
    // We be done!
    return  this->p_cf_3dtmActiveMarker;
}

cf_3DTextMarker*
    cf_3DTextMarkerSeries
        ::markerLabelAt(    double  dPctWidthFromCtr,
                            double  dPctHeightFromCtr,
                            double  dPctDepthFromCtr,
                            const char  *a_cStringForLabel )
{
    // Create a new text-marker and add its address into 
    // our dynamic array of pointers to text-markers
    cf_3DTextMarker *p_cf_3dtmNewMarker
                    = this->addTextMarker(  new cf_3DPoint,
                                            new cf_3DPoint,
                                            new cf_3DPoint );
    // Set the new text marker's attributes:
    this->p_cf_3dtmActiveMarker = p_cf_3dtmNewMarker;
    this->p_cf_3dtmActiveMarker->labelNew( a_cStringForLabel );
    this->activeMarkerPositText(    dPctWidthFromCtr,
                                    dPctHeightFromCtr,
                                    dPctDepthFromCtr );
    // We be done!
    return  this->p_cf_3dtmActiveMarker;
}


int
    cf_3DTextMarkerSeries
        ::numMarks()
            { return this->iNumMarkAllocs; }
    
void
    cf_3DTextMarkerSeries
        ::on()
            { this->bIsActive = TRUE; }

void
    cf_3DTextMarkerSeries
        ::off()
            { this->bIsActive = FALSE; }
    
void    
    cf_3DTextMarkerSeries
        ::reset()
            { this->iNumMarkAllocs = 0; }
//
//-----------------------------------------------------
// End of cf_3DTextMarkerSeries object implementation.
//-----------------------------------------------------
//


//
//=============================================================
// Implement the Chart Framework "Title Marker Manager" class.
//============================================================
//
cf_TitleMarkerMgr::
    cf_TitleMarkerMgr( cf_ChartEngine *p_cf_engINowBelongTo )
        : cf_3DTextMarkerSeries( p_cf_engINowBelongTo )
{
    // Create "Display Attributes" to use for any 
    // "to-be-instantiated" markers:
    this->p_cf_datbTitle = new cf_DispAttribs();
    this->p_cf_datbSubTitle = new cf_DispAttribs();

    // Create Auto-Scale Font Specs to use for any 
    // "to-be-instantiated" markers:
    this->p_cf_asfsTitle    = new cf_AutoScaleFontSpec();
    this->p_cf_asfsSubTitle = new cf_AutoScaleFontSpec();

    this->p_cf_asfsTitle->preferFontToProjRectPct( 5.0 );
    this->p_cf_asfsSubTitle->preferFontToProjRectPct( 4.0 );

    this->p_cf_asfsTitle->colorText( CF_BLACK );
    this->p_cf_asfsTitle
            ->colorBack( this->p_cf_engParentEngine
                                        ->colorDispBack() );
    this->p_cf_asfsSubTitle->colorText( CF_DARK_BLUE );
    this->p_cf_asfsSubTitle
            ->colorBack( this->p_cf_engParentEngine
                                        ->colorDispBack() );

    // Init ALL the title text marker pointers to a KNOWN
    // (and testable) state):
    //
    this->p_cf_3dtmTitle        = NULL; // dispAttribs().on()
    this->p_cf_3dtmSubTitle     = NULL; 

    this->p_cf_3dtmXAxisBackFace    = NULL; // dispAttribs().off()
    this->p_cf_3dtmXAxisBottomFace  = NULL;
    this->p_cf_3dtmYAxisLeftFace    = NULL;
    this->p_cf_3dtmYAxisRightFace   = NULL;
    this->p_cf_3dtmZAxisLeftFace    = NULL;
    this->p_cf_3dtmZAxisBottomFace  = NULL;
    this->p_cf_3dtmPolarFloor   = NULL;

}   // end of cf_TitleMarkerMgr::c-tor()
            

cf_TitleMarkerMgr::
    ~cf_TitleMarkerMgr()
{
    // Get rid of the shared Disp Attribs we new-ed:
    if ( this->p_cf_datbTitle != NULL )
                    delete this->p_cf_datbTitle;
    if ( this->p_cf_datbSubTitle != NULL )
                    delete this->p_cf_datbSubTitle;

    // Get rid of the shared ASFS's we new-ed:
    if ( this->p_cf_asfsTitle != NULL ) 
                    delete this->p_cf_asfsTitle;
    if ( this->p_cf_asfsSubTitle != NULL ) 
                    delete this->p_cf_asfsSubTitle;

}   // end of d-tor


//----------------------------------------------------------------
// (re)set text back color for any active (auto-scale) font spec:
//----------------------------------------------------------------
void
    cf_TitleMarkerMgr
        ::colorBack( const bf_Color &cr_bf_clrNewVal )
{
    // Reset any pre-allocated ASFS text-back-colors:
    this->p_cf_asfsTitle->colorBack( cr_bf_clrNewVal );
    this->p_cf_asfsSubTitle->colorBack( cr_bf_clrNewVal );

    // We do NOT need to reset any of our "shared" ASFS text
    // back-colors that have been allocated by someone someone 
    // else, e.g. an axis-grid/tick/series, because they SHOULD
    // take care of their own text back-color updates...
    return;
}

//---------------------------------------------------
// DRAW this cf_3DTextMarkerSeries on a bf_Display.  
//---------------------------------------------------
void    
    cf_TitleMarkerMgr
        ::draw2D( bf_Display *p_bf_dspOutputDisplay ) 
            { this->draw3D( p_bf_dspOutputDisplay ); }
void
    cf_TitleMarkerMgr
        ::draw3D( bf_Display *p_bf_dspOutputDisplay )
{
    // Draw any allocated title markers:
    if ( this->p_cf_3dtmTitle != NULL )
        this->p_cf_3dtmTitle->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmSubTitle != NULL )  
        this->p_cf_3dtmSubTitle->draw3D( p_bf_dspOutputDisplay );
    
    // RE-draw any shared axis title markers:
    if ( this->p_cf_3dtmXAxisBackFace != NULL )
        this->p_cf_3dtmXAxisBackFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmXAxisBottomFace != NULL )
        this->p_cf_3dtmXAxisBottomFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmYAxisLeftFace != NULL )
        this->p_cf_3dtmYAxisLeftFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmYAxisRightFace != NULL )
        this->p_cf_3dtmYAxisRightFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmZAxisLeftFace != NULL )
        this->p_cf_3dtmZAxisLeftFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmZAxisBottomFace != NULL )
        this->p_cf_3dtmZAxisBottomFace->draw3D( p_bf_dspOutputDisplay );
    if ( this->p_cf_3dtmPolarFloor != NULL )
        this->p_cf_3dtmPolarFloor->draw3D( p_bf_dspOutputDisplay );

    return;
}


cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerTitle()
{   // If we don't yet have one...
    if( this->p_cf_3dtmTitle == NULL ) 
    {   // Make One !
        this->p_cf_3dtmTitle = this->markerLabelAt( 0.0, 120.0, 0.0,    
                                                      "Chart Title" );
        // ... give it attributes!
        this->p_cf_3dtmTitle->dispAttribsPointTo( this->p_cf_datbTitle );
        this->p_cf_3dtmTitle->autoScaleFontSpecUses( this->p_cf_asfsTitle );
        this->p_cf_3dtmTitle->textShiftVertUp();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmTitle );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerSubTitle()
{   // If we don't yet have one...
    if( this->p_cf_3dtmSubTitle == NULL ) 
    {   // Make One !
        this->p_cf_3dtmSubTitle = this->markerLabelAt( 0.0, 118.0, 0.0, 
                                              "(optional sub-title)" );
        // ... give it attributes!
        this->p_cf_3dtmSubTitle
                    ->dispAttribsPointTo( this->p_cf_datbSubTitle );
        this->p_cf_3dtmSubTitle
                    ->autoScaleFontSpecUses( this->p_cf_asfsSubTitle );
        this->p_cf_3dtmSubTitle->textShiftVertDown();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmSubTitle );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_X_BackFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmXAxisBackFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmXAxisBackFace 
                    = this->p_cf_engParentEngine
                            ->axis_X_BackFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmXAxisBackFace );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_X_BottomFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmXAxisBottomFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmXAxisBottomFace 
                    = this->p_cf_engParentEngine
                            ->axis_X_BottomFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmXAxisBottomFace );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_Y_LeftFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmYAxisLeftFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmYAxisLeftFace
                    = this->p_cf_engParentEngine
                            ->axis_Y_LeftFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmYAxisLeftFace );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_Y_BackFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmYAxisRightFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmYAxisRightFace
                    = this->p_cf_engParentEngine
                            ->axis_Y_BackFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmYAxisRightFace );
}




cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_Z_LeftFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmZAxisLeftFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmZAxisLeftFace
                    = this->p_cf_engParentEngine
                            ->axis_Z_LeftFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmZAxisLeftFace );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_Z_BottomFace()
{   // If we don't yet have one...
    if( this->p_cf_3dtmZAxisBottomFace == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmZAxisBottomFace
                    = this->p_cf_engParentEngine
                            ->axis_Z_BottomFace()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmZAxisBottomFace );
}

cf_3DTextMarker*
    cf_TitleMarkerMgr
        ::markerAxis_Polar_Floor()
{   // If we don't yet have one...
    if( this->p_cf_3dtmPolarFloor == NULL ) 
    {   // Do NOT make one -- just POINT to the one we want!
        this->p_cf_3dtmPolarFloor
                    = this->p_cf_engParentEngine
                            ->axis_Polar_Floor()->markerLabel();
    }   // OK, so we've now got something to point to!...
    return ( this->p_cf_3dtmActiveMarker = this->p_cf_3dtmPolarFloor );
}


//-----------------------------------------------------
// End of cf_TitleMarkerMgr object implementation.
//-----------------------------------------------------

////////////////////////////////////////
// End of File: CO1_TMS.CPP
////////////////////////////////////////

