//	Zinc Application Framework - Z_ERROR1.CPP
//	COPYRIGHT (C) 1990-1997.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#include <z_error.hpp>
#include <z_stdarg.hpp>
#include <z_app.hpp>
#define ZAF_ERROR_SYSTEM_INFO
#include "lang_def.cpp"
#include "gbl_def.cpp"

// ----- ZafErrorSystem -----------------------------------------------------

ZafErrorSystem::ZafErrorSystem(void) :
	ZafErrorStub()
{
	// Initialize the global variable.
	if (!zafErrorSystem)
		zafErrorSystem = this;

	// Check the language and local information.
	LanguageAllocate();
}

ZafErrorSystem::ZafErrorSystem(const ZafErrorSystem &) :
	ZafErrorStub()
{
}

ZafErrorSystem::~ZafErrorSystem(void)
{
	// Check the language and local information.
	LanguageFree();
}

void ZafErrorSystem::LanguageAllocate(void)
{
	// Check for initialization.
	if (language)
		return;
	ZafI18nData::AddStaticModule(LanguageFree, LanguageAllocate, ZafLanguageData::classID);

	// Try the zafDataManager.
	if (!language && zafDataManager && zafDataPersistence && zafLanguage)
	{
		zafDataPersistence->PushLanguage(zafLanguage->StringID());
		language = DynamicPtrCast(zafDataManager->AllocateData(className, ZafLanguageData::className, ZafLanguageData::classID), ZafLanguageData);
		zafDataPersistence->PopLanguage();
	}

	// Default to code initialization.
	if (!language)
		language = new ZafLanguageData(defaultErrorStrings, true);
}

void ZafErrorSystem::LanguageFree(bool globalRequest)
{
	// Take care of resetting the international settings.
	if (globalRequest && zafDataManager && language)
	{
		language->SubtractNotification(zafDataManager);
		zafDataManager->Subtract(language);
	}
	// Destroy the static language.
	if (globalRequest && language && language->Destroyable() &&
		language->NotifyCount() == 0)
	{
		delete language;
		language = ZAF_NULLP(ZafLanguageData);
	}
}

